@@ -23,14 +23,26 @@
 
 #include <opensync/opensync.h>
 #include <opensync/opensync_internals.h>
-#include <opensync/opensync-merger.h>
+#include <opensync/opensync-xmlformat.h>
 #include <opensync/opensync-format.h>
 
+osync_bool convert_func(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
+{
+        *free_input = TRUE;
+        *output = g_strdup(input);
+        *outpsize = inpsize;
+        return TRUE;
+}
+
 static osync_bool detect_plain_as_xmlformat(const char *objtype, const char *data, int size)
 {
 	osync_assert(objtype);
 
-	if (!g_pattern_match_simple("<?xml version=\"*\"?>*", data))
+	GString *string = g_string_new("<?xml version=\"*\"?>*");
+	g_string_append(string, "<");
+	g_string_append(string, objtype);
+	g_string_append(string, ">*");
+	if (!g_pattern_match_simple(g_string_free(string, FALSE), data))
 		return FALSE;
 
 	OSyncXMLFormat *xmlformat = osync_xmlformat_parse(data, size, NULL);
@@ -48,33 +60,33 @@ static osync_bool detect_plain_as_xmlformat(const char *objtype, const char *dat
 	return TRUE;
 }
 
-static osync_bool detect_plain_as_xmlformat_contact(const char *data, int size)
+static osync_bool detect_plain_as_xmlformat_contact(const char *data, int size, void *userdata)
 {
 	return detect_plain_as_xmlformat("contact", data, size);
 }
 
-static osync_bool detect_plain_as_xmlformat_event(const char *data, int size)
+static osync_bool detect_plain_as_xmlformat_event(const char *data, int size, void *userdata)
 {
 	return detect_plain_as_xmlformat("event", data, size);
 }
 
-static osync_bool detect_plain_as_xmlformat_todo(const char *data, int size)
+static osync_bool detect_plain_as_xmlformat_todo(const char *data, int size, void *userdata)
 {
 	return detect_plain_as_xmlformat("todo", data, size);
 }
 
-static osync_bool detect_plain_as_xmlformat_note(const char *data, int size)
+static osync_bool detect_plain_as_xmlformat_note(const char *data, int size, void *userdata)
 {
 	return detect_plain_as_xmlformat("note", data, size);
 }
 
-static osync_bool from_xml(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+static osync_bool from_xml(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	*free_input = TRUE;
 	return osync_xmlformat_assemble((OSyncXMLFormat *)input, output, outpsize);
 }
 
-static osync_bool to_xml(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+static osync_bool to_xml(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	*free_input = TRUE;
 	OSyncXMLFormat *ret = osync_xmlformat_parse(input, inpsize, error);
@@ -129,6 +141,7 @@ static osync_bool register_converter(OSyncFormatEnv *env, const char *fromname,
 
 	OSyncObjFormat *fromformat = osync_format_env_find_objformat(env, fromname);
 	OSyncObjFormat *toformat = osync_format_env_find_objformat(env, toname);
+	OSyncObjFormat *plain = osync_format_env_find_objformat(env, "plain");
 
 	if (!fromformat || !toformat) {
 		osync_trace(TRACE_ERROR, "Unable to register converter for %s->%s, format not found\n", fromname, toname);
@@ -155,6 +168,16 @@ static osync_bool register_converter(OSyncFormatEnv *env, const char *fromname,
 	osync_converter_unref(conv);
 
 
+	/* plain as xmlformat */
+	conv = osync_converter_new(OSYNC_CONVERTER_DECAP, plain, fromformat, convert_func, &error);
+	if (!conv) {
+		osync_trace(TRACE_ERROR, "Unable to register converter: %s", osync_error_print(&error));
+		osync_error_unref(&error);
+		return FALSE;
+	}
+	osync_format_env_register_converter(env, conv);
+	osync_converter_unref(conv);
+
 	return TRUE;
 }
 
