@@ -24,7 +24,6 @@
 #include "opensync-context.h"
 #include "opensync-plugin.h"
 #include "opensync-ipc.h"
-#include "opensync-serializer.h"
 #include "opensync-format.h"
 #include "opensync-client.h"
 #include "opensync_client_internals.h"
@@ -90,12 +89,33 @@ static void _osync_client_connect_callback(void *data, OSyncError *error)
 	OSyncMessage *message = baton->message;
 	OSyncClient *client = baton->client;
 
-	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(client->plugin_info);
+	char *objtype = NULL;
+	osync_message_read_string(message, &objtype);
+	OSyncObjTypeSink *sink = NULL;
+	if (objtype) {
+		// objtype sink (e.g. event, contact, ...)
+		sink = osync_plugin_info_find_objtype(client->plugin_info, objtype);
+		
+		if (!sink) {
+			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Unable to find sink for %s", objtype);
+			g_free(objtype);
+			goto error;
+		}
+		
+		g_free(objtype);
+	} else {
+		// main sink
+		sink = osync_plugin_info_get_sink(client->plugin_info);
+	}
 	int slowsync = osync_objtype_sink_get_slowsync(sink);
+	osync_trace(TRACE_INTERNAL, "%s: slowsync %i", __func__, slowsync);
 
 	OSyncMessage *reply = NULL;
 	if (!osync_error_is_set(&error)) {
 		reply = osync_message_new_reply(message, &locerror);
+		if (!reply)
+			goto error;
+
 		//Send connect specific reply data
 		osync_message_write_int(reply, slowsync);
 
@@ -283,6 +303,9 @@ static void _osync_client_read_callback(void *data, OSyncError *error)
 	OSyncMessage *reply = NULL;
 	if (!osync_error_is_set(&error)) {
 		reply = osync_message_new_reply(message, &locerror);
+		if (!reply)
+			goto error;
+
 		//Send get_changes specific reply data
 		osync_message_write_string(reply, osync_change_get_uid(baton->change));
 	} else {
@@ -327,6 +350,9 @@ static void _osync_client_commit_change_callback(void *data, OSyncError *error)
 	OSyncMessage *reply = NULL;
 	if (!osync_error_is_set(&error)) {
 		reply = osync_message_new_reply(message, &locerror);
+		if (!reply)
+			goto error;
+
 		//Send get_changes specific reply data
 		osync_message_write_string(reply, osync_change_get_uid(baton->change));
 	} else {
@@ -445,7 +471,8 @@ static osync_bool _osync_client_handle_initialize(OSyncClient *client, OSyncMess
 	char *groupname = NULL;
 	char *configdir = NULL;
 	char *formatdir = NULL;
-	char *config = NULL;
+	int haspluginconfig = 0;
+	OSyncPluginConfig *config = NULL;
 	OSyncQueue *outgoing = NULL;
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, client, message, error);
@@ -456,7 +483,10 @@ static osync_bool _osync_client_handle_initialize(OSyncClient *client, OSyncMess
 	osync_message_read_string(message, &pluginname);
 	osync_message_read_string(message, &groupname);
 	osync_message_read_string(message, &configdir);
-	osync_message_read_string(message, &config);
+	osync_message_read_int(message, &haspluginconfig);
+
+	if (haspluginconfig && !osync_demarshal_pluginconfig(message, &config, error))
+		goto error;
 	
 	osync_trace(TRACE_INTERNAL, "enginepipe %s, formatdir %s, plugindir %s, pluginname %s", enginepipe, formatdir, plugindir, pluginname);
 		
@@ -505,16 +535,54 @@ static osync_bool _osync_client_handle_initialize(OSyncClient *client, OSyncMess
 		goto error;
 	
 	osync_plugin_info_set_configdir(client->plugin_info, configdir);
-	osync_plugin_info_set_config(client->plugin_info, config);
 	osync_plugin_info_set_loop(client->plugin_info, client->context);
 	osync_plugin_info_set_format_env(client->plugin_info, client->format_env);
 	osync_plugin_info_set_groupname(client->plugin_info, groupname);
+
+	if (config)
+		osync_plugin_info_set_config(client->plugin_info, config);
 	
 #ifdef OPENSYNC_UNITTESTS
 	long long int memberid;
 	osync_message_read_long_long_int(message, &memberid); // Introduced (only) for testing/debugging purpose (mock-sync)
 	client->plugin_info->memberid = memberid;
 #endif	
+
+	/* Enable active sinks */
+	OSyncList *r = NULL;
+
+	if (config)
+		r = osync_plugin_config_get_resources(config);
+
+	for (; r; r = r->next) {
+		OSyncPluginResource *res = r->data;
+		OSyncObjTypeSink *sink;
+
+		/* Don't add disabled Resources to PluginInfo objtypes list: #817 */
+		if (!osync_plugin_resource_is_enabled(res))
+			continue;
+
+		const char *objtype = osync_plugin_resource_get_objtype(res); 
+		/* Check for ObjType sink */
+		if (!(sink = osync_plugin_info_find_objtype(client->plugin_info, objtype))) {
+			sink = osync_objtype_sink_new(objtype, error);
+			if (!sink)
+				goto error_finalize;
+
+			osync_plugin_info_add_objtype(client->plugin_info, sink);
+		} else {
+			osync_error_set(error, OSYNC_ERROR_MISCONFIGURATION, "Duplicate sink objtype \"%s\" configured in plugin %s", objtype, pluginname);
+			goto error;
+		}
+
+		const char *preferred_format = osync_plugin_resource_get_preferred_format(res);
+		osync_objtype_sink_set_preferred_format(sink, preferred_format); 
+		OSyncList *o = osync_plugin_resource_get_objformat_sinks(res);
+		for (; o; o = o->next) {
+			OSyncObjFormatSink *format_sink = (OSyncObjFormatSink *) o->data; 
+			osync_objtype_sink_add_objformat_sink(sink, format_sink);
+		}
+	}
 	
 	client->plugin_data = osync_plugin_initialize(client->plugin, client->plugin_info, error);
 	if (!client->plugin_data)
@@ -535,7 +603,9 @@ static osync_bool _osync_client_handle_initialize(OSyncClient *client, OSyncMess
 	g_free(plugindir);
 	g_free(groupname);
 	g_free(formatdir);
-	g_free(config);
+	
+	if (config)
+		osync_plugin_config_unref(config);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
@@ -551,7 +621,10 @@ error:
 	g_free(plugindir);
 	g_free(groupname);
 	g_free(formatdir);
-	g_free(config);
+
+	if (config)
+		osync_plugin_config_unref(config);
+
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
@@ -610,7 +683,9 @@ error:
 static osync_bool _osync_client_handle_discover(OSyncClient *client, OSyncMessage *message, OSyncError **error)
 {
 	OSyncMessage *reply = NULL;
-	int i = 0;
+	unsigned int i = 0;
+	OSyncPluginConfig *config = osync_plugin_info_get_config(client->plugin_info);
+	OSyncList *res = osync_plugin_config_get_resources(config);
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, client, message, error);
 
@@ -626,8 +701,8 @@ static osync_bool _osync_client_handle_discover(OSyncClient *client, OSyncMessag
 	else
 		osync_message_write_int(reply, 0);
 
-	int numobjs = osync_plugin_info_num_objtypes(client->plugin_info);
-	int avail = 0;
+	unsigned int numobjs = osync_plugin_info_num_objtypes(client->plugin_info);
+	unsigned int avail = 0;
 	for (i = 0; i < numobjs; i++) {
 		OSyncObjTypeSink *sink = osync_plugin_info_nth_objtype(client->plugin_info, i);
 		if (osync_objtype_sink_is_available(sink)) {
@@ -635,7 +710,7 @@ static osync_bool _osync_client_handle_discover(OSyncClient *client, OSyncMessag
 		}
 	}
 
-	osync_message_write_int(reply, avail);
+	osync_message_write_uint(reply, avail);
 	
 	for (i = 0; i < numobjs; i++) {
 		OSyncObjTypeSink *sink = osync_plugin_info_nth_objtype(client->plugin_info, i);
@@ -659,6 +734,7 @@ static osync_bool _osync_client_handle_discover(OSyncClient *client, OSyncMessag
 	}else
 		osync_message_write_int(reply, 0);
 	
+	/* Report detected capabilities */
 	OSyncCapabilities *capabilities = osync_plugin_info_get_capabilities(client->plugin_info);
 	if (capabilities) {
 		char* buffer;
@@ -671,6 +747,17 @@ static osync_bool _osync_client_handle_discover(OSyncClient *client, OSyncMessag
 	}else
 		osync_message_write_int(reply, 0);
 
+	/* Report, detected Resources */
+	res = osync_plugin_config_get_resources(config);
+	unsigned int num_res = osync_list_length(res);
+
+	osync_message_write_uint(reply, num_res);
+	for (; res; res = res->next) {
+		OSyncPluginResource *resource = res->data;
+		if (!osync_marshal_pluginresource(reply, resource, error))
+			goto error_free_message;
+	}
+
 	if (!osync_queue_send_message(client->outgoing, NULL, reply, error))
 		goto error_free_message;
 	
@@ -693,9 +780,16 @@ static osync_bool _osync_client_handle_connect(OSyncClient *client, OSyncMessage
 	char *objtype = NULL;
 	int slowsync;
 	OSyncMessage *reply = NULL;
-	
+
+	/* The message content is read and the objtype is written back immediately
+	 * because the connect callback needs a safe way to determine the correct
+	 * sink. The current sink of the plugin info is usually not correct
+	 * because the connect functions work asynchronous and the last sink
+	 * (usually the main sink) is the current sink.
+	 */
 	osync_message_read_string(message, &objtype);
 	osync_message_read_int(message, &slowsync);
+	osync_message_write_string(message, objtype);
 	osync_trace(TRACE_INTERNAL, "Searching sink for %s", objtype);
 	
 	OSyncObjTypeSink *sink = NULL;
@@ -1347,7 +1441,8 @@ static gboolean osyncClientConnectCallback(gpointer data)
 	osync_trace(TRACE_INTERNAL, "About to connect to the incoming queue");
 	
 	/* We now connect to our incoming queue */
-	osync_queue_connect(client->incoming, OSYNC_QUEUE_RECEIVER, NULL);
+	if (!osync_queue_connect(client->incoming, OSYNC_QUEUE_RECEIVER, NULL))
+		return TRUE;
 	
 	return FALSE;
 }
@@ -1473,8 +1568,10 @@ void osync_client_error_shutdown(OSyncClient *client, OSyncError *error)
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, client, error);
 
 	OSyncMessage *message = osync_message_new_error(error, NULL);
-	if (message)
+	if (message) {
 		osync_queue_send_message(client->outgoing, NULL, message, NULL);
+		osync_message_unref(message);
+	}
 
 	osync_client_shutdown(client);
 
