@@ -23,7 +23,6 @@
 
 #include "opensync-data.h"
 #include "opensync-ipc.h"
-#include "opensync-serializer.h"
 #include "opensync-client.h"
 #include "opensync-group.h"
 #include "opensync-plugin.h"
@@ -109,7 +108,7 @@ end:
 
 /*static char *_osync_client_pid_filename(OSyncClientProxy *proxy)
 {
-	return g_strdup_printf("%s/osplugin.pid", proxy->path);
+	return g_strdup_printf("%s%cosplugin.pid", proxy->path, G_DIR_SEPARATOR);
 }*/
 
 /*static osync_bool osync_client_remove_pidfile(OSyncClientProxy *proxy, OSyncError **error)
@@ -257,8 +256,9 @@ static void _osync_client_proxy_init_handler(OSyncMessage *message, void *user_d
 	return;
 	
 error:
+	ctx->init_callback(proxy, ctx->init_callback_data, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -289,8 +289,9 @@ static void _osync_client_proxy_fin_handler(OSyncMessage *message, void *user_da
 	return;
 	
 error:
+	ctx->fin_callback(proxy, ctx->fin_callback_data, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -310,12 +311,12 @@ static void _osync_client_proxy_discover_handler(OSyncMessage *message, void *us
 		
 		osync_message_read_int(message, &proxy->has_main_sink);
 		
-		int num_sinks = 0;
-		osync_message_read_int(message, &num_sinks);
+		unsigned int num_sinks = 0;
+		osync_message_read_uint(message, &num_sinks);
 		
 		osync_trace(TRACE_INTERNAL, "main sink?: %i, num objs?: %i", proxy->has_main_sink, num_sinks);
 		
-		int i = 0;
+		unsigned int i = 0;
 		for (i = 0; i < num_sinks; i++) {
 			OSyncObjTypeSink *sink = NULL;
 			if (!osync_demarshal_objtype_sink(message, &sink, &locerror))
@@ -398,15 +399,39 @@ static void _osync_client_proxy_discover_handler(OSyncMessage *message, void *us
  			if (capabilities) {
 				if (!osync_member_set_capabilities(member, capabilities, &locerror))
 					goto error_free_capabilities; 
- 				if (!osync_member_save(member, &locerror)) /* TODO: Merger we have to save? */
- 					goto error_free_capabilities;
 
 				osync_capabilities_unref(capabilities);
  			}
  		}
+
 		if (version)
 			osync_version_unref(version);
-		
+
+ 
+		/* Store stuff in member configuration */
+		if (member) {
+
+			/* Demarshal discovered resources */
+			unsigned int num_res;
+			OSyncPluginConfig *config = osync_member_get_config(member, &locerror);
+			if (!config)
+				goto error;
+
+			osync_plugin_config_flush_resources(config);
+			osync_message_read_uint(message, &num_res);
+			for (i=0; i < num_res; i++) {
+				OSyncPluginResource *resource;
+				if (!osync_demarshal_pluginresource(message, &resource, &locerror))
+					goto error;
+
+				osync_plugin_config_add_resource(config, resource);
+				osync_plugin_resource_unref(resource);
+			}
+
+ 			if (!osync_member_save(member, &locerror))
+ 				goto error;
+		}
+
 		ctx->discover_callback(proxy, ctx->discover_callback_data, NULL);
 	} else if (osync_message_get_cmd(message) == OSYNC_MESSAGE_ERRORREPLY) {
 		osync_demarshal_error(message, &error);
@@ -429,8 +454,9 @@ error_free_version:
 	if (version)
 		osync_version_unref(version);
 error:
+	ctx->discover_callback(proxy, ctx->discover_callback_data, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -463,8 +489,9 @@ static void _osync_client_proxy_connect_handler(OSyncMessage *message, void *use
 	return;
 	
 error:
+	ctx->connect_callback(proxy, ctx->connect_callback_data, FALSE, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -495,8 +522,9 @@ static void _osync_client_proxy_disconnect_handler(OSyncMessage *message, void *
 	return;
 	
 error:
+	ctx->disconnect_callback(proxy, ctx->disconnect_callback_data, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -527,8 +555,9 @@ static void _osync_client_proxy_read_handler(OSyncMessage *message, void *user_d
 	return;
 	
 error:
+	ctx->read_callback(proxy, ctx->read_callback_data, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -559,8 +588,9 @@ static void _osync_client_proxy_get_changes_handler(OSyncMessage *message, void
 	return;
 	
 error:
+	ctx->get_changes_callback(proxy, ctx->get_changes_callback_data, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -594,8 +624,9 @@ static void _osync_client_proxy_commit_change_handler(OSyncMessage *message, voi
 	return;
 	
 error:
+	ctx->commit_change_callback(proxy, ctx->commit_change_callback_data, NULL, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -626,8 +657,9 @@ static void _osync_client_proxy_committed_all_handler(OSyncMessage *message, voi
 	return;
 	
 error:
+	ctx->committed_all_callback(proxy, ctx->committed_all_callback_data, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -658,8 +690,9 @@ static void _osync_client_proxy_sync_done_handler(OSyncMessage *message, void *u
 	return;
 	
 error:
+	ctx->sync_done_callback(proxy, ctx->sync_done_callback_data, locerror);
 	g_free(ctx);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&locerror));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 	osync_error_unref(&locerror);
 	return;
 }
@@ -694,7 +727,7 @@ static void _osync_client_proxy_message_handler(OSyncMessage *message, void *use
 	return;
 
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(&error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
 	osync_error_unref(&error);
 }
 
@@ -719,7 +752,7 @@ OSyncClientProxy *osync_client_proxy_new(OSyncFormatEnv *formatenv, OSyncMember
 	return proxy;
 	
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return NULL;
 }
 
@@ -885,13 +918,13 @@ osync_bool osync_client_proxy_spawn(OSyncClientProxy *proxy, OSyncStartType type
 		if (!osync_queue_connect(proxy->outgoing, OSYNC_QUEUE_SENDER, error))
 			goto error;
 	} else {
-		char *name = g_strdup_printf("%s/pluginpipe", path);
+		char *name = g_strdup_printf("%s%cpluginpipe", path, G_DIR_SEPARATOR);
 		proxy->outgoing = osync_queue_new(name, error);
 		g_free(name);
 		if (!proxy->outgoing)
 			goto error;
 		
-		name = g_strdup_printf("%s/enginepipe", path);
+		name = g_strdup_printf("%s%cenginepipe", path, G_DIR_SEPARATOR);
 		proxy->incoming = osync_queue_new(name, error);
 		g_free(name);
 		if (!proxy->incoming)
@@ -988,11 +1021,13 @@ error:
 	return FALSE;
 }
 
-osync_bool osync_client_proxy_initialize(OSyncClientProxy *proxy, initialize_cb callback, void *userdata, const char *formatdir, const char *plugindir, const char *plugin, const char *groupname, const char *configdir, const char *config, OSyncError **error)
+osync_bool osync_client_proxy_initialize(OSyncClientProxy *proxy, initialize_cb callback, void *userdata, const char *formatdir, const char *plugindir, const char *plugin, const char *groupname, const char *configdir, OSyncPluginConfig *config, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %s, %s, %s, %s, %p, %p)", __func__, proxy, callback, userdata, formatdir, plugindir, plugin, groupname, configdir, config, error);
 	osync_assert(proxy);
 	
+	int haspluginconfig = config ? TRUE : FALSE;
+
 	callContext *ctx = osync_try_malloc0(sizeof(callContext), error);
 	if (!ctx)
 		goto error;
@@ -1011,7 +1046,28 @@ osync_bool osync_client_proxy_initialize(OSyncClientProxy *proxy, initialize_cb
 	osync_message_write_string(message, plugin);
 	osync_message_write_string(message, groupname);
 	osync_message_write_string(message, configdir);
-	osync_message_write_string(message, config);
+	osync_message_write_int(message, haspluginconfig);
+
+	if (haspluginconfig && !osync_marshal_pluginconfig(message, config, error))
+		goto error;
+
+	if (haspluginconfig) {
+		OSyncList *r = osync_plugin_config_get_resources(config);
+		for (; r; r = r->next) {
+			OSyncPluginResource *res = r->data;
+
+			if (!osync_plugin_resource_is_enabled(res))
+				continue;
+
+			const char *objtype = osync_plugin_resource_get_objtype(res);
+			OSyncObjTypeSink *sink = osync_client_proxy_find_objtype_sink(proxy, objtype);
+			 /* TODO: In discovery phase *sink COULD be NULL. Review if this is correct behavior. */
+			if (sink) {
+				osync_objtype_sink_ref(sink);
+				proxy->objtypes = g_list_append(proxy->objtypes, sink);
+			}
+		}
+	}
 
 #ifdef OPENSYNC_UNITTESTS
 	// Introduced (only) for testing/debugging purpose (mock-sync)
@@ -1260,10 +1316,10 @@ osync_bool osync_client_proxy_disconnect(OSyncClientProxy *proxy, disconnect_cb
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error_free_context:
-	g_free(ctx);
 error_free_message:
 	osync_message_unref(message);
+error_free_context:
+	g_free(ctx);
 error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
@@ -1304,10 +1360,10 @@ osync_bool osync_client_proxy_read(OSyncClientProxy *proxy, read_cb callback, vo
 	osync_trace(TRACE_EXIT, "%s", __func__);
     	return TRUE;
 
-error_free_context:
-	g_free(ctx);
 error_free_message:
 	osync_message_unref(message);
+error_free_context:
+	g_free(ctx);
 error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
@@ -1348,10 +1404,10 @@ osync_bool osync_client_proxy_get_changes(OSyncClientProxy *proxy, get_changes_c
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error_free_context:
-	g_free(ctx);
 error_free_message:
 	osync_message_unref(message);
+error_free_context:
+	g_free(ctx);
 error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
@@ -1394,10 +1450,10 @@ osync_bool osync_client_proxy_commit_change(OSyncClientProxy *proxy, commit_chan
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error_free_context:
-	g_free(ctx);
 error_free_message:
 	osync_message_unref(message);
+error_free_context:
+	g_free(ctx);
 error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
@@ -1438,10 +1494,10 @@ osync_bool osync_client_proxy_committed_all(OSyncClientProxy *proxy, committed_a
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error_free_context:
-	g_free(ctx);
 error_free_message:
 	osync_message_unref(message);
+error_free_context:
+	g_free(ctx);
 error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
@@ -1482,10 +1538,10 @@ osync_bool osync_client_proxy_sync_done(OSyncClientProxy *proxy, sync_done_cb ca
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error_free_context:
-	g_free(ctx);
 error_free_message:
 	osync_message_unref(message);
+error_free_context:
+	g_free(ctx);
 error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
