@@ -60,7 +60,7 @@ osync_bool osync_db_open(OSyncDB *db, const char *dbfile, OSyncError **error)
 	osync_assert(dbfile);
 
 	if (sqlite3_open(dbfile, &(db->sqlite3db)) != SQLITE_OK) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Cannot open database: %s", sqlite3_errmsg(db->sqlite3db));
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Cannot open database \"%s\": %s", dbfile, sqlite3_errmsg(db->sqlite3db));
 		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, sqlite3_errmsg(db->sqlite3db));
 		return FALSE;
 	}
@@ -154,22 +154,22 @@ osync_bool osync_db_query(OSyncDB *db, const char *query, OSyncError **error)
 }
 
 /**
- * @brief Exectues a SQL query and fill all requested data in a GList. 
+ * @brief Exectues a SQL query and fill all requested data in a OSyncList. 
  *        Check error with osync_error_is_set().
  * 
  * @param db Pointer to database struct
  * @param query SQL database query 
  * @param error Pointer to a error struct 
- * @return Returns pointer to GList which contains the each result as another GList ptr. Freeing is recommend with osync_db_free_list() 
+ * @return Returns pointer to OSyncList which contains the each result as another OSyncList ptr. Freeing is recommend with osync_db_free_list() 
  */
-GList *osync_db_query_table(OSyncDB *db, const char *query, OSyncError **error)
+OSyncList *osync_db_query_table(OSyncDB *db, const char *query, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, db, query, error);
 	
 	osync_assert(db);
 	osync_assert(query);
 
-	GList *table = NULL;
+	OSyncList *table = NULL;
 	int i, j, column_count = 0;
 	int numrows = 0, numcolumns = 0;
 	char **result = NULL;
@@ -187,13 +187,21 @@ GList *osync_db_query_table(OSyncDB *db, const char *query, OSyncError **error)
 	column_count = numcolumns;
 
 	for (j=0; j < numrows; j++) {
-		GList *row = NULL;
+		OSyncList *row = NULL;
 		for (i=0; i < numcolumns; i++)
-			row = g_list_append(row, g_strdup(result[column_count++]));
+			/* speed up - prepend instead of append */
+			row = osync_list_prepend(row, g_strdup(result[column_count++]));
 
-		table = g_list_append(table, row);
+		/* items got prepended, reverse the list again */
+		row = osync_list_reverse(row);
+
+		/* speed up - prepend instead of append. */
+		table = osync_list_prepend(table, row);
 	}
 
+	/* items got prepended, reverse the list again */
+	table = osync_list_reverse(table);
+
 	sqlite3_free_table(result);
 
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, table);
@@ -203,18 +211,18 @@ GList *osync_db_query_table(OSyncDB *db, const char *query, OSyncError **error)
 /**
  * @brief Frees the full result of osync_db_query_table().
  *
- * @param list Result GList pointer of osync_db_query_table()
+ * @param list Result OSyncList pointer of osync_db_query_table()
  */
-void osync_db_free_list(GList *list) {
+void osync_db_free_list(OSyncList *list) {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, list);
 
-	GList *row;
+	OSyncList *row;
 	for (row = list; row; row = row->next) {
-		g_list_foreach((GList *) row->data, (GFunc) g_free, NULL);
-		g_list_free((GList *) row->data);
+		osync_list_foreach((OSyncList *) row->data, (GFunc) g_free, NULL);
+		osync_list_free((OSyncList *) row->data);
 	}
 
-	g_list_free(list);
+	osync_list_free(list);
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
@@ -326,7 +334,7 @@ error:
  * @param error Pointer to a error struct 
  * @return TRUE on success otherwise FALSE
  */
-osync_bool osync_db_reset(OSyncDB *db, const char *tablename, OSyncError **error)
+osync_bool osync_db_reset_table(OSyncDB *db, const char *tablename, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, db, tablename, error);
 
@@ -364,7 +372,7 @@ osync_bool osync_db_reset_full(OSyncDB *db, OSyncError **error)
 
 	while (sqlite3_step(ppStmt) == SQLITE_ROW) {
 		const char *table = (const char *) sqlite3_column_text(ppStmt, 0);
-		if (!osync_db_reset(db, table, error))
+		if (!osync_db_reset_table(db, table, error))
 			goto error;
 	}
 
@@ -409,7 +417,7 @@ error:
  * @param error Pointer to a error struct 
  * @return If the table exist 1 else 0. On error -1.
  */
-int osync_db_exists(OSyncDB *db, const char *tablename, OSyncError **error)
+int osync_db_table_exists(OSyncDB *db, const char *tablename, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, db, tablename, error);
 
