@@ -30,11 +30,16 @@
 #include "opensync-plugin.h"
 #include "opensync-archive.h"
 #include "opensync-merger.h"
+#include "opensync-xmlformat.h"
 
 #include "opensync_obj_engine.h"
 #include "opensync_engine.h"
 #include "opensync_engine_internals.h"
 
+#ifdef OPENSYNC_UNITTESTS
+#include "xmlformat/opensync-xmlformat_internals.h"
+#endif
+
 static void osync_engine_set_error(OSyncEngine *engine, OSyncError *error)
 {
 	osync_assert(engine);
@@ -119,6 +124,35 @@ static int _osync_engine_get_objengine_position(OSyncEngine *engine, OSyncObjEng
 	return ret;
 }
 
+gboolean foreach_schema(void *key, void *value, void *userdata) {
+	osync_xmlformat_schema_unref((OSyncXMLFormatSchema *)value);
+	return TRUE;
+}
+
+static void _osync_engine_finalize_internal_schemas(OSyncEngine *engine)
+{
+
+	if ( engine->internalSchemas != NULL ) {
+		g_hash_table_foreach_remove(engine->internalSchemas, foreach_schema, NULL);
+	} 
+
+}
+
+static void _osync_engine_set_internal_schema(OSyncEngine *engine, const char *objtype, OSyncError **error) 
+{
+	osync_trace(TRACE_INTERNAL, "Setting internal schema for objtype %s", objtype);
+
+	// init OSyncXMLFormatSchemas
+	OSyncXMLFormat *xmlformat = osync_xmlformat_new(objtype, NULL);
+#ifdef OPENSYNC_UNITTESTS
+	OSyncXMLFormatSchema *schema = osync_xmlformat_schema_get_instance_with_path(xmlformat, engine->schema_dir, error);
+#else
+	OSyncXMLFormatSchema *schema = osync_xmlformat_schema_get_instance(xmlformat, error);
+#endif
+	osync_xmlformat_unref(xmlformat);
+	g_hash_table_insert(engine->internalSchemas, g_strdup(objtype), schema);
+}
+
 static void _osync_engine_set_internal_format(OSyncEngine *engine, const char *objtype, OSyncObjFormat *format)
 {
 	osync_trace(TRACE_INTERNAL, "Setting internal format of %s to %p:%s", objtype, format, osync_objformat_get_name(format));
@@ -127,6 +161,25 @@ static void _osync_engine_set_internal_format(OSyncEngine *engine, const char *o
 	g_hash_table_insert(engine->internalFormats, g_strdup(objtype), g_strdup(osync_objformat_get_name(format)));
 }
 
+static OSyncFormatConverterPath *_osync_engine_get_converter_path(OSyncEngine *engine, const char *member_objtype)
+{
+	OSyncFormatConverterPath *converter_path = g_hash_table_lookup(engine->converterPathes, member_objtype);
+	return converter_path;
+}
+
+static void _osync_engine_set_converter_path(OSyncEngine *engine, const char *member_objtype, OSyncFormatConverterPath *converter_path)
+{
+	osync_trace(TRACE_INTERNAL, "Setting converter_path of %s to %p", member_objtype, converter_path);
+	if (!converter_path)
+		return;
+	g_hash_table_insert(engine->converterPathes, g_strdup(member_objtype), converter_path);
+}
+
+static void _osync_engine_converter_path_unref(gpointer data) {
+	OSyncFormatConverterPath * converter_path = data;
+	osync_converter_path_unref(converter_path);
+}
+
 static void _osync_engine_receive_change(OSyncClientProxy *proxy, void *userdata, OSyncChange *change)
 {
 	OSyncEngine *engine = userdata;
@@ -135,15 +188,21 @@ static void _osync_engine_receive_change(OSyncClientProxy *proxy, void *userdata
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, proxy, userdata, change);
 
-	long long int memberid = osync_member_get_id(osync_client_proxy_get_member(proxy));
+	OSyncMember *member = osync_client_proxy_get_member(proxy);
+	long long int memberid = osync_member_get_id(member);
 	const char *uid = osync_change_get_uid(change);		
-	int changetype = osync_change_get_changetype(change);
+	OSyncChangeType changetype = osync_change_get_changetype(change);
 	const char *format = osync_objformat_get_name(osync_change_get_objformat(change));
-	const char *format_config = osync_objformat_get_config(osync_change_get_objformat(change));
 	const char *objtype = osync_change_get_objtype(change);
 
-	osync_trace(TRACE_INTERNAL, "Received change %s, changetype %i, format %s, format conversion config %s, objtype %s from member %lli", uid, changetype, format, format_config, objtype, memberid);
-	
+	OSyncObjTypeSink *objtype_sink = osync_member_find_objtype_sink(member, objtype);
+
+	osync_trace(TRACE_INTERNAL, "Received change %s, changetype %i, format %s, objtype %s from member %lli", uid, changetype, format, objtype, memberid);
+	GString *member_objtype_str = g_string_new(g_strdup_printf("%lli", memberid));
+	g_string_append(member_objtype_str,"_");
+	g_string_append(member_objtype_str, objtype);
+	const char *member_objtype = g_string_free(member_objtype_str, FALSE);
+
 	OSyncData *data = osync_change_get_data(change);
 
 	/* Convert the format to the internal format */
@@ -155,20 +214,25 @@ static void _osync_engine_receive_change(OSyncClientProxy *proxy, void *userdata
 	   Do not convert anything if the chagetype is DELETED. */
 	if (internalFormat && osync_group_get_converter_enabled(engine->group) && (osync_change_get_changetype(change) != OSYNC_CHANGE_TYPE_DELETED)) {
 		osync_trace(TRACE_INTERNAL, "converting to common format %s", osync_objformat_get_name(internalFormat));
-	
-		OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(engine->formatenv, osync_change_get_data(change), internalFormat, &error);
+
+		OSyncFormatConverterPath *path = _osync_engine_get_converter_path(engine, member_objtype);
+		if(!path) {
+			path = osync_format_env_find_path_with_detectors(engine->formatenv, osync_change_get_data(change), internalFormat, NULL, &error);
+			_osync_engine_set_converter_path(engine, member_objtype, path);
+		}
 
 		if (!path)
 			goto error;
 	
-		osync_converter_path_set_config(path, osync_objformat_get_config(osync_change_get_objformat(change)));
+		OSyncObjFormatSink *formatsink = osync_objtype_sink_find_objformat_sink(objtype_sink, internalFormat);
+		if (formatsink) {
+			const char *config = osync_objformat_sink_get_config(formatsink); 
+			osync_converter_path_set_config(path, config);
+		}
 
 		if (!osync_format_env_convert(engine->formatenv, path, data, &error)) {
-			osync_converter_path_unref(path);
 			goto error;
 		}
-		
-		osync_converter_path_unref(path);
 	}
 	
 	/* Merger - Merge lost information to the change (don't merger anything when changetype is DELETED.) */
@@ -181,7 +245,7 @@ static void _osync_engine_receive_change(OSyncClientProxy *proxy, void *userdata
 	{
 		osync_trace(TRACE_INTERNAL, "Merge the XMLFormat.");
 		char *buffer = NULL;
-		unsigned int size = 0;
+		unsigned int xmlformat_size = 0, size = 0;
 		OSyncXMLFormat *xmlformat = NULL;
 		OSyncXMLFormat *xmlformat_entire = NULL;
 
@@ -203,7 +267,9 @@ static void _osync_engine_receive_change(OSyncClientProxy *proxy, void *userdata
 				if(!xmlformat_entire)
 					goto error;
 					
-				xmlformat = (OSyncXMLFormat *) osync_data_get_data_ptr(osync_change_get_data(change));
+				osync_data_get_data(osync_change_get_data(change), (char **) &xmlformat, &xmlformat_size);
+				osync_assert(xmlformat_size == osync_xmlformat_size());
+
 				osync_merger_merge(merger, xmlformat, xmlformat_entire);
 				osync_xmlformat_unref(xmlformat_entire);
 			}
@@ -361,6 +427,8 @@ OSyncEngine *osync_engine_new(OSyncGroup *group, OSyncError **error)
 		g_thread_init (NULL);
 	
 	engine->internalFormats = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	engine->internalSchemas = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+	engine->converterPathes = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, _osync_engine_converter_path_unref);
 	
 	engine->context = g_main_context_new();
 	engine->thread = osync_thread_new(engine->context, error);
@@ -375,7 +443,7 @@ OSyncEngine *osync_engine_new(OSyncGroup *group, OSyncError **error)
 	if (!osync_group_get_configdir(group)) {
 		osync_trace(TRACE_INTERNAL, "No config dir found. Making stateless sync");
 	} else {
-		char *filename = g_strdup_printf("%s/archive.db", osync_group_get_configdir(group));
+		char *filename = g_strdup_printf("%s%carchive.db", osync_group_get_configdir(group), G_DIR_SEPARATOR);
 		engine->archive = osync_archive_new(filename, error);
 		g_free(filename);
 		if (!engine->archive)
@@ -400,8 +468,8 @@ OSyncEngine *osync_engine_new(OSyncGroup *group, OSyncError **error)
 	g_source_attach(engine->command_source, engine->context);
 	g_main_context_ref(engine->context);
 
-	char *enginesdir = g_strdup_printf("%s/.opensync/engines", g_get_home_dir());
-	engine->engine_path = g_strdup_printf("%s/enginepipe", enginesdir);
+	char *enginesdir = g_strdup_printf("%s%c.opensync%cengines", g_get_home_dir(), G_DIR_SEPARATOR, G_DIR_SEPARATOR);
+	engine->engine_path = g_strdup_printf("%s%cenginepipe", enginesdir, G_DIR_SEPARATOR);
 	
 	if (g_mkdir_with_parents(enginesdir, 0755) < 0) {
 		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't create engines directory: %s", g_strerror(errno));
@@ -450,6 +518,9 @@ void osync_engine_unref(OSyncEngine *engine)
 
 		if (engine->internalFormats)
 			g_hash_table_destroy(engine->internalFormats);
+
+		if (engine->converterPathes)
+			g_hash_table_destroy(engine->converterPathes);
 		
 		if (engine->group)
 			osync_group_unref(engine->group);
@@ -495,6 +566,9 @@ void osync_engine_unref(OSyncEngine *engine)
 		
 		if (engine->error)
 			osync_error_unref(&(engine->error));
+
+		if (engine->internalSchemas)
+			g_hash_table_destroy(engine->internalSchemas);
 		
 		g_free(engine);
 		osync_trace(TRACE_EXIT, "%s", __func__);
@@ -598,7 +672,7 @@ error:
 
 static OSyncClientProxy *_osync_engine_initialize_member(OSyncEngine *engine, OSyncMember *member, OSyncError **error)
 {
-	const char *config = NULL;
+	OSyncPluginConfig *config = NULL;
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, member, error);
 
@@ -688,7 +762,7 @@ static osync_bool _osync_engine_generate_connected_event(OSyncEngine *engine)
 			osync_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
 			osync_error_unref(&locerror);
 		} else if (osync_bitcount(engine->proxy_errors) || osync_bitcount(engine->obj_errors)) {
-			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "At least one objenit hat problems while connecting. Aborting");
+			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "At least one object engine failed while connecting. Aborting");
 			osync_trace(TRACE_ERROR, "%s", osync_error_print(&locerror));
 			osync_engine_set_error(engine, locerror);
 			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_ERROR, locerror);
@@ -805,9 +879,12 @@ static osync_bool _osync_engine_generate_disconnected_event(OSyncEngine *engine)
 
 static void _osync_engine_connect_callback(OSyncClientProxy *proxy, void *userdata, osync_bool slowsync, OSyncError *error)
 {
-	OSyncEngine *engine = userdata;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %i, %p)", __func__, proxy, userdata, slowsync, error);
-	
+
+	GList *o = NULL;
+	OSyncEngine *engine = userdata;
+
 	int position = _osync_engine_get_proxy_position(engine, proxy);
 	
 	if (error) {
@@ -818,6 +895,14 @@ static void _osync_engine_connect_callback(OSyncClientProxy *proxy, void *userda
 		engine->proxy_connects = engine->proxy_connects | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_CONNECTED, NULL, NULL);
 	}
+
+	/* If MainSink request a SlowSync, flag all objengines with SlowSync */
+	if (slowsync) {
+		for (o = engine->object_engines; o; o = o->next) {
+			OSyncObjEngine *objengine = o->data;
+			osync_obj_engine_set_slowsync(objengine, TRUE);
+		}
+	}
 	
 	_osync_engine_generate_connected_event(engine);
 	
@@ -972,7 +1057,6 @@ static void _osync_engine_generate_event(OSyncEngine *engine, OSyncEngineEvent e
 
 static void _osync_engine_event_callback(OSyncObjEngine *objengine, OSyncEngineEvent event, OSyncError *error, void *userdata)
 {
-	OSyncError *locerror = NULL;
 	OSyncEngine *engine = userdata;
 	osync_trace(TRACE_ENTRY, "%s(%p, %i, %p, %p)", __func__, objengine, event, error, userdata);
 
@@ -1058,6 +1142,11 @@ osync_bool osync_engine_initialize(OSyncEngine *engine, OSyncError **error)
 	_osync_engine_set_internal_format(engine, "event", osync_format_env_find_objformat(engine->formatenv, "xmlformat-event"));
 	_osync_engine_set_internal_format(engine, "todo", osync_format_env_find_objformat(engine->formatenv, "xmlformat-todo"));
 	_osync_engine_set_internal_format(engine, "note", osync_format_env_find_objformat(engine->formatenv, "xmlformat-note"));
+	/* init schemas */
+	_osync_engine_set_internal_schema(engine, "contact", error);
+	_osync_engine_set_internal_schema(engine, "event", error);
+	_osync_engine_set_internal_schema(engine, "todo", error);
+	_osync_engine_set_internal_schema(engine, "note", error);
 	
 	osync_trace(TRACE_INTERNAL, "Running the main loop");
 	if (!_osync_engine_start(engine, error))
@@ -1143,6 +1232,9 @@ osync_bool osync_engine_finalize(OSyncEngine *engine, OSyncError **error)
 		osync_plugin_env_free(engine->pluginenv);
 		engine->pluginenv = NULL;
 	}
+	
+	/* free internal schemas */
+	_osync_engine_finalize_internal_schemas(engine);
 
 	if (!engine->error)
 		osync_group_unlock(engine->group);
@@ -1230,6 +1322,20 @@ void osync_engine_command(OSyncEngine *engine, OSyncEngineCommand *command)
 				goto error;
 		
 			break;
+		case OSYNC_ENGINE_COMMAND_ABORT:
+			/* For nwo Command Aborting is just trigger ENGINE_EVENT_ERROR.
+			   Which is basically just calling the disconnect functions and not setting
+			   the synchrnoization as a successful one. */
+			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Synchronization got aborted by user!");
+
+			osync_engine_set_error(engine, locerror);
+			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_ERROR, locerror);
+
+			osync_error_unref(&locerror);
+
+			osync_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
+			break;
+
 	}
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
@@ -1305,6 +1411,7 @@ void osync_engine_event(OSyncEngine *engine, OSyncEngineEvent event)
 			break;
 		case OSYNC_ENGINE_EVENT_ERROR:
 			osync_trace(TRACE_ERROR, "Engine aborting due to an error: %s", osync_error_print(&(engine->error)));
+			/* Fall through! - To emit disconnect commands for clean connection termination, in error condition */
 		case OSYNC_ENGINE_EVENT_SYNC_DONE:
 			/* Lets disconnect */
 			for (o = engine->object_engines; o; o = o->next) {
@@ -1723,3 +1830,88 @@ void osync_engine_set_message_callback(OSyncEngine *engine, void *(* function) (
 	engine->plgmsg_userdata = user_data;
 }
 #endif
+
+/*! @brief Aborts running synchronization
+ * 
+ * This is aborting the current synchronization while flushing the pending
+ * commands in the engine command queue and pushing the abort command on this
+ * queue. The abort command will send the disconnect command to the client/plugins.
+ * This could be also used within a conflict handler function which aborts the
+ * synchronization instead of resolving the conflicts.
+ *
+ * FIXME: Currently aborting of the current synchronization is not yet perfect! It
+ *        will not preempt already running commands. For example the batch_commit
+ *        will not be preempted and the engine will abort after the batch_commit is done.
+ *
+ * TODO: Review XMPM Benq patches for abort hander. Is sigaction really sane way
+ *       to abort? It's very important that the plugins get called with the disconnect
+ *       functions, since plugins/devices rely on clean termination of connections.
+ *
+ * TODO: Introduce plugin abort function for protocol specific abort implementations
+ *       (SyncML?, OBEX-based?, ...?)
+ * 
+ * @param engine A pointer to the engine with a running synchronization which gets aborted. 
+ * @param error A pointer to a error struct
+ * @returns TRUE on success, FALSE otherwise.
+ * 
+ */
+osync_bool osync_engine_abort(OSyncEngine *engine, OSyncError **error)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, engine, error);
+	
+	OSyncEngineCommand *pending_command, *cmd;
+
+	if (engine->state != OSYNC_ENGINE_STATE_INITIALIZED) {
+		osync_error_set(error, OSYNC_ERROR_MISCONFIGURATION, "This engine was not in state initialized: %i", engine->state);
+		goto error;
+	}
+
+	cmd = osync_try_malloc0(sizeof(OSyncEngineCommand), error);
+	if (!cmd)
+		goto error;
+	
+	cmd->cmd = OSYNC_ENGINE_COMMAND_ABORT;
+	
+	/* Lock the engine command queue ... */
+	g_async_queue_lock(engine->command_queue);
+
+	/* ...and flush all pending commands.
+	   To make sure the abort command will be the next and last command. */
+	while ((pending_command = g_async_queue_try_pop_unlocked(engine->command_queue)))
+		g_free(pending_command);
+
+	/* Push the abort command on the empty queue. */
+	g_async_queue_push_unlocked(engine->command_queue, cmd);
+
+	/* Done. Unlock the command queue again. */
+	g_async_queue_unlock(engine->command_queue);
+	
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
+
+error:	
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	return FALSE;
+}
+
+#ifdef OPENSYNC_UNITTESTS
+/** @brief Set the schemadir for schema validation to a custom directory.
+ *  This is actually only inteded for UNITTESTS to run tests without 
+ *  having OpenSync installed.
+ * 
+ * @param engine Pointer to engine
+ * @param schemadir Custom schemadir path
+ * 
+ */
+void osync_engine_set_schemadir(OSyncEngine *engine, const char *schema_dir)
+{
+	osync_assert(engine);
+	osync_assert(schema_dir);
+
+	if (engine->schema_dir)
+		g_free(engine->schema_dir);
+
+	engine->schema_dir = g_strdup(schema_dir); 
+}
+#endif /* OPENSYNC_UNITTESTS */
+
