@@ -28,8 +28,6 @@
 #include "opensync-client.h"
 #include "opensync-data.h"
 #include "opensync-mapping.h"
-#include "opensync-format.h"
-#include "opensync-merger.h"
 #include "opensync-plugin.h"
 
 #include "opensync_obj_engine.h"
@@ -44,7 +42,9 @@
 OSyncMappingEngine *osync_mapping_engine_new(OSyncObjEngine *parent, OSyncMapping *mapping, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, parent, mapping, error);
-	g_assert(mapping);
+
+	osync_assert(parent);
+	osync_assert(mapping);
 	
 	OSyncMappingEngine *engine = osync_try_malloc0(sizeof(OSyncMappingEngine), error);
 	if (!engine)
@@ -551,8 +551,7 @@ osync_bool osync_mapping_engine_solve(OSyncMappingEngine *engine, OSyncChange *c
 	engine->parent->conflicts = g_list_remove(engine->parent->conflicts, engine);
 	
 	if (osync_engine_check_get_changes(engine->parent->parent) && osync_bitcount(engine->parent->sink_errors | engine->parent->sink_get_changes) == g_list_length(engine->parent->sink_engines)) {
-		OSyncError *error = NULL;
-		if (!osync_obj_engine_command(engine->parent, OSYNC_ENGINE_COMMAND_WRITE, &error))
+		if (!osync_obj_engine_command(engine->parent, OSYNC_ENGINE_COMMAND_WRITE, error))
 			goto error;
 	} else
 		osync_trace(TRACE_INTERNAL, "Not triggering write. didnt receive all reads yet");
@@ -587,8 +586,7 @@ osync_bool osync_mapping_engine_ignore(OSyncMappingEngine *engine, OSyncError **
 	engine->parent->conflicts = g_list_remove(engine->parent->conflicts, engine);
 	
 	if (osync_engine_check_get_changes(engine->parent->parent) && osync_bitcount(engine->parent->sink_errors | engine->parent->sink_get_changes) == g_list_length(engine->parent->sink_engines)) {
-		OSyncError *error = NULL;
-		if (!osync_obj_engine_command(engine->parent, OSYNC_ENGINE_COMMAND_WRITE, &error))
+		if (!osync_obj_engine_command(engine->parent, OSYNC_ENGINE_COMMAND_WRITE, error))
 			goto error;
 	} else
 		osync_trace(TRACE_INTERNAL, "Not triggering write. didnt receive all reads yet");
@@ -706,6 +704,7 @@ osync_bool osync_mapping_engine_duplicate(OSyncMappingEngine *existingMapping, O
 			if (!change || osync_change_compare(existingEntry->change, change) == OSYNC_CONV_DATA_SAME){
 				existingChange = existingEntry->change;
 				osync_change_ref(existingChange);
+				osync_assert(osync_change_get_uid(existingChange));
 				break;
 			}
 
@@ -714,6 +713,7 @@ osync_bool osync_mapping_engine_duplicate(OSyncMappingEngine *existingMapping, O
 			elevation++;
 		
 			existingChange = osync_change_clone(existingEntry->change, error);
+			osync_assert(osync_change_get_uid(existingChange));
 		}
 		
 		
@@ -761,7 +761,8 @@ osync_bool osync_mapping_engine_duplicate(OSyncMappingEngine *existingMapping, O
 	osync_status_update_mapping(objengine->parent, existingMapping, OSYNC_MAPPING_EVENT_SOLVED, NULL);
 	
 	if (osync_engine_check_get_changes(objengine->parent) && osync_bitcount(objengine->sink_errors | objengine->sink_get_changes) == g_list_length(objengine->sink_engines)) {
-		osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_WRITE, error);
+		if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_WRITE, error))
+			goto error;
 	} else
 		osync_trace(TRACE_INTERNAL, "Not triggering write. didnt receive all reads yet");
 
