@@ -30,6 +30,7 @@
 #include "opensync-format.h"
 #include "opensync-merger.h"
 #include "opensync-plugin.h"
+#include "opensync-xmlformat.h"
 
 #include "opensync_sink_engine_internals.h"
 #include "opensync_mapping_entry_engine_internals.h"
@@ -99,10 +100,7 @@ static void _osync_obj_engine_connect_callback(OSyncClientProxy *proxy, void *us
 	}
 			
 	if (osync_bitcount(engine->sink_errors | engine->sink_connects) == g_list_length(engine->sink_engines)) {
-		if (osync_bitcount(engine->sink_connects) < 2) {
-			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Less than 2 sink_engines are connected");
-			osync_obj_engine_set_error(engine, locerror);
-		} else if (osync_bitcount(engine->sink_errors)) {
+		if (osync_bitcount(engine->sink_errors)) {
 			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "At least one sink_engine failed while connecting");
 			osync_obj_engine_set_error(engine, locerror);
 		}
@@ -322,6 +320,8 @@ osync_bool osync_obj_engine_receive_change(OSyncObjEngine *objengine, OSyncClien
 {
 	OSyncSinkEngine *sinkengine = NULL;
 	
+	osync_assert(objengine);
+	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, objengine, proxy, change, error);
 	
 	/* Find the sinkengine for the proxy */
@@ -622,9 +622,10 @@ static osync_bool _inject_changelog_entries(OSyncObjEngine *engine, OSyncError *
 
 OSyncObjEngine *osync_obj_engine_new(OSyncEngine *parent, const char *objtype, OSyncFormatEnv *formatenv, OSyncError **error)
 {
+	osync_assert(parent);
+	osync_assert(objtype);
+	
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p, %p)", __func__, parent, objtype, formatenv, error);
-	g_assert(parent);
-	g_assert(objtype);
 	
 	OSyncObjEngine *engine = osync_try_malloc0(sizeof(OSyncObjEngine), error);
 	if (!engine)
@@ -719,11 +720,20 @@ osync_bool osync_obj_engine_initialize(OSyncObjEngine *engine, OSyncError **erro
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, engine, error);
 
 	osync_trace(TRACE_INTERNAL, "Loaded %i mappings", osync_mapping_table_num_mappings(engine->mapping_table));
+
+	const char *objtype = osync_obj_engine_get_objtype(engine);
 	
 	int num = osync_engine_num_proxies(engine->parent);
 	int i = 0;
 	for (i = 0; i < num; i++) {
 		OSyncClientProxy *proxy = osync_engine_nth_proxy(engine->parent, i);
+		OSyncObjTypeSink *sink = osync_client_proxy_find_objtype_sink(proxy, objtype); 
+		if (!sink) {
+			/* "data" sink engine counts also as valid. */
+			sink = osync_client_proxy_find_objtype_sink(proxy, "data"); 
+			if (!sink)
+				continue;
+		}
 		
 		OSyncSinkEngine *sinkengine = osync_sink_engine_new(i, proxy, engine, error);
 		if (!sinkengine)
@@ -765,6 +775,8 @@ void osync_obj_engine_finalize(OSyncObjEngine *engine)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, engine);
 
+	OSyncMappingEngine *mapping_engine;
+
 	engine->slowsync = FALSE;
 	engine->written = FALSE;
 
@@ -782,8 +794,15 @@ void osync_obj_engine_finalize(OSyncObjEngine *engine)
 		engine->sink_engines = g_list_remove(engine->sink_engines, sinkengine);
 	}
 	
+	while (engine->conflicts) {
+		mapping_engine = engine->conflicts->data;
+		/* No need to unref the mapping engine. They get unref while emptying
+		   the mapping_engines list. See next loop. */
+		engine->conflicts = g_list_remove(engine->conflicts, mapping_engine);
+	}
+
 	while (engine->mapping_engines) {
-		OSyncMappingEngine *mapping_engine = engine->mapping_engines->data;
+		mapping_engine = engine->mapping_engines->data;
 		osync_mapping_engine_unref(mapping_engine);
 		
 		engine->mapping_engines = g_list_remove(engine->mapping_engines, mapping_engine);
@@ -813,53 +832,6 @@ osync_bool osync_obj_engine_get_slowsync(OSyncObjEngine *engine)
 	return engine->slowsync;
 }
 
-static OSyncObjFormat **_get_member_formats(OSyncFormatEnv *env, OSyncClientProxy *proxy, const char *objtype, OSyncError **error)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %s, %p)", __func__, env, proxy, objtype, error);
-	OSyncMember *member = osync_client_proxy_get_member(proxy);
-	
-	const OSyncList *formats = osync_member_get_objformats(member, objtype, error);
-	if (!formats) {
-		if (!osync_error_is_set(error))
-			osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to find a valid target format");
-		goto error;
-	}
-
-	osync_trace(TRACE_INTERNAL, "Found %i possible sink formats", osync_list_length(formats));
-						
-	int num = osync_list_length(formats);
-	
-	OSyncObjFormat **formatArray = osync_try_malloc0(sizeof(OSyncObjFormat *) * (num + 1), error);
-	if (!formatArray)
-		goto error;
-	
-	const OSyncList *f = NULL;
-	int i = 0;
-	for (f = formats; f; f = f->next) {
-		const char **format_vertice = f->data;
-		const char *formatstr = format_vertice[0];
-		const char *format_configstr = format_vertice[1];
-		OSyncObjFormat *format = osync_format_env_find_objformat(env, formatstr);
-		osync_objformat_set_config(format, format_configstr);
-		if (!format) {
-			g_free(formatArray);
-			osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to find a valid object format for \"%s\"", formatstr);
-			goto error;
-		}
-		
-		formatArray[i] = format;
-		i++;
-	}
-	formatArray[i] = NULL;
-	
-	osync_trace(TRACE_EXIT, "%s: %p", __func__, formatArray);
-	return formatArray;
-
-error:	
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return NULL;
-}
-
 osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd, OSyncError **error)
 {
 	GList *p = NULL;
@@ -952,7 +924,12 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 				long long int memberid = osync_member_get_id(member);
 
 				OSyncObjTypeSink *objtype_sink = osync_member_find_objtype_sink(member, engine->objtype);
+				/* If sink could not be found use "data" sink if available */
+				if (!objtype_sink)
+					objtype_sink = osync_member_find_objtype_sink(member, "data");
+				/* TODO: Review if objtype_sink = NULL is valid at all. */
 
+				OSyncFormatConverterPath *path = NULL;
 				for (e = sinkengine->entries; e; e = e->next) {
 					OSyncMappingEntryEngine *entry_engine = e->data;
 					osync_assert(entry_engine);
@@ -969,12 +946,14 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 
 						osync_trace(TRACE_INTERNAL, "Save the entire XMLFormat and demerge.");
 						char *buffer = NULL;
-						unsigned int size = 0;
+						unsigned int xmlformat_size = 0, size = 0;
 						OSyncXMLFormat *xmlformat = NULL;
 						const char *objtype = osync_change_get_objtype(entry_engine->change);
 						OSyncMapping *mapping = entry_engine->mapping_engine->mapping;
 						
-						xmlformat = (OSyncXMLFormat *) osync_data_get_data_ptr(osync_change_get_data(entry_engine->change));
+						osync_data_get_data(osync_change_get_data(entry_engine->change), (char **) &xmlformat, &xmlformat_size);
+						osync_assert(xmlformat_size == osync_xmlformat_size());
+
 						if(!osync_xmlformat_assemble(xmlformat, &buffer, &size)) {
 							osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not assamble the xmlformat");
 							goto error;	
@@ -1009,38 +988,44 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 							
 							/* Now we have to convert to one of the formats
 							 * that the client can understand */
-							OSyncObjFormat **formats = _get_member_formats(engine->formatenv, sinkengine->proxy, osync_change_get_objtype(entry_engine->change), error);
-							if (!formats)
-								goto error;
-							
-							OSyncFormatConverterPath *path = osync_format_env_find_path_formats_with_detectors(engine->formatenv, osync_change_get_data(entry_engine->change), formats, error);
-							if (!path) {
-								g_free(formats);
+							OSyncList *format_sinks = osync_objtype_sink_get_objformat_sinks(objtype_sink);
+							if (!format_sinks) {
+								osync_error_set(error, OSYNC_ERROR_GENERIC, "There are no available format sinks.");
 								goto error;
 							}
+						
+							/* We cache the converter path for each sink/member couple */
+							if (!path) {
+								path = osync_format_env_find_path_formats_with_detectors(engine->formatenv, osync_change_get_data(entry_engine->change), format_sinks, osync_objtype_sink_get_preferred_format(objtype_sink), error);
+							}
+							if (!path)
+								goto error;
 
-							int length = osync_converter_path_num_edges(path);
+							unsigned int length = osync_converter_path_num_edges(path);
 							OSyncFormatConverter *converter = osync_converter_path_nth_edge(path, length - 1);
 							if (converter) {
 								OSyncObjFormat *format = osync_converter_get_targetformat(converter);
-								osync_converter_path_set_config( path, osync_objformat_get_config(format));
+								OSyncObjFormatSink *formatsink = osync_objtype_sink_find_objformat_sink(objtype_sink, format);
+								osync_converter_path_set_config(path, osync_objformat_sink_get_config(formatsink));
 							}
 
-							g_free(formats);
-							
 							if (!osync_format_env_convert(engine->formatenv, path, osync_change_get_data(entry_engine->change), error)) {
 								osync_converter_path_unref(path);
 								goto error;
 							}
 							osync_trace(TRACE_INTERNAL, "converted to format %s", osync_objformat_get_name(osync_change_get_objformat(entry_engine->change)));
 							
-							osync_converter_path_unref(path);
 							
 							osync_change_set_objtype(change, objtype);
 							g_free(objtype);
 						}
 						
-						osync_trace(TRACE_INTERNAL, "Writing change %s, changetype %i, format %s , format conversion config %s , objtype %s from member %lli", osync_change_get_uid(change), osync_change_get_changetype(change), osync_objformat_get_name(osync_change_get_objformat(change)), osync_objformat_get_config(osync_change_get_objformat(change)), osync_change_get_objtype(change), osync_member_get_id(osync_client_proxy_get_member(sinkengine->proxy)));
+						osync_trace(TRACE_INTERNAL, "Writing change %s, changetype %i, format %s , objtype %s from member %lli", 
+								osync_change_get_uid(change), 
+								osync_change_get_changetype(change), 
+								osync_objformat_get_name(osync_change_get_objformat(change)), 
+								osync_change_get_objtype(change), 
+								osync_member_get_id(osync_client_proxy_get_member(sinkengine->proxy)));
 	
 						if (!osync_client_proxy_commit_change(sinkengine->proxy, _osync_obj_engine_commit_change_callback, entry_engine, osync_entry_engine_get_change(entry_engine), error))
 							goto error;
@@ -1061,7 +1046,10 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 						}
 					}
 				}
-				
+			
+				if (path)
+					osync_converter_path_unref(path);
+
 				if (!osync_client_proxy_committed_all(sinkengine->proxy, _osync_obj_engine_written_callback, sinkengine, engine->objtype, error))
 					goto error;
 			}
@@ -1101,6 +1089,7 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 			break;
 		case OSYNC_ENGINE_COMMAND_SOLVE:
 		case OSYNC_ENGINE_COMMAND_DISCOVER:
+		case OSYNC_ENGINE_COMMAND_ABORT:
 			break;
 	}
 	
