@@ -45,7 +45,7 @@
 OSyncFormatConverter *osync_converter_new(OSyncConverterType type, OSyncObjFormat *sourceformat, OSyncObjFormat *targetformat, OSyncFormatConvertFunc convert_func, OSyncError **error)
 {
 	OSyncFormatConverter *converter = NULL;
-	osync_trace(TRACE_ENTRY, "%s(%i, %p, %p, %p, %p)", __func__, type, sourceformat, targetformat, convert_func, error);
+	osync_trace(TRACE_ENTRY, "%s(%i, %s %p, %s %p, %p, %p)", __func__, type, __NULLSTR(osync_objformat_get_name(sourceformat)), sourceformat, __NULLSTR(osync_objformat_get_name(targetformat)), targetformat, convert_func, error);
 	
 	converter = osync_try_malloc0(sizeof(OSyncFormatConverter), error);
 	if (!converter) {
@@ -60,6 +60,9 @@ OSyncFormatConverter *osync_converter_new(OSyncConverterType type, OSyncObjForma
 	osync_objformat_ref(targetformat);
 	
 	converter->convert_func = convert_func;
+	converter->initialize_func = NULL;
+	converter->finalize_func = NULL;
+	converter->userdata = NULL;
 	converter->type = type;
 	converter->ref_count = 1;
 	
@@ -67,10 +70,19 @@ OSyncFormatConverter *osync_converter_new(OSyncConverterType type, OSyncObjForma
 	return converter;
 }
 
+/**
+ * @brief Creates a new detector
+ *
+ * @param sourceformat the source format for the converter
+ * @param targetformat the target format for the converter
+ * @param detect_func the detection function
+ * @param error Pointer to an error struct
+ * @returns The pointer to the newly allocated detector or NULL in case of error
+ */
 OSyncFormatConverter *osync_converter_new_detector(OSyncObjFormat *sourceformat, OSyncObjFormat *targetformat, OSyncFormatDetectFunc detect_func, OSyncError **error)
 {
 	OSyncFormatConverter *converter = NULL;
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, sourceformat, targetformat, detect_func, error);
+	osync_trace(TRACE_ENTRY, "%s(%s %p, %s %p, %p, %p)", __func__, __NULLSTR(osync_objformat_get_name(sourceformat)), sourceformat, __NULLSTR(osync_objformat_get_name(targetformat)), targetformat, detect_func, error);
 
 	converter = osync_try_malloc0(sizeof(OSyncFormatConverter), error);
 	if (!converter) {
@@ -85,6 +97,9 @@ OSyncFormatConverter *osync_converter_new_detector(OSyncObjFormat *sourceformat,
 	osync_objformat_ref(targetformat);
 	
 	converter->detect_func = detect_func;
+	converter->initialize_func = NULL;
+	converter->finalize_func = NULL;
+	converter->userdata = NULL;
 	converter->type = OSYNC_CONVERTER_DETECTOR;
 	converter->ref_count = 1;
 	
@@ -159,17 +174,23 @@ OSyncConverterType osync_converter_get_type(OSyncFormatConverter *converter)
 	return converter->type;
 }
 
-OSyncObjFormat *osync_converter_detect(OSyncFormatConverter *converter, OSyncData *data)
+/**
+ * @brief Detects the Object Format of passed OSyncData 
+ * @param detector Pointer to the detector 
+ * @param data Pointer to OSyncData object which should be detected
+ * @returns The detected Object Format or NULL 
+ */
+OSyncObjFormat *osync_converter_detect(OSyncFormatConverter *detector, OSyncData *data)
 {
 	OSyncObjFormat *sourceformat = NULL;
 	char *buffer = NULL;
 	unsigned int size = 0;
 	
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, converter, data);
-	osync_assert(converter);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, detector, data);
+	osync_assert(detector);
 	osync_assert(data);
 	
-	if (converter->type != OSYNC_CONVERTER_DETECTOR) {
+	if (detector->type != OSYNC_CONVERTER_DETECTOR) {
 		osync_trace(TRACE_EXIT, "%s: Not a detector", __func__);
 		return NULL;
 	}
@@ -177,30 +198,38 @@ OSyncObjFormat *osync_converter_detect(OSyncFormatConverter *converter, OSyncDat
 	sourceformat = osync_data_get_objformat(data);
 	
 	/* First, we check if this is a "inverse" detection. So we check if our targetformat
-	 * is the sourceformat of the data. There must always be a converter capable of converting
+	 * is the sourceformat of the data. There must always be a detector capable of converting
 	 * this case */
-	if (osync_objformat_is_equal(converter->target_format, sourceformat)) {
-		osync_trace(TRACE_EXIT, "%s: %p", __func__, converter->source_format);
-		return converter->source_format;
+	if (osync_objformat_is_equal(detector->target_format, sourceformat)) {
+		osync_trace(TRACE_EXIT, "%s: %p", __func__, detector->source_format);
+		return detector->source_format;
 	}
 	
-	if (!osync_objformat_is_equal(converter->source_format, sourceformat)) {
+	if (!osync_objformat_is_equal(detector->source_format, sourceformat)) {
 		osync_trace(TRACE_EXIT, "%s: Format does not match", __func__);
 		return NULL;
 	}
 	
 	osync_data_get_data(data, &buffer, &size);
-	if (!converter->detect_func || converter->detect_func(buffer, size)) {
+	if (!detector->detect_func || detector->detect_func(buffer, size, detector->userdata)) {
 		/* Successfully detected the data */
-		osync_trace(TRACE_EXIT, "%s: %p", __func__, converter->target_format);
-		return converter->target_format;
+		osync_trace(TRACE_EXIT, "%s: %p", __func__, detector->target_format);
+		return detector->target_format;
 	}
 	
 	osync_trace(TRACE_EXIT, "%s: Not detected", __func__);
 	return NULL;
 }
 
-/** Function does not replace the objtype */
+/**
+ * @brief Invokes converter for OSyncData object with passed format converter configuration 
+ *
+ * @param converter Pointer to the converter 
+ * @param data Pointer to OSyncData object which should be detected
+ * @param config Format converter configuration
+ * @param error Pointer to an error struct
+ * @returns TRUE on successful conversion, FALSE on error
+ */
 osync_bool osync_converter_invoke(OSyncFormatConverter *converter, OSyncData *data, const char *config, OSyncError **error)
 {
 	char *input_data = NULL;
@@ -209,7 +238,10 @@ osync_bool osync_converter_invoke(OSyncFormatConverter *converter, OSyncData *da
 	unsigned int output_size = 0;
 	osync_bool free_input = FALSE;
 	
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %s, %p)", __func__, converter, data, config, error);
+	osync_assert(converter);
+	osync_assert(data);
+	
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %s, %p)", __func__, converter, data, __NULLSTR(config), error);
 	osync_trace(TRACE_INTERNAL, "Converter of type %i, from %p(%s) to %p(%s)", converter->type, converter->source_format, osync_objformat_get_name(converter->source_format), converter->target_format, osync_objformat_get_name(converter->target_format));
 	
 	if (converter->type != OSYNC_CONVERTER_DETECTOR) {
@@ -219,7 +251,7 @@ osync_bool osync_converter_invoke(OSyncFormatConverter *converter, OSyncData *da
 			osync_assert(converter->convert_func);
 		
 			/* Invoke the converter */
-			if (!converter->convert_func(input_data, input_size, &output_data, &output_size, &free_input, config, error))
+			if (!converter->convert_func(input_data, input_size, &output_data, &output_size, &free_input, config, converter->userdata, error))
 				goto error;
 			
 			/* Good. We now have some new data. Now we have to see what to do with the old data */
@@ -241,6 +273,12 @@ error:
 	return FALSE;
 }
 
+/**
+ * @brief Checks if the Format Converter fits the Object Format of OSyncData object 
+ * @param converter Pointer to the converter 
+ * @param data Pointer to OSyncData object which should be converted 
+ * @returns TRUE if Object Format matches the Converter source format, FALSE otherwise
+ */
 osync_bool osync_converter_matches(OSyncFormatConverter *converter, OSyncData *data)
 {
 	OSyncObjFormat *format = NULL;
@@ -272,6 +310,7 @@ OSyncFormatConverterPath *osync_converter_path_new(OSyncError **error)
 /*! @brief Increase the reference count on a converter path
  * 
  * @param path Pointer to the converter path
+ * @returns Pointer of converter path object
  * 
  */
 OSyncFormatConverterPath *osync_converter_path_ref(OSyncFormatConverterPath *path)
@@ -314,6 +353,7 @@ void osync_converter_path_add_edge(OSyncFormatConverterPath *path, OSyncFormatCo
 {
 	osync_assert(path);
 	osync_assert(edge);
+
 	path->converters = g_list_append(path->converters, edge);
 	osync_converter_ref(edge);
 }
@@ -322,7 +362,7 @@ void osync_converter_path_add_edge(OSyncFormatConverterPath *path, OSyncFormatCo
  * @param path Pointer to the converter path
  * @returns the number of converters in the specified path
  */
-int osync_converter_path_num_edges(OSyncFormatConverterPath *path)
+unsigned int osync_converter_path_num_edges(OSyncFormatConverterPath *path)
 {
 	osync_assert(path);
 	return g_list_length(path->converters);
@@ -339,18 +379,60 @@ OSyncFormatConverter *osync_converter_path_nth_edge(OSyncFormatConverterPath *pa
 	return g_list_nth_data(path->converters, nth);
 }
 
+/*! @brief Returns configuration for converter path 
+ * @param path Pointer to the converter path
+ * @returns The converter path configuration 
+ */
 const char *osync_converter_path_get_config(OSyncFormatConverterPath *path)
 {
 	osync_assert(path);
 	return path->config;
 }
 
+/*! @brief Sets the configuration for converter path 
+ * @param path Pointer to the converter path
+ * @param config The converter path configuration
+ */
 void osync_converter_path_set_config(OSyncFormatConverterPath *path, const char *config)
 {
 	osync_assert(path);
+	osync_assert(config);
+
 	if (path->config)
 		g_free(path->config);
+
 	path->config = g_strdup(config);
 }
 
+void osync_converter_set_initialize_func(OSyncFormatConverter *converter, OSyncFormatConverterInitializeFunc initialize_func)
+{
+	osync_assert(converter);
+	converter->initialize_func = initialize_func;
+	
+}
+
+void osync_converter_set_finalize_func(OSyncFormatConverter *converter, OSyncFormatConverterFinalizeFunc finalize_func)
+{
+	osync_assert(converter);
+	converter->finalize_func = finalize_func;
+}
+
+void osync_converter_initialize(OSyncFormatConverter *converter, const char *config, OSyncError **error) {
+
+	osync_assert(converter);
+
+	if (converter->initialize_func) {
+		converter->userdata = converter->initialize_func(config, error);
+	}
+}
+
+void osync_converter_finalize(OSyncFormatConverter *converter)
+{
+	osync_assert(converter);
+
+	if (converter->finalize_func) {
+		converter->finalize_func(converter->userdata);
+	}
+}
+
 /*@}*/
