@@ -28,8 +28,11 @@ struct OSyncFormatConverter {
 	OSyncObjFormat *target_format;
 	OSyncFormatConvertFunc convert_func;
 	OSyncFormatDetectFunc detect_func;
+	OSyncFormatConverterInitializeFunc initialize_func;
+	OSyncFormatConverterFinalizeFunc finalize_func;
 	OSyncConverterType type;
 	int ref_count;
+	void *userdata;
 };
 
 struct OSyncFormatConverterPath {
