@@ -21,9 +21,6 @@
 #ifndef _OPENSYNC_FORMAT_ENV_INTERNALS_H_
 #define _OPENSYNC_FORMAT_ENV_INTERNALS_H_
 
-
-
-
 /** A target function for osync_conv_find_path_fn() */
 typedef osync_bool (*OSyncPathTargetFn)(const void *data, OSyncObjFormat *fmt);
 
@@ -51,26 +48,34 @@ struct OSyncFormatEnv {
  */
 /*@{*/
 
-typedef struct conv_tree {
+/*! @brief search tree for format converters
+ */
+typedef struct OSyncFormatConverterTree {
 	/* The converters that weren't reached yet */
 	GList *unused;
 	/* The search queue for the Breadth-first search */
 	GList *search;
-} conv_tree;
+} OSyncFormatConverterTree;
 
-typedef struct vertice {
-	/** The format associated with this vertice */
+typedef struct OSyncFormatConverterPathVertice {
+	/** The format associated with this OSyncFormatConverterPathVertice */
 	OSyncObjFormat *format;
 	OSyncData *data;
 
-	/** The path of converters taken to this vertice. If this vertice is a target, we will
+	/** The path of converters taken to this OSyncFormatConverterPathVertice. If this OSyncFormatConverterPathVertice is a target, we will
 	 * return this list as the result */
 	GList *path;
 
 	unsigned losses;
 	unsigned objtype_changes;
 	unsigned conversions;
+	guint id;
+	guint neighbour_id;
+	osync_bool preferred;
 
-} vertice;
+	int ref_count;
 
+} OSyncFormatConverterPathVertice;
+
+typedef osync_bool (*OSyncTargetLastConverterFn)(const void *data, OSyncFormatConverterTree *tree);
 #endif //_OPENSYNC_FORMAT_ENV_INTERNALS_H_
