@@ -27,6 +27,10 @@
 #include "opensync_group_internals.h"
 #include "opensync-db.h"
 
+#ifdef OPENSYNC_UNITTESTS
+#include "opensync_member_internals.h"
+#endif /* OPENSYNC_UNITTESTS */
+
 #ifndef _WIN32
 #include <sys/file.h>
 
@@ -89,7 +93,7 @@ static long long int _osync_group_create_member_id(OSyncGroup *group)
 		i++;
 		if (filename)
 			g_free(filename);
-		filename = g_strdup_printf("%s/%lli", group->configdir, i);
+		filename = g_strdup_printf("%s%c%lli", group->configdir, G_DIR_SEPARATOR, i);
 	} while (g_file_test(filename, G_FILE_TEST_EXISTS));
 	
 	g_free(filename);
@@ -125,7 +129,7 @@ static osync_bool _osync_group_load_members(OSyncGroup *group, const char *path,
 	}
 
 	while ((de = g_dir_read_name(dir))) {
-		filename = g_strdup_printf ("%s/%s/syncmember.conf", osync_group_get_configdir(group), de);
+		filename = g_strdup_printf ("%s%c%s%csyncmember.conf", osync_group_get_configdir(group), G_DIR_SEPARATOR, de, G_DIR_SEPARATOR);
 		if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
 			g_free(filename);
 			continue;
@@ -135,8 +139,13 @@ static osync_bool _osync_group_load_members(OSyncGroup *group, const char *path,
 		member = osync_member_new(error);
 		if (!member)
 			goto error_close;
+
+#ifdef OPENSYNC_UNITTESTS
+		if (group->schemadir)
+			osync_member_set_schemadir(member, group->schemadir);
+#endif /* OPENSYNC_UNITTESTS */
 		
-		member_path = g_strdup_printf ("%s/%s", osync_group_get_configdir(group), de);
+		member_path = g_strdup_printf ("%s%c%s", osync_group_get_configdir(group), G_DIR_SEPARATOR, de);
 		if (!osync_member_load(member, member_path, error)) {
 			g_free(member_path);
 			goto error_free_member;
@@ -222,6 +231,27 @@ static GList *_osync_group_get_supported_objtypes(OSyncGroup *group)
 	return ret;
 }
 
+#ifdef OPENSYNC_UNITTESTS
+/** @brief Set the schemadir for configuration validation to a custom directory.
+ *  This is actually only inteded for UNITTESTS to run tests without 
+ *  having OpenSync installed.
+ * 
+ * @param group Pointer to group
+ * @param schemadir Custom schemadir path
+ * 
+ */
+void osync_group_set_schemadir(OSyncGroup *group, const char *schemadir)
+{
+	osync_assert(group);
+	osync_assert(schemadir);
+
+	if (group->schemadir)
+		g_free(group->schemadir);
+
+	group->schemadir = g_strdup(schemadir); 
+}
+#endif /* OPENSYNC_UNITTESTS */
+
 /*@}*/
 
 /**
@@ -268,6 +298,7 @@ error:
 /** @brief Increase the reference count of the group
  * 
  * @param group The group
+ * @returns The referenced group pointer
  * 
  */
 OSyncGroup *osync_group_ref(OSyncGroup *group)
@@ -299,6 +330,11 @@ void osync_group_unref(OSyncGroup *group)
 		if (group->configdir)
 			g_free(group->configdir);
 			
+#ifdef OPENSYNC_UNITTESTS
+		if (group->schemadir)
+			g_free(group->schemadir);
+#endif /* OPENSYNC_UNITTESTS */
+
 		g_free(group);
 	}
 }
@@ -340,7 +376,7 @@ OSyncLockState osync_group_lock(OSyncGroup *group)
 		return OSYNC_LOCKED;
 	}
 	
-	lockfile = g_strdup_printf("%s/lock", group->configdir);
+	lockfile = g_strdup_printf("%s%clock", group->configdir, G_DIR_SEPARATOR);
 
 	if (g_file_test(lockfile, G_FILE_TEST_EXISTS)) {
 		osync_trace(TRACE_INTERNAL, "locking group: file exists");
@@ -426,7 +462,7 @@ void osync_group_unlock(OSyncGroup *group)
 	
 	group->lock_fd = 0;
 	
-	lockfile = g_strdup_printf("%s/lock", group->configdir);
+	lockfile = g_strdup_printf("%s%clock", group->configdir, G_DIR_SEPARATOR);
 	
 	g_unlink(lockfile);
 	g_free(lockfile);
@@ -494,11 +530,15 @@ osync_bool osync_group_save(OSyncGroup *group, OSyncError **error)
 		}
 	}
 	
-	filename = g_strdup_printf ("%s/syncgroup.conf", group->configdir);
+	filename = g_strdup_printf ("%s%csyncgroup.conf", group->configdir, G_DIR_SEPARATOR);
 	osync_trace(TRACE_INTERNAL, "Saving group to file %s", filename);
 	
 	doc = xmlNewDoc((xmlChar*)"1.0");
 	doc->children = xmlNewDocNode(doc, NULL, (xmlChar*)"syncgroup", NULL);
+
+	char *version_str = g_strdup_printf("%u.%u", OSYNC_GROUP_MAJOR_VERSION, OSYNC_GROUP_MINOR_VERSION);
+	xmlSetProp(doc->children, (const xmlChar*)"version", (const xmlChar *)version_str);	
+	g_free(version_str);
 	
 	// TODO: reimplement the filter!
 	//The filters
@@ -612,7 +652,7 @@ osync_bool osync_group_reset(OSyncGroup *group, OSyncError **error)
 		OSyncMember *member = m->data;
 
 		/* flush hashtable */
-		path = g_strdup_printf("%s/hashtable.db", osync_member_get_configdir(member));
+		path = g_strdup_printf("%s%chashtable.db", osync_member_get_configdir(member), G_DIR_SEPARATOR);
 		if (!(db = osync_db_new(error)))
 			goto error_and_free;
 
@@ -624,7 +664,7 @@ osync_bool osync_group_reset(OSyncGroup *group, OSyncError **error)
 		g_free(path);
 
 		/* flush anchor db */ 
-		path = g_strdup_printf("%s/anchor.db", osync_member_get_configdir(member));
+		path = g_strdup_printf("%s%canchor.db", osync_member_get_configdir(member), G_DIR_SEPARATOR);
 		if (!(db = osync_db_new(error)))
 			goto error_and_free;
 
@@ -637,7 +677,7 @@ osync_bool osync_group_reset(OSyncGroup *group, OSyncError **error)
 
 	}
 
-	path = g_strdup_printf("%s/archive.db", osync_group_get_configdir(group));
+	path = g_strdup_printf("%s%carchive.db", osync_group_get_configdir(group), G_DIR_SEPARATOR);
 	if (!(db = osync_db_new(error)))
 		goto error_and_free;
 
@@ -676,20 +716,20 @@ osync_bool osync_group_load(OSyncGroup *group, const char *path, OSyncError **er
 	xmlNodePtr cur;
 	//xmlNodePtr filternode;
 	
-	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, group, path, error);
 	osync_assert(group);
 	osync_assert(path);
+	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, group, path, error);
 	
 	if (!g_path_is_absolute(path)) {
 		char *curdir = g_get_current_dir();
-		real_path = g_strdup_printf("%s/%s", curdir, path);
+		real_path = g_strdup_printf("%s%c%s", curdir, G_DIR_SEPARATOR, path);
 		g_free(curdir);
 	} else {
 		real_path = g_strdup(path);
 	}
 	
 	osync_group_set_configdir(group, real_path);
-	filename = g_strdup_printf("%s/syncgroup.conf", real_path);
+	filename = g_strdup_printf("%s%csyncgroup.conf", real_path, G_DIR_SEPARATOR);
 	g_free(real_path);
 	
 	if (!osync_xml_open_file(&doc, &cur, filename, "syncgroup", error)) {
@@ -819,7 +859,7 @@ void osync_group_add_member(OSyncGroup *group, OSyncMember *member)
 	g_assert(group);
 	
 	if (!osync_member_get_configdir(member)) {
-		char *configdir = g_strdup_printf("%s/%lli", group->configdir, _osync_group_create_member_id(group));
+		char *configdir = g_strdup_printf("%s%c%lli", group->configdir, G_DIR_SEPARATOR, _osync_group_create_member_id(group));
 		osync_member_set_configdir(member, configdir);
 		g_free(configdir);
 	}
@@ -1185,4 +1225,60 @@ void osync_group_set_converter_enabled(OSyncGroup *group, osync_bool converter_e
 	group->converter_enabled = converter_enabled;
 }
 
+/*! @brief Check if group configuration is up to date. 
+ * 
+ * @param group The group
+ * @returns TRUE if the group configuration is up to date, FALSE otherwise. 
+ */
+osync_bool osync_group_is_uptodate(OSyncGroup *group)
+{
+	osync_assert(group);
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, group);
+
+	xmlDocPtr doc = NULL;
+	xmlNodePtr cur = NULL;
+	OSyncError *error = NULL;
+	unsigned int version_major;
+	unsigned int version_minor;
+	xmlChar *version_str = NULL;
+	osync_bool uptodate = FALSE;
+
+	char *config = g_strdup_printf("%s%c%s",
+			osync_group_get_configdir(group),
+			G_DIR_SEPARATOR, "syncgroup.conf");
+	
+	/* If syncgroup isn't present, we assume that update is required. */
+	if (!osync_xml_open_file(&doc, &cur, config, "syncgroup", &error)) {
+		osync_trace(TRACE_ERROR, "%s: %s", __func__, osync_error_print(&error));
+		osync_error_unref(&error);
+		goto end;
+	}
+
+	version_str = xmlGetProp(cur->parent, (const xmlChar *)"version");
+
+	/* No version node, means very outdated version. */
+	if (!version_str)
+		goto end;
+
+      	sscanf((const char *) version_str, "%u.%u", &version_major, &version_minor);
+
+	osync_trace(TRACE_INTERNAL, "Version: %s (current %u.%u required %u.%u)",
+			version_str, version_major, version_minor, 
+			OSYNC_GROUP_MAJOR_VERSION, OSYNC_GROUP_MINOR_VERSION ); 
+
+	if (OSYNC_GROUP_MAJOR_VERSION == version_major 
+			&& OSYNC_GROUP_MINOR_VERSION == version_minor)
+		uptodate = TRUE;
+
+	xmlFree(version_str);
+end:
+	g_free(config);
+
+	if (doc)
+		xmlFreeDoc(doc);
+
+	osync_trace(TRACE_EXIT, "%s(%p)", __func__, group);
+	return uptodate;
+}
+
 /*@}*/
