@@ -23,6 +23,7 @@
 
 #include "opensync-plugin.h"
 #include "opensync-group.h"
+#include "opensync-format.h"
 #include "opensync-merger.h"
 #include "opensync_member_internals.h"
 
@@ -112,6 +113,8 @@ static OSyncObjTypeSink *_osync_member_parse_objtype(xmlNode *cur, OSyncError **
 		if (str) {
 			if (!xmlStrcmp(cur->name, (const xmlChar *)"name")) {
 				osync_objtype_sink_set_name(sink, str);
+			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"preferred_format")) {
+				osync_objtype_sink_set_preferred_format(sink, str);
 			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"enabled")) {
 				osync_objtype_sink_set_enabled(sink, atoi(str));
 			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"read")) {
@@ -123,7 +126,14 @@ static OSyncObjTypeSink *_osync_member_parse_objtype(xmlNode *cur, OSyncError **
 			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objformat")) {
 				char *str_name = osync_xml_find_node(cur, "name");
 				char *str_config = osync_xml_find_node(cur, "config");
-				osync_objtype_sink_add_objformat_with_config(sink, str_name, str_config);
+				OSyncObjFormatSink *format_sink = osync_objformat_sink_new(str_name, error);
+				if (!format_sink)
+					return NULL;
+
+				osync_objformat_sink_set_config(format_sink, str_config);
+				osync_objtype_sink_add_objformat_sink(sink, format_sink);
+				osync_objformat_sink_unref(format_sink);
+
 				xmlFree(str_name);
 				xmlFree(str_config);
 			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"timeout")) {
@@ -138,6 +148,27 @@ static OSyncObjTypeSink *_osync_member_parse_objtype(xmlNode *cur, OSyncError **
 	return sink;
 }
 
+#ifdef OPENSYNC_UNITTESTS
+/** @brief Set the schemadir for configuration validation to a custom directory.
+ *  This is actually only inteded for UNITTESTS to run tests without 
+ *  having OpenSync installed.
+ * 
+ * @param member Pointer to member 
+ * @param schemadir Custom schemadir path
+ * 
+ */
+void osync_member_set_schemadir(OSyncMember *member, const char *schemadir)
+{
+	osync_assert(member);
+	osync_assert(schemadir);
+
+	if (member->schemadir)
+		g_free(member->schemadir);
+
+	member->schemadir = g_strdup(schemadir); 
+}
+#endif /* OPENSYNC_UNITTESTS */
+
 /*@}*/
 
 /**
@@ -150,7 +181,7 @@ static OSyncObjTypeSink *_osync_member_parse_objtype(xmlNode *cur, OSyncError **
 
 /** @brief Creates a new member for a group
  * 
- * @param group The parent group. NULL if none
+ * @param error Pointer to a error-struct
  * @returns A newly allocated member
  * 
  */
@@ -206,8 +237,8 @@ void osync_member_unref(OSyncMember *member)
 		if (member->configdir)
 			g_free(member->configdir);
 		
-		if (member->configdata)
-			g_free(member->configdata);
+		if (member->config)
+			osync_plugin_config_unref(member->config);
 			
 		if (osync_member_get_capabilities(member))
 			osync_capabilities_unref(osync_member_get_capabilities(member));
@@ -217,6 +248,12 @@ void osync_member_unref(OSyncMember *member)
 		
 		osync_member_flush_objtypes(member);
 
+#ifdef OPENSYNC_UNITTESTS
+		if (member->schemadir)
+			g_free(member->schemadir);
+#endif /* OPENSYNC_UNITTESTS */
+
+
 		g_free(member);
 	}
 }
@@ -314,62 +351,65 @@ void osync_member_set_configdir(OSyncMember *member, const char *configdir)
  * @returns The member configuration of the plugin default configuration if the member isn't configuered already 
  * 
  */
-const char *osync_member_get_config_or_default(OSyncMember *member, OSyncError **error)
+OSyncPluginConfig *osync_member_get_config_or_default(OSyncMember *member, OSyncError **error)
 {
 	char *filename = NULL;
-	char *data = NULL;
-	const char *config = NULL;
+	OSyncPluginConfig *config = NULL;
+	const char *schemadir = NULL;
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, member, error);
 	g_assert(member);
 
-	if (member->configdata) {
+	if (member->config) {
 		osync_trace(TRACE_EXIT, "%s: Configdata already in memory", __func__);
-		return member->configdata;
+		return member->config;
 	}
-	
+
 	filename = g_strdup_printf("%s"G_DIR_SEPARATOR_S"%s.conf", member->configdir, member->pluginname);
-	osync_trace(TRACE_INTERNAL, "Reading %s", filename);
-	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
-		if (!osync_file_read(filename, &data, NULL, error))
-			goto error_free_filename;
-		g_free(filename);
-		
-		osync_member_set_config(member, data);
-		
-		/* Free the data and return the const pointer */
-		g_free(data);
-		const char *config = osync_member_get_config(member, error);
-		
-		osync_trace(TRACE_EXIT, "%s: Read from member directory", __func__);
-		return config;
-	}
-	g_free(filename);
 
-	filename = g_strdup_printf(OPENSYNC_CONFIGDIR G_DIR_SEPARATOR_S"%s", member->pluginname);
-	osync_trace(TRACE_INTERNAL, "Reading default %s", filename);
-	if (!osync_file_read(filename, &data, NULL, error))
-		goto error_free_filename;
-	g_free(filename);
+	config = osync_plugin_config_new(error);
+	if (!config)
+		goto error;
 	
-	osync_member_set_config(member, data);
-	g_free(data);
-		
-	config = osync_member_get_config(member, error);
-		
-	osync_trace(TRACE_EXIT, "%s: Read default config", __func__);
-	return osync_member_get_config(member, NULL);
+	osync_trace(TRACE_INTERNAL, "Reading %s", filename);
+	if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
+		g_free(filename);
+		filename = g_strdup_printf(OPENSYNC_CONFIGDIR G_DIR_SEPARATOR_S"%s", member->pluginname);
+		osync_trace(TRACE_INTERNAL, "Reading default %s", filename);
+	}
 
-error_free_filename:
+#ifdef OPENSYNC_UNITTESTS
+	if (member->schemadir)
+		schemadir = member->schemadir;
+#endif
+
+	if (!osync_plugin_config_file_load(config, filename, schemadir, error))
+		goto error_free_config;
+		
+	osync_member_set_config(member, config);
+
+	g_free(filename);
+
+	osync_trace(TRACE_EXIT, "%s: Read default config", __func__);
+	return config;
+
+error_free_config:
+	osync_plugin_config_unref(config);
+error:
 	g_free(filename);
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return NULL;
 }
 
+/** @brief Checks if Member has configuration
+ *
+ * @param member The member
+ * @returns TURE if member has configuration, FALSE otherwise
+ */
 osync_bool osync_member_has_config(OSyncMember *member)
 {
 	osync_assert(member);
-	return member->configdata ? TRUE : FALSE;
+	return member->config ? TRUE : FALSE;
 }
 
 /** @brief Gets the configuration data of this member
@@ -388,39 +428,50 @@ osync_bool osync_member_has_config(OSyncMember *member)
  * @returns Member configuration 
  * 
  */
-const char *osync_member_get_config(OSyncMember *member, OSyncError **error)
+OSyncPluginConfig *osync_member_get_config(OSyncMember *member, OSyncError **error)
 {
 	char *filename = NULL;
-	char *data = NULL;
+	const char *schemadir = NULL;
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, member, error);
 	osync_assert(member);
 
-	if (member->configdata) {
+	if (member->config) {
 		osync_trace(TRACE_EXIT, "%s: Configdata already in memory", __func__);
-		return member->configdata;
+		return member->config;
 	}
 	
-	filename = g_strdup_printf("%s/%s.conf", member->configdir, member->pluginname);
+	filename = g_strdup_printf("%s%c%s.conf", member->configdir, G_DIR_SEPARATOR, member->pluginname);
 	osync_trace(TRACE_INTERNAL, "Reading config from: %s", filename);
 	
-	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
-		if (!osync_file_read(filename, &data, NULL, error)) {
-			g_free(filename);
-			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-			return NULL;
-		}
-		g_free(filename);
-		
-		osync_member_set_config(member, data);
-		g_free(data);
-		
-		osync_trace(TRACE_EXIT, "%s: Read set config from member", __func__);
-		return osync_member_get_config(member, NULL);
+	if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Plugin is not configured");
+		goto error;
 	}
+
+	OSyncPluginConfig *config = osync_plugin_config_new(error);
+	if (!config)
+		goto error;
+
+#ifdef OPENSYNC_UNITTESTS
+	if (member->schemadir)
+		schemadir = member->schemadir;
+#endif
+
+	if (!osync_plugin_config_file_load(config, filename, schemadir, error))
+		goto error_free_config;
+
+	g_free(filename);
+
+	osync_member_set_config(member, config);
+
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return config;
+
+error_free_config:
+	osync_plugin_config_unref(config);
+error:
 	g_free(filename);
-	
-	osync_error_set(error, OSYNC_ERROR_GENERIC, "Plugin is not configured");
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return NULL;
 }
@@ -433,15 +484,15 @@ const char *osync_member_get_config(OSyncMember *member, OSyncError **error)
  * @param data The new config data
  * 
  */
-void osync_member_set_config(OSyncMember *member, const char *data)
+void osync_member_set_config(OSyncMember *member, OSyncPluginConfig *config)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, member, data);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, member, config);
 	g_assert(member);
 	
-	if (member->configdata)
-		g_free(member->configdata);
+	if (member->config)
+		osync_plugin_config_unref(member->config);
 
-	member->configdata = g_strdup(data);
+	member->config = osync_plugin_config_ref(config);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
@@ -462,7 +513,7 @@ osync_bool osync_member_load(OSyncMember *member, const char *path, OSyncError *
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, member, path, error);
 
-	filename = g_strdup_printf ("%s/syncmember.conf", path);
+	filename = g_strdup_printf ("%s%csyncmember.conf", path, G_DIR_SEPARATOR);
 	
 	basename = g_path_get_basename(path);
 	member->id = atoi(basename);
@@ -475,29 +526,12 @@ osync_bool osync_member_load(OSyncMember *member, const char *path, OSyncError *
 	}
 	g_free(filename);
 
-	xmlChar *version_str = xmlGetProp(cur->parent, (const xmlChar *)"version");
-	osync_bool uptodate = FALSE;
-	if (version_str && strlen(version_str) > 0) {
-		unsigned int version_major;
-		unsigned int version_minor;
-	      	sscanf(version_str, "%u.%u", &version_major, &version_minor);
-		if (MEMBER_MAJOR_VERSION == version_major && MEMBER_MINOR_VERSION == version_minor)
-			uptodate = TRUE;
-		else
-		    osync_trace(TRACE_INTERNAL, "syncmember version str : %s current %u.%u required %u.%u", version_str, version_major, version_minor, MEMBER_MAJOR_VERSION, MEMBER_MINOR_VERSION ); 
-	}
-
-	xmlFree(version_str);
-
-	if (!uptodate)
-		osync_trace(TRACE_INTERNAL, "syncmember.conf version does not match the one required by this version of opensync !" );
-		
 	while (cur != NULL) {
 		char *str = (char*)xmlNodeGetContent(cur);
 		if (str) {
 			if (!xmlStrcmp(cur->name, (const xmlChar *)"pluginname")) {
 				member->pluginname = g_strdup(str);
-			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objtype") && uptodate ) {
+			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objtype")) {
 				OSyncObjTypeSink *sink = _osync_member_parse_objtype(cur->xmlChildrenNode, error);
 				if (!sink)
 					goto error_free_doc;
@@ -549,7 +583,7 @@ static osync_bool _osync_member_save_sink_add_timeout(xmlNode *cur, const char *
 		return TRUE;
 
 	char *str = g_strdup_printf("%u", timeout);
-	xmlNewChild(cur, NULL, (xmlChar*)timeoutname, str);
+	xmlNewChild(cur, NULL, (xmlChar*)timeoutname, BAD_CAST str);
 	g_free(str);
 
 	return TRUE;
@@ -558,7 +592,6 @@ static osync_bool _osync_member_save_sink_add_timeout(xmlNode *cur, const char *
 static osync_bool _osync_member_save_sink_timeout(xmlNode *cur, OSyncObjTypeSink *sink, OSyncError **error)
 {
 	xmlNode *node = xmlNewChild(cur, NULL, (xmlChar*)"timeout", NULL);
-	char *timeout = NULL;
 
 	_osync_member_save_sink_add_timeout(node, "connect", osync_objtype_sink_get_connect_timeout(sink), error);
 	_osync_member_save_sink_add_timeout(node, "disconnect", osync_objtype_sink_get_disconnect_timeout(sink), error);
@@ -603,9 +636,12 @@ static osync_bool _osync_member_save_sink(xmlDoc *doc, OSyncObjTypeSink *sink, O
 	/* Objtype specific settings */
 	xmlNewChild(node, NULL, (xmlChar*)"name", (xmlChar*)osync_objtype_sink_get_name(sink));
 
-	for (i = 0; i < osync_objtype_sink_num_objformats(sink); i++) {
-		const char *format = osync_objtype_sink_nth_objformat(sink, i);
-		const char *format_config = osync_objtype_sink_nth_objformat_config(sink, i);
+	xmlNewChild(node, NULL, (xmlChar*)"preferred_format", (xmlChar*)osync_objtype_sink_get_preferred_format(sink));
+
+	for (i = 0; i < osync_objtype_sink_num_objformat_sinks(sink); i++) {
+		OSyncObjFormatSink *format_sink = osync_objtype_sink_nth_objformat_sink(sink, i);
+		const char *format = osync_objformat_sink_get_objformat(format_sink);
+		const char *format_config = osync_objformat_sink_get_config(format_sink);
 		xmlNode *objformat_node = xmlNewChild(node, NULL, (xmlChar*)"objformat", NULL);
 		xmlNewChild(objformat_node, NULL, (xmlChar*)"name", (xmlChar*)format);
 		xmlNewChild(objformat_node, NULL, (xmlChar*)"config", (xmlChar*)format_config);
@@ -641,7 +677,7 @@ osync_bool osync_member_save(OSyncMember *member, OSyncError **error)
 	doc = xmlNewDoc((xmlChar*)"1.0");
 	doc->children = xmlNewDocNode(doc, NULL, (xmlChar*)"syncmember", NULL);
 
-	char *version_str = g_strdup_printf("%u.%u", MEMBER_MAJOR_VERSION, MEMBER_MINOR_VERSION);
+	char *version_str = g_strdup_printf("%u.%u", OSYNC_MEMBER_MAJOR_VERSION, OSYNC_MEMBER_MINOR_VERSION);
 	xmlSetProp(doc->children, (const xmlChar*)"version", (const xmlChar *)version_str);	
 	g_free(version_str);
 
@@ -668,24 +704,21 @@ osync_bool osync_member_save(OSyncMember *member, OSyncError **error)
 	/* TODO Validate file before storing! */
 
 	//Saving the syncmember.conf
-	filename = g_strdup_printf ("%s/syncmember.conf", member->configdir);
+	filename = g_strdup_printf ("%s%csyncmember.conf", member->configdir, G_DIR_SEPARATOR);
 	xmlSaveFormatFile(filename, doc, 1);
 	g_free(filename);
 
 	xmlFreeDoc(doc);
 	
 	//Saving the config if it exists
-	if (member->configdata) {
-		filename = g_strdup_printf("%s/%s.conf", member->configdir, member->pluginname);
-		if (!osync_file_write(filename, member->configdata, strlen(member->configdata), 0600, error)) {
+	if (member->config) {
+		filename = g_strdup_printf("%s%c%s.conf", member->configdir, G_DIR_SEPARATOR, member->pluginname);
+		if (!osync_plugin_config_file_save(member->config, filename, error)) {
 			g_free(filename);
 			goto error;
 		}
 		
 		g_free(filename);
-		
-		g_free(member->configdata);
-		member->configdata = NULL;
 	}
 	
 	OSyncCapabilities* capabilities = osync_member_get_capabilities(member);
@@ -756,8 +789,9 @@ OSyncObjTypeSink *osync_member_find_objtype_sink(OSyncMember *member, const char
 	GList *o;
 	for (o = member->objtypes; o; o = o->next) {
 		OSyncObjTypeSink *sink = o->data;
-		if (!strcmp(osync_objtype_sink_get_name(sink), objtype))
+		if (!strcmp(osync_objtype_sink_get_name(sink), objtype)) {
 			return sink;
+		}
 	}
 	return NULL;
 }
@@ -775,7 +809,10 @@ void osync_member_add_objformat(OSyncMember *member, const char *objtype, const
 	if (!sink)
 		return;
 	
-	osync_objtype_sink_add_objformat(sink, format);
+	/* TODO: handle error */
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new(format, NULL);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 }
 
 /** @brief Add a specifc Object Format with a conversion path config to member 
@@ -783,6 +820,7 @@ void osync_member_add_objformat(OSyncMember *member, const char *objtype, const
  * @param member The member pointer
  * @param objtype The searched object type 
  * @param format The name of the Object Format 
+ * @param format_config The Object Format specific configuration
  * 
  */
 void osync_member_add_objformat_with_config(OSyncMember *member, const char *objtype, const char *format, const char *format_config)
@@ -791,7 +829,10 @@ void osync_member_add_objformat_with_config(OSyncMember *member, const char *obj
 	if (!sink)
 		return;
 	
-	osync_objtype_sink_add_objformat_with_config(sink, format, format_config);
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new(format, NULL);
+	osync_objformat_sink_set_config(format_sink, format_config);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 }
 
 /** @brief List of all available object formats for a specifc object type of this member 
@@ -806,14 +847,18 @@ const OSyncList *osync_member_get_objformats(OSyncMember *member, const char *ob
 {
 	OSyncObjTypeSink *sink = osync_member_find_objtype_sink(member, objtype);
 	if (!sink) {
+		/* FIXME: Nonsnse?!
 		sink = osync_member_find_objtype_sink(member, "data");
 		if (!sink) {
+		*/
 			osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to find objtype %s", objtype);
 			return NULL;
+		/*
 		}
+		*/
 	}
 	
-	return osync_objtype_sink_get_objformats(sink);
+	return osync_objtype_sink_get_objformat_sinks(sink);
 }
 
 /** @brief Add an OSyncObjTypeSink object to the member list of supported object types of this member
@@ -893,7 +938,7 @@ osync_bool osync_member_objtype_enabled(OSyncMember *member, const char *objtype
 /** @brief Enables or disables a object type on a member
  * 
  * @param member The member
- * @param objtypestr The name of the object type to change
+ * @param objtype The name of the object type to change
  * @param enabled Set to TRUE if you want to sync the object type, FALSE otherwise
  * 
  * Note: this function should be called only after sink information for the member
@@ -1001,4 +1046,111 @@ OSyncObjTypeSink *osync_member_get_main_sink(OSyncMember *member)
 	return member->main_sink;
 }
 
+/** @brief Checks if the member configuration is up to date. 
+ * 
+ * @param member The member
+ * @returns TRUE if member configuration is up to date. 
+ *
+ */
+osync_bool osync_member_config_is_uptodate(OSyncMember *member)
+{
+	osync_assert(member);
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, member);
+
+	xmlDocPtr doc;
+	xmlNodePtr cur;
+	OSyncError *error = NULL;
+	unsigned int version_major;
+	unsigned int version_minor;
+	xmlChar *version_str = NULL;
+	osync_bool uptodate = FALSE;
+
+	char *config = g_strdup_printf("%s%c%s",
+			osync_member_get_configdir(member),
+			G_DIR_SEPARATOR, "syncmember.conf");
+	
+	/* If syncmember isn't present, we assume that update is required. */
+	if (!osync_xml_open_file(&doc, &cur, config, "syncmember", &error))
+		goto end;
+
+	version_str = xmlGetProp(cur->parent, (const xmlChar *)"version");
+
+	/* No version node, means very outdated version. */
+	if (!version_str)
+		goto end;
+
+      	sscanf((const char *) version_str, "%u.%u", &version_major, &version_minor);
+
+	osync_trace(TRACE_INTERNAL, "Version: %s (current %u.%u required %u.%u)",
+			version_str, version_major, version_minor, 
+			OSYNC_MEMBER_MAJOR_VERSION, OSYNC_MEMBER_MINOR_VERSION );
+
+	if (OSYNC_MEMBER_MAJOR_VERSION == version_major 
+			&& OSYNC_MEMBER_MINOR_VERSION == version_minor)
+		uptodate = TRUE;
+
+	xmlFree(version_str);
+
+end:
+	g_free(config);
+
+	osync_trace(TRACE_EXIT, "%s(%p)", __func__, member);
+	return uptodate;
+}
+
+/** @brief Checks if the plugin configuration is up to date. 
+ * 
+ * @param member The member
+ * @returns TRUE if plugin configuration is up to date. 
+ *
+ */
+osync_bool osync_member_plugin_is_uptodate(OSyncMember *member)
+{
+	osync_assert(member);
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, member);
+
+	xmlDocPtr doc = NULL;
+	xmlNodePtr cur = NULL;
+	OSyncError *error = NULL;
+	unsigned int version_major;
+	unsigned int version_minor;
+	xmlChar *version_str = NULL;
+	osync_bool uptodate = FALSE;
+
+	char *config = g_strdup_printf("%s%c%s",
+			osync_member_get_configdir(member),
+			G_DIR_SEPARATOR, osync_member_get_pluginname(member));
+	
+	/* If syncmember isn't present, we assume that update is required. */
+	if (!osync_xml_open_file(&doc, &cur, config, "plugin", &error))
+		goto end;
+
+	version_str = xmlGetProp(cur->parent, (const xmlChar *)"version");
+
+	/* No version node, means very outdated version. */
+	if (!version_str)
+		goto end;
+
+      	sscanf((const char *) version_str, "%u.%u", &version_major, &version_minor);
+
+	osync_trace(TRACE_INTERNAL, "Version: %s (current %u.%u required %u.%u)",
+			version_str, version_major, version_minor, 
+			OSYNC_PLUGIN_MAJOR_VERSION, OSYNC_PLUGIN_MINOR_VERSION ); 
+
+	if (OSYNC_PLUGIN_MAJOR_VERSION == version_major 
+			&& OSYNC_PLUGIN_MINOR_VERSION == version_minor)
+		uptodate = TRUE;
+
+	xmlFree(version_str);
+
+end:
+	g_free(config);
+
+	if (doc)
+		xmlFreeDoc(doc);
+
+	osync_trace(TRACE_EXIT, "%s(%p)", __func__, member);
+	return uptodate;
+}
+
 /*@}*/
