@@ -27,9 +27,6 @@
 
 #include "opensync_capabilities_internals.h"
 #include "opensync_capability_internals.h"
-#include "opensync_xmlformat_internals.h"
-#include "opensync_xmlfield_internals.h"
-#include "opensync_xmlfieldlist_internals.h"
 #include "opensync_merger_internals.h"
 
 #endif /*OPENSYNCMERGER_INTERNALS_H_*/
