@@ -23,9 +23,14 @@
 #include "opensync.h"
 #include "opensync_internals.h"
 
+#include "opensync-xmlformat.h"
+/* FIXME: osync_merge_merge() requires direct xmlNode access -> dirty! */ 
+#include "../xmlformat/opensync-xmlformat_internals.h"
+
 #include "opensync-merger.h"
 #include "opensync-merger_internals.h"
 
+
 /**
  * @defgroup OSyncMergerPrivateAPI OpenSync Merger Internals
  * @ingroup OSyncPrivate
@@ -46,6 +51,7 @@
 
 /**
  * @brief Creates a new merger object
+ * @param capabilities Pointer to capabilities which should be taken in account by the Merger
  * @param error The error which will hold the info in case of an error
  * @return The pointer to the newly allocated merger object or NULL in case of error
  */
@@ -124,7 +130,7 @@ void osync_merger_merge(OSyncMerger *merger, OSyncXMLFormat *xmlformat, OSyncXML
 	 {		 	
 	 	ret = strcmp(osync_xmlfield_get_name(new_cur), osync_xmlfield_get_name(old_cur));
 	 	if(ret < 0) {
-	 		if(new_cur->next != NULL) {
+	 		if(osync_xmlfield_get_next(new_cur) != NULL) {
 				new_cur = osync_xmlfield_get_next(new_cur);
 				continue;
 	 		}
@@ -204,8 +210,6 @@ void osync_merger_merge(OSyncMerger *merger, OSyncXMLFormat *xmlformat, OSyncXML
 						if(new_node == NULL) {
 							for(tmp=list; tmp != NULL; tmp = g_slist_next(tmp)) {
 								xmlUnlinkNode(tmp->data);
-								xmlDOMWrapAdoptNode(NULL, ((xmlNodePtr)tmp->data)->doc, tmp->data, new_node->doc, new_par_node, 0);
-								//xmlAddChild(new_par_node, tmp->data);
 							}
 						}else{
 							for(tmp=list; tmp != NULL; tmp = g_slist_next(tmp)) {
@@ -245,10 +249,12 @@ void osync_merger_merge(OSyncMerger *merger, OSyncXMLFormat *xmlformat, OSyncXML
 		g_assert_not_reached();
 	 }
 
-	 /* FIXME: Merger is broken! After merging xmlformat is not sorted. Sorting is very expensive! Avoid it! */
-	 xmlformat->sorted = FALSE;
-	 osync_xmlformat_sort(xmlformat);
+	 /* FIXME: Merger is broken! 
+	    After merging xmlformat is not sorted. Sorting is very expensive!
+	    Avoid it! Ticket: #754 */
+	 osync_assert(osync_xmlformat_is_sorted(xmlformat));
 
+#ifndef NDEBUG
 	 /* XXX Debugging only. Fix Merger and remove osync_xmlformat_assemble() */
 	 unsigned int size;
 	 char *buffer = NULL;
@@ -257,6 +263,10 @@ void osync_merger_merge(OSyncMerger *merger, OSyncXMLFormat *xmlformat, OSyncXML
 	 osync_trace(TRACE_EXIT, "%s:\nXML:\n%s ", __func__, buffer);
 
 	 g_free(buffer);
+#else
+	 osync_trace(TRACE_EXIT, "%s", __func__);
+#endif /* NDEBUG */
+
 }
 
 /**
