@@ -75,6 +75,9 @@ void osync_module_free(OSyncModule *module)
  */
 void *osync_module_get_function(OSyncModule *module, const char *name, OSyncError **error)
 {
+	osync_assert(module);
+	osync_assert(name);
+	
 	void *function = NULL;
 	if (!module->module) {
 		osync_error_set(error, OSYNC_ERROR_MISCONFIGURATION, "You need to load the module before getting a function");
@@ -82,7 +85,7 @@ void *osync_module_get_function(OSyncModule *module, const char *name, OSyncErro
 	}
 	
 	if (!g_module_symbol(module->module, name, &function)) {
-		osync_error_set(error, OSYNC_ERROR_PARAMETER, "Unable to locate symbol %s: %s", name, g_module_error());
+		osync_error_set(error, OSYNC_ERROR_PARAMETER, "Unable to locate symbol %s: %s", __NULLSTR(name), g_module_error());
 		return NULL;
 	}
 	
@@ -94,11 +97,11 @@ osync_bool osync_module_get_sync_info(OSyncModule *module, OSyncPluginEnv *env,
  	fkt_b_plugenv_error fct_info = NULL;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, module, env, error);
 	
-	/* Load the get_info symbol */
-	fct_info = (fkt_b_plugenv_error) osync_module_get_function(module, "get_sync_info", NULL);
+	/* Load the get_sync_info symbol */
+	fct_info = (fkt_b_plugenv_error) osync_module_get_function(module, "get_sync_info", error);
 	if (!fct_info) {
-		osync_trace(TRACE_EXIT, "%s: Not get_sync_info function", __func__);
-		return TRUE;
+		osync_trace(TRACE_EXIT_ERROR, "%s: Not get_sync_info function", __func__);
+		return FALSE;
 	}
 	
 	/* Call the get_info function */
@@ -122,7 +125,7 @@ osync_bool osync_module_get_format_info(OSyncModule *module, OSyncFormatEnv *env
 	fct_info = (fkt_b_fmtenv_error) osync_module_get_function(module, "get_format_info", NULL);
 	if (!fct_info) {
 		osync_trace(TRACE_EXIT, "%s: Not get_format_info function", __func__);
-		return TRUE;
+		return FALSE;
 	}
 	
 	/* Call the get_info function */
@@ -146,7 +149,7 @@ osync_bool osync_module_get_conversion_info(OSyncModule *module, OSyncFormatEnv
 	fct_info = (fkt_b_fmtenv_error) osync_module_get_function(module, "get_conversion_info", NULL);
 	if (!fct_info) {
 		osync_trace(TRACE_EXIT, "%s: Not get_format_info function", __func__);
-		return TRUE;
+		return FALSE;
 	}
 	
 	/* Call the get_info function */
@@ -163,6 +166,8 @@ error:
 
 int osync_module_get_version(OSyncModule *module)
 {
+	osync_assert(module);
+	
 	void *function = NULL;
 	int (* fct_version)(void) = NULL;
 	int version = 0;
@@ -189,7 +194,8 @@ osync_bool osync_module_check(OSyncModule *module, OSyncError **error)
 	
 	version = osync_module_get_version(module);
 	if (!version) {
-		osync_trace(TRACE_EXIT, "%s: No version", __func__);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not load plugin \"%s\". Not a opensync library?", __NULLSTR(module->path));
+		osync_trace(TRACE_EXIT_ERROR, "%s: No version", __func__);
 		return FALSE;
 	}
 	
@@ -216,6 +222,7 @@ osync_bool osync_module_check(OSyncModule *module, OSyncError **error)
 osync_bool osync_module_load(OSyncModule *module, const char *path, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, module, path, error);
+	osync_assert(module);
 	osync_assert(!module->module);
 	
 	if (!g_module_supported()) {
@@ -249,6 +256,7 @@ error:
 void osync_module_unload(OSyncModule *module)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, module);
+	osync_assert(module);
 	osync_assert(module->module);
 	
 	if (!osync_module_get_function(module, "dont_free", NULL))
