@@ -21,14 +21,19 @@
 #ifndef OPENSYNCPLUGIN_H_
 #define OPENSYNCPLUGIN_H_
 
-#include <glib/gmacros.h>
-G_BEGIN_DECLS
+OPENSYNC_BEGIN_DECLS
 
 #include "plugin/opensync_plugin.h"
 #include "plugin/opensync_plugin_env.h"
 #include "plugin/opensync_plugin_info.h"
+#include "plugin/opensync_plugin_config.h"
+#include "plugin/opensync_plugin_advancedoptions.h"
+#include "plugin/opensync_plugin_authentication.h"
+#include "plugin/opensync_plugin_connection.h"
+#include "plugin/opensync_plugin_localization.h"
+#include "plugin/opensync_plugin_resource.h"
 #include "plugin/opensync_sink.h"
 
-G_END_DECLS
+OPENSYNC_END_DECLS
 
 #endif //OPENSYNCPLUGIN_H_
