@@ -24,6 +24,7 @@
 #include <sys/types.h>
 #include <time.h>
 #include <fcntl.h>
+#include <stdlib.h>
 
 #ifdef __cplusplus
 
@@ -57,6 +58,21 @@
 
 OPENSYNC_BEGIN_DECLS
 
+
+/**************************************************************
+ * Versions 
+ *************************************************************/
+
+#define OSYNC_GROUP_MAJOR_VERSION 1
+#define OSYNC_GROUP_MINOR_VERSION 0
+
+#define OSYNC_MEMBER_MAJOR_VERSION 1
+#define OSYNC_MEMBER_MINOR_VERSION 0
+
+#define OSYNC_PLUGIN_MAJOR_VERSION 1
+#define OSYNC_PLUGIN_MINOR_VERSION 0
+
+
 /**************************************************************
  * Defines
  *************************************************************/
@@ -68,6 +84,16 @@ OPENSYNC_BEGIN_DECLS
 #define FALSE 0
 #endif
 
+#ifndef NDEBUG
+#define osync_assert(x) if (!(x)) { fprintf(stderr, "%s:%i:E:%s: Assertion \"" #x "\" failed\n", __FILE__, __LINE__, __func__); abort();}
+#define osync_assert_msg(x, msg) if (!(x)) { fprintf(stderr, "%s:%i:E:%s: %s\n", __FILE__, __LINE__, __func__, msg); abort();}
+#define segfault_me char **blablabla = NULL; *blablabla = "test";
+#else
+#define osync_assert(x)
+#define osync_assert_msg(x, msg)
+#define segfault_me
+#endif
+
 /**************************************************************
  * Enums
  *************************************************************/
@@ -124,13 +150,20 @@ typedef struct OSyncFormatConverterPath OSyncFormatConverterPath;
 typedef struct OSyncFormatConverter OSyncFormatConverter;
 typedef struct OSyncFilter OSyncFilter;
 typedef struct OSyncObjFormatSink OSyncObjFormatSink;
-typedef struct OSyncObjTypeSink OSyncObjTypeSink;
 
 /* Plugin component */
 typedef struct OSyncPlugin OSyncPlugin;
 typedef struct OSyncPluginInfo OSyncPluginInfo;
 typedef struct OSyncPluginEnv OSyncPluginEnv;
 typedef struct OSyncModule OSyncModule;
+typedef struct OSyncObjTypeSink OSyncObjTypeSink;
+typedef struct OSyncPluginConfig OSyncPluginConfig;
+typedef struct OSyncPluginAuthentication OSyncPluginAuthentication;
+typedef struct OSyncPluginAdvancedOption OSyncPluginAdvancedOption;
+typedef struct OSyncPluginAdvancedOptionParameter OSyncPluginAdvancedOptionParameter;
+typedef struct OSyncPluginConnection OSyncPluginConnection;
+typedef struct OSyncPluginLocalization OSyncPluginLocalization;
+typedef struct OSyncPluginResource OSyncPluginResource;
 
 /* Engine component */
 typedef struct OSyncEngine OSyncEngine;
@@ -157,6 +190,7 @@ typedef struct OSyncArchive OSyncArchive;
 typedef struct OSyncCapabilities OSyncCapabilities;
 typedef struct OSyncCapability OSyncCapability;
 typedef struct OSyncXMLFormat OSyncXMLFormat;
+typedef struct OSyncXMLFormatSchema OSyncXMLFormatSchema;
 typedef struct OSyncXMLField OSyncXMLField;
 typedef struct OSyncXMLFieldList OSyncXMLFieldList;
 typedef struct OSyncMerger OSyncMerger;
