@@ -31,6 +31,7 @@ GPrivate* current_tabs = NULL;
 GPrivate* thread_id = NULL;
 GPrivate* trace_disabled = NULL;
 GPrivate* trace_sensitive = NULL;
+GPrivate* print_stderr = NULL;
 const char *trace = NULL;
 
 #ifndef _WIN32
@@ -45,13 +46,20 @@ const char *trace = NULL;
  */
 /*@{*/
 
-/** This function will reset the indentation of the trace function. use this
- * after you forked your process. the new process should call this function */
+/*! @brief Reset the indentation of the trace function. 
+ *
+ * Use this function after when process got forked. It's up to the forked
+ * process (child) to reset the indent.
+ * 
+ */
 void osync_trace_reset_indent(void)
 {
 	g_private_set(current_tabs, GINT_TO_POINTER(0));
 }
 
+/*! @brief Disable tracing
+ *
+ */
 void osync_trace_disable(void)
 {
 	if (!trace_disabled)
@@ -60,6 +68,9 @@ void osync_trace_disable(void)
 	g_private_set(trace_disabled, GINT_TO_POINTER(1));
 }
 
+/*! @brief Enable tracing
+ *
+ */
 void osync_trace_enable(void)
 {
 	if (!trace_disabled)
@@ -72,20 +83,32 @@ void osync_trace_enable(void)
 
 }
 
+/*! @brief Initailize tracing 
+ *
+ */
 static void _osync_trace_init()
 {
 	trace = g_getenv("OSYNC_TRACE");
 	if (!trace)
 		return;
 	
-	const char *sensitive = g_getenv("OSYNC_PRIVACY");
+	const char *noprivacy = g_getenv("OSYNC_NOPRIVACY");
 	if (!trace_sensitive)
 		trace_sensitive = g_private_new(NULL);
 
-	if (sensitive)
+	if (noprivacy)
 		g_private_set(trace_sensitive, GINT_TO_POINTER(1));
 	else
 		g_private_set(trace_sensitive, GINT_TO_POINTER(0));
+
+	const char *error = g_getenv("OSYNC_PRINTERROR");
+	if (!print_stderr)
+		print_stderr = g_private_new(NULL);
+
+	if (error)
+		g_private_set(print_stderr, GINT_TO_POINTER(1));
+	else
+		g_private_set(print_stderr, GINT_TO_POINTER(0));
 	
 	if (!g_file_test(trace, G_FILE_TEST_IS_DIR)) {
 		printf("OSYNC_TRACE argument is no directory\n");
@@ -153,7 +176,7 @@ void osync_trace(OSyncTraceType type, const char *message, ...)
 	pid = getpid();
 	endline = "\n";
 #endif
-	logfile = g_strdup_printf("%s/Thread%lu-%i.log", trace, id, pid);
+	logfile = g_strdup_printf("%s%cThread%lu-%i.log", trace, G_DIR_SEPARATOR, id, pid);
 	
 	va_start(arglist, message);
 	
@@ -172,32 +195,39 @@ void osync_trace(OSyncTraceType type, const char *message, ...)
 	g_get_current_time(&curtime);
 	switch (type) {
 		case TRACE_ENTRY:
-			logmessage = g_strdup_printf("[%li.%li]\t%s>>>>>>>  %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
+			logmessage = g_strdup_printf("[%li.%06li]\t%s>>>>>>>  %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
 			tabs++;
 			break;
 		case TRACE_INTERNAL:
-			logmessage = g_strdup_printf("[%li.%li]\t%s%s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
+			logmessage = g_strdup_printf("[%li.%06li]\t%s%s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
 			break;
 		case TRACE_SENSITIVE:
-			if (!GPOINTER_TO_INT(g_private_get(trace_sensitive)))
-				logmessage = g_strdup_printf("[%li.%li]\t%s[SENSITIVE] %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
+			if (GPOINTER_TO_INT(g_private_get(trace_sensitive)))
+				logmessage = g_strdup_printf("[%li.%06li]\t%s[SENSITIVE] %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
 			else
-				logmessage = g_strdup_printf("[%li.%li]\t%s[SENSITIVE CONTENT HIDDEN]%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, endline);
+				logmessage = g_strdup_printf("[%li.%06li]\t%s[SENSITIVE CONTENT HIDDEN]%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, endline);
 			break;
 		case TRACE_EXIT:
-			logmessage = g_strdup_printf("[%li.%li]%s<<<<<<<  %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
+			logmessage = g_strdup_printf("[%li.%06li]%s<<<<<<<  %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
 			tabs--;
 			if (tabs < 0)
 				tabs = 0;
 			break;
 		case TRACE_EXIT_ERROR:
-			logmessage = g_strdup_printf("[%li.%li]%s<--- ERROR --- %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
+			logmessage = g_strdup_printf("[%li.%06li]%s<--- ERROR --- %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
 			tabs--;
 			if (tabs < 0)
 				tabs = 0;
+
+			if (print_stderr)
+				fprintf(stderr, "EXIT_ERROR: %s\n", buffer);
 			break;
 		case TRACE_ERROR:
-			logmessage = g_strdup_printf("[%li.%li]%sERROR: %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
+			logmessage = g_strdup_printf("[%li.%06li]%sERROR: %s%s", curtime.tv_sec, curtime.tv_usec, tabstr->str, buffer, endline);
+
+			if (print_stderr)
+				fprintf(stderr, "ERROR: %s\n", buffer);
+
 			break;
 	}
 	g_free(buffer);
@@ -247,34 +277,48 @@ char *osync_print_binary(const unsigned char *data, int len)
   return g_string_free(str, FALSE);
 }
 
-/*! @brief Creates a random string
- * 
- * Creates a random string of given length or less
- * 
- * @param maxlength The maximum length of the string
- * @returns The random string
+/*@}*/
+
+/**
+ * @defgroup OSyncEnvAPIMisc OpenSync Misc
+ * @ingroup OSyncPrivate
+ * @brief Some helper functions
  * 
  */
-char *osync_rand_str(int maxlength)
+/*@{*/
+
+/*! @brief Stop callback function to stop thread mainloop
+ *
+ * @param data Pointer to passed callback data
+ * @returns Always FALSE
+ */
+static gboolean osyncThreadStopCallback(gpointer data)
 {
-	char *randchars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIKLMNOPQRSTUVWXYZ1234567890";
+	OSyncThread *thread = data;
 	
-	int length;
-	char *retchar;
-	int i = 0;
-
-	length = g_random_int_range(1, maxlength + 1);
-	retchar = malloc(length * sizeof(char) + 1);
-	retchar[0] = 0;
-
-	for (i = 0; i < length; i++) {
-		retchar[i] = randchars[g_random_int_range(0, strlen(randchars))];
-		retchar[i + 1] = 0;
-	}
-
-	return retchar;
+	g_main_loop_quit(thread->loop);
+	
+	return FALSE;
 }
 
+/*! @brief Start callback function to emit signal when thread's mainloop got started
+ *
+ * @param data Pointer to passed callback data
+ * @returns Always FALSE
+ */
+static gboolean osyncThreadStartCallback(gpointer data)
+{
+	OSyncThread *thread = data;
+	
+	g_mutex_lock(thread->started_mutex);
+	g_cond_signal(thread->started);
+	g_mutex_unlock(thread->started_mutex);
+	
+	return FALSE;
+}
+
+/*@}*/
+
 /**
  * @defgroup OSyncEnvAPIMisc OpenSync Misc
  * @ingroup OSyncPublic
@@ -398,7 +442,15 @@ const char *osync_get_version(void)
  */
 void *osync_try_malloc0(unsigned int size, OSyncError **error)
 {
-	void *result = g_try_malloc(size);
+	void *result = NULL;
+	
+#ifdef OPENSYNC_UNITTESTS 	
+	if (!g_getenv("OSYNC_NOMEMORY"))
+		result = g_try_malloc(size);
+#else		
+	result = g_try_malloc(size);
+#endif /*OPENSYNC_UNITTESTS*/
+
 	if (!result) {
 		osync_error_set(error, OSYNC_ERROR_GENERIC, "No memory left");
 		return NULL;
@@ -407,8 +459,13 @@ void *osync_try_malloc0(unsigned int size, OSyncError **error)
 	return result;
 }
 
-/*@}*/
-
+/*! @brief Allocates a new thread with a g_mainloop 
+ * 
+ * @param context Pointer to GMainContext 
+ * @param error The error which will hold the info in case of an error
+ * @returns A pointer to the new allocated OSyncThread with inactive thread and mainloop 
+ * 
+ */
 OSyncThread *osync_thread_new(GMainContext *context, OSyncError **error)
 {
 	OSyncThread *thread = NULL;
@@ -435,6 +492,11 @@ error:
 	return NULL;
 }
 
+/*! @brief Frees thread object 
+ * 
+ * @param thread Pointer to OSyncThread
+ * 
+ */
 void osync_thread_free(OSyncThread *thread)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, thread);
@@ -469,15 +531,6 @@ void osync_thread_free(OSyncThread *thread)
 	return NULL;
 }*/
 
-static gboolean osyncThreadStopCallback(gpointer data)
-{
-	OSyncThread *thread = data;
-	
-	g_main_loop_quit(thread->loop);
-	
-	return FALSE;
-}
-
 /*void osync_thread_start(OSyncThread *thread)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, thread);
@@ -492,17 +545,51 @@ static gboolean osyncThreadStopCallback(gpointer data)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }*/
 
-static gboolean osyncThreadStartCallback(gpointer data)
+
+/*! @brief Start thread and it's mainloop 
+ * 
+ * @param func GThreadFunc Pointer  
+ * @param userdata Custom data poitner which get supplied to thread function
+ * @param error Pointer to error struct
+ * @return Newly allocate OSyncThread object with inactive mainloop
+ * 
+ */
+OSyncThread *osync_thread_create(GThreadFunc func, void *userdata, OSyncError **error)
 {
-	OSyncThread *thread = data;
+	osync_assert(func);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, func, userdata, error);
+
+	GError *gerror;
 	
-	g_mutex_lock(thread->started_mutex);
-	g_cond_signal(thread->started);
-	g_mutex_unlock(thread->started_mutex);
-	
-	return FALSE;
+	OSyncThread *thread = osync_try_malloc0(sizeof(OSyncThread), error);
+	if (!thread)
+		goto error;
+
+	if (!g_thread_supported ())
+		g_thread_init (NULL);
+
+	thread->thread = g_thread_create(func, userdata, TRUE, &gerror);
+
+	if (!thread->thread) {
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "%s", gerror->message);
+		g_error_free(gerror);
+		goto error;
+	}
+
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return thread;
+
+error:	
+	osync_trace(TRACE_EXIT_ERROR, "%s", __func__, osync_error_print(error));
+	return NULL;
 }
 
+
+/*! @brief Start thread and it's mainloop 
+ * 
+ * @param thread Thread object 
+ * 
+ */
 void osync_thread_start(OSyncThread *thread)
 {
 	GSource *idle = NULL;
@@ -519,8 +606,11 @@ void osync_thread_start(OSyncThread *thread)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }	
 
-
-
+/*! @brief Stops thread's mainloop and joins the thread
+ * 
+ * @param thread Thread object 
+ * 
+ */
 void osync_thread_stop(OSyncThread *thread)
 {
 	GSource *source = NULL;
@@ -539,6 +629,38 @@ void osync_thread_stop(OSyncThread *thread)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
+/*! @brief Exit thread 
+ * 
+ * @param thread Thread object 
+ * @param retval Return value of thread while exiting
+ * 
+ */
+void osync_thread_exit(OSyncThread *thread, int retval)
+{
+	GSource *source = NULL;
+	osync_trace(TRACE_ENTRY, "%s(%p, %i)", __func__, thread, retval);
+	osync_assert(thread);
+
+        source = g_idle_source_new();
+	g_source_set_callback(source, osyncThreadStopCallback, thread, NULL);
+        g_source_attach(source, thread->context);
+	g_source_unref(source);
+	thread->thread = NULL;
+
+	g_thread_exit(GINT_TO_POINTER(retval));
+	
+	osync_trace(TRACE_EXIT, "%s", __func__);
+}
+
+
+/*! @brief String replace
+ * 
+ * @param input Input string to work on
+ * @param delimiter Delimiter
+ * @param replacement Replacement
+ * @returns Replaced/Modified string result 
+ * 
+ */
 char *osync_strreplace(const char *input, const char *delimiter, const char *replacement)
 {
 	osync_return_val_if_fail(input != NULL, NULL);
@@ -552,9 +674,46 @@ char *osync_strreplace(const char *input, const char *delimiter, const char *rep
 	return ret;
 }
 
+/*! @brief Bit counting
+ * 
+ * MIT HAKMEM Count, Bit counting in constant time and memory. 
+ * 
+ * @param u unsigned integer value to count bits
+ * @returns The bit counting result 
+ * 
+ */
 int osync_bitcount(unsigned int u)
 {
 	unsigned int uCount = u - ((u >> 1) & 033333333333) - ((u >> 2) & 011111111111);
 	return ((uCount + (uCount >> 3)) & 030707070707) % 63;
 }
 
+/*! @brief Creates a random string
+ * 
+ * Creates a random string of given length or less
+ * 
+ * @param maxlength The maximum length of the string
+ * @returns The random string
+ * 
+ */
+char *osync_rand_str(int maxlength)
+{
+	char *randchars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIKLMNOPQRSTUVWXYZ1234567890";
+	
+	int length;
+	char *retchar;
+	int i = 0;
+
+	length = g_random_int_range(1, maxlength + 1);
+	retchar = malloc(length * sizeof(char) + 1);
+	retchar[0] = 0;
+
+	for (i = 0; i < length; i++) {
+		retchar[i] = randchars[g_random_int_range(0, strlen(randchars))];
+		retchar[i + 1] = 0;
+	}
+
+	return retchar;
+}
+/*@}*/
+
