@@ -1,5 +1,25 @@
-#ifndef HAVE_OPENSYNC_XML_H
-#define HAVE_OPENSYNC_XML_H
+/*
+ * libopensync - A synchronization framework
+ * Copyright (C) 2004-2005  Armin Bauer <armin.bauer@opensync.org>
+ * 
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ * 
+ */
+
+#ifndef _OPENSYNC_XML_H
+#define _OPENSYNC_XML_H
 
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
@@ -66,4 +86,4 @@ xmlChar *osync_xml_attr_get_content(xmlAttrPtr node);
 
 osync_bool osync_xml_open_file(xmlDocPtr *doc, xmlNodePtr *cur, const char *path, const char *topentry, OSyncError **error);
 
-#endif
+#endif /*_OPENSYNC_XML_H*/
