@@ -41,7 +41,7 @@
  */
 
 /**
- * @defgroup OSyncEnvAPI OpenSync Environment
+ * @defgroup OSyncPluginEnvAPI OpenSync Environment
  * @ingroup OSyncPublic
  * @brief The public API of the opensync environment
  * 
@@ -81,7 +81,7 @@ OSyncPluginEnv *osync_plugin_env_new(OSyncError **error)
 void osync_plugin_env_free(OSyncPluginEnv *env)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, env);
-	g_assert(env);
+	osync_assert(env);
 	
 	/* Free the plugins */
 	while (env->plugins) {
@@ -105,7 +105,7 @@ void osync_plugin_env_free(OSyncPluginEnv *env)
  * 
  * Loads all sync modules from a directory into a osync environment
  * 
- * @param env Pointer to a OSyncEnv environment
+ * @param env Pointer to a OSyncPluginEnv environment
  * @param path The path where to look for plugins
  * @param error Pointer to a error struct to return a error
  * @returns TRUE on success, FALSE otherwise
@@ -119,7 +119,7 @@ osync_bool osync_plugin_env_load(OSyncPluginEnv *env, const char *path, OSyncErr
 	char *filename = NULL;
 	const gchar *de = NULL;
 	
-	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, env, path, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, env, __NULLSTR(path), error);
 	
 	if (!path) {
 		path = OPENSYNC_PLUGINDIR;
@@ -145,16 +145,15 @@ osync_bool osync_plugin_env_load(OSyncPluginEnv *env, const char *path, OSyncErr
 	}
 	
 	while ((de = g_dir_read_name(dir))) {
-		filename = g_strdup_printf ("%s/%s", path, de);
+		filename = g_strdup_printf ("%s%c%s", path, G_DIR_SEPARATOR, de);
 		
-		if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR) || !g_pattern_match_simple("*.so", filename)) {
+		if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR) || !g_pattern_match_simple("*."G_MODULE_SUFFIX, filename)) {
 			g_free(filename);
 			continue;
 		}
 		
 		if (!osync_plugin_env_load_module(env, filename, error)) {
 			osync_trace(TRACE_ERROR, "Unable to load module: %s", osync_error_print(error));
-			osync_error_unref(error);
 		}
 		
 		g_free(filename);
@@ -170,6 +169,12 @@ error:
 	return FALSE;
 }
 
+/*! @brief Register plugin to plugin environment 
+ * 
+ * @param env Pointer to a plugin environment
+ * @param plugin Pointer to plugin which should get added to environment
+ * 
+ */
 void osync_plugin_env_register_plugin(OSyncPluginEnv *env, OSyncPlugin *plugin)
 {
 	osync_assert(env);
@@ -179,6 +184,14 @@ void osync_plugin_env_register_plugin(OSyncPluginEnv *env, OSyncPlugin *plugin)
 	osync_plugin_ref(plugin);
 }
 
+/*! @brief Loads a module into the plugin environment 
+ * 
+ * @param env Pointer to a plugin environment
+ * @param filename Module filename, as full path, to load
+ * @param error Pointer to error-struct
+ * @returns TRUE on success, FALSE otherwise
+ * 
+ */
 osync_bool osync_plugin_env_load_module(OSyncPluginEnv *env, const char *filename, OSyncError **error)
 {
 	OSyncModule *module = NULL;
@@ -193,18 +206,16 @@ osync_bool osync_plugin_env_load_module(OSyncPluginEnv *env, const char *filenam
 	
 	if (!osync_module_load(module, filename, error)) {
 		osync_trace(TRACE_INTERNAL, "Unable to load module %s: %s", filename, osync_error_print(error));
-		osync_error_unref(error);
 		osync_module_free(module);
 	} else {
 		if (!osync_module_check(module, error)) {
 			if (osync_error_is_set(error)) {
 				osync_trace(TRACE_INTERNAL, "Module check error for %s: %s", filename, osync_error_print(error));
-				osync_error_unref(error);
 			}
 			osync_module_unload(module);
 			osync_module_free(module);
 			osync_trace(TRACE_EXIT, "%s: Unable to load module", __func__);
-			return TRUE;
+			return FALSE;
 		}
 		
 		if (!osync_module_get_sync_info(module, env, error)) {
@@ -213,8 +224,8 @@ osync_bool osync_plugin_env_load_module(OSyncPluginEnv *env, const char *filenam
 			
 			osync_module_unload(module);
 			osync_module_free(module);
-			osync_trace(TRACE_EXIT, "%s: No get_info function", __func__);
-			return TRUE;
+			osync_trace(TRACE_EXIT, "%s: No get_sync_info function", __func__);
+			return FALSE;
 		}
 		env->modules = g_list_append(env->modules, module);
 	}
@@ -234,7 +245,7 @@ error:
  * 
  * Finds the plugin with the given name
  * 
- * @param env Pointer to a OSyncEnv environment
+ * @param env Pointer to a OSyncPluginEnv environment
  * @param name The name to search for
  * @returns The plugin or NULL if not found
  * 
@@ -255,7 +266,7 @@ OSyncPlugin *osync_plugin_env_find_plugin(OSyncPluginEnv *env, const char *name)
  * 
  * Returns the number of loaded plugins. 0 if used before initialization
  * 
- * @param env Pointer to a OSyncEnv environment
+ * @param env Pointer to a OSyncPluginEnv environment
  * @returns Number of plugins
  * 
  */
@@ -268,7 +279,7 @@ int osync_plugin_env_num_plugins(OSyncPluginEnv *env)
  * 
  * Returns pointer to nth plugin
  * 
- * @param env Pointer to a OSyncEnv environment
+ * @param env Pointer to a OSyncPluginEnv environment
  * @param nth Which plugin to return
  * @returns Pointer to plugin
  * 
@@ -278,8 +289,17 @@ OSyncPlugin *osync_plugin_env_nth_plugin(OSyncPluginEnv *env, int nth)
 	return (OSyncPlugin *)g_list_nth_data(env->plugins, nth);
 }
 
+/*! @brief Checks if plugin is usable 
+ * 
+ * @param env Pointer to a OSyncPluginEnv environment
+ * @param pluginname The name of the plugin
+ * @param error Pointer to error-struct
+ * @returns TRUE if plugin is usable, FALSE otherwise 
+ * 
+ */
 osync_bool osync_plugin_env_plugin_is_usable(OSyncPluginEnv *env, const char *pluginname, OSyncError **error)
 {
+	/* TODO: implement USABLE functions */
 	return TRUE;
 }
 
