@@ -22,13 +22,15 @@
 #include "opensync_internals.h"
 
 #include "opensync-context.h"
+#include "opensync-format.h"
+
 #include "opensync_sink.h"
 #include "opensync_sink_internals.h"
 
 /**
- * @defgroup OSyncSinkAPI OpenSync Sink
+ * @defgroup OSyncObjTypeSinkAPI OpenSync Object Type Sink
  * @ingroup OSyncPublic
- * @brief Functions to register and manage sinks
+ * @brief Functions to register and manage object type sinks
  * 
  */
 /*@{*/
@@ -39,7 +41,7 @@
  * neutral actions. Actions like connecting/disconnecting which could
  * be a object type neutral - e.g. connecting to a device via bluetooth.
  * Object type specific example would be a connection to different 
- * object type specific databases/ressources.
+ * object type specific databases/resources.
  *
  * Main sink is not limited to the connect and disconnect functions! 
  * 
@@ -65,6 +67,8 @@ OSyncObjTypeSink *osync_objtype_sink_new(const char *objtype, OSyncError **error
 	
 	sink->objtype = g_strdup(objtype);
 	sink->ref_count = 1;
+
+	sink->preferred_format = NULL;
 	
 	sink->read = TRUE;
 	sink->getchanges = TRUE;
@@ -98,21 +102,17 @@ OSyncObjTypeSink *osync_objtype_sink_ref(OSyncObjTypeSink *sink)
  */
 void osync_objtype_sink_unref(OSyncObjTypeSink *sink)
 {
-	OSyncList *f = NULL;
 	osync_assert(sink);
 	
 	if (g_atomic_int_dec_and_test(&(sink->ref_count))) {
-		while (sink->objformats) {
-			char **format_vertice = sink->objformats->data;
-			char *format = format_vertice[0];
-			char *format_config = format_vertice[1];
-			g_free(format);
-			if(format_config)
-			      g_free(format_config);
-			g_free(format_vertice);
-			sink->objformats = osync_list_remove(sink->objformats, sink->objformats->data);
+		while (sink->objformatsinks) {
+			osync_objformat_sink_unref(sink->objformatsinks->data);
+			sink->objformatsinks = osync_list_remove(sink->objformatsinks, sink->objformatsinks->data);
 		}
 		
+		if (sink->preferred_format)
+			g_free(sink->preferred_format);
+
 		if (sink->objtype)
 			g_free(sink->objtype);
 		
@@ -146,16 +146,30 @@ void osync_objtype_sink_set_name(OSyncObjTypeSink *sink, const char *name)
 	sink->objtype = g_strdup(name);
 }
 
-static osync_bool _osync_objtype_sink_find_objformat(OSyncObjTypeSink *sink, const char *format)
+/*! @brief Return the preferred format for the conversion 
+ * 
+ * @param sink Pointer to the sink
+ * @returns the name of the preferred format
+ * 
+ */
+const char *osync_objtype_sink_get_preferred_format(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
-	OSyncList *f = sink->objformats;
-	for (; f; f = f->next) {
-		const char **format_vertice = f->data;
-		if (!strcmp(format_vertice[0], format))
-			return TRUE;
-	}
-	return FALSE;
+	return sink->preferred_format;
+}
+
+/*! @brief Set the preferred format for the conversion
+ * 
+ * @param sink Pointer to the sink
+ * @param name the name of the preferred format to set
+ * 
+ */
+void osync_objtype_sink_set_preferred_format(OSyncObjTypeSink *sink, const char *preferred_format)
+{
+	osync_assert(sink);
+	if (sink->preferred_format)
+		g_free(sink->preferred_format);
+	sink->preferred_format = g_strdup(preferred_format);
 }
 
 /*! @brief Returns the number of object formats in the sink
@@ -164,10 +178,10 @@ static osync_bool _osync_objtype_sink_find_objformat(OSyncObjTypeSink *sink, con
  * @returns the number of object formats in the sink
  * 
  */
-int osync_objtype_sink_num_objformats(OSyncObjTypeSink *sink)
+unsigned int osync_objtype_sink_num_objformat_sinks(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
-	return osync_list_length(sink->objformats);
+	return osync_list_length(sink->objformatsinks);
 }
 
 /*! @brief Returns the nth object format in the sink
@@ -177,82 +191,61 @@ int osync_objtype_sink_num_objformats(OSyncObjTypeSink *sink)
  * @returns the name of the object format at the specified index
  * 
  */
-const char *osync_objtype_sink_nth_objformat(OSyncObjTypeSink *sink, int nth)
+OSyncObjFormatSink *osync_objtype_sink_nth_objformat_sink(OSyncObjTypeSink *sink, unsigned int nth)
 {
 	osync_assert(sink);
-	char **format_vertice = osync_list_nth_data(sink->objformats, nth);
-	if (format_vertice)
-		return format_vertice[0];
-	else
-		return NULL;
+	return osync_list_nth_data(sink->objformatsinks, nth);
 }
 
-/*! @brief Returns the nth object format conversion config in the sink
+/*! @brief Finds the objformat sink for the corresponding objformat 
  * 
  * @param sink Pointer to the sink
- * @param nth the index of the object format conversion config to return
- * @returns the conversion config of the object format at the specified index
+ * @param objformat the objformat to look for the corresponding objformat sink
+ * @returns Pointer to the corresponding objformat sink if found, NULL otherwise 
  * 
  */
-const char *osync_objtype_sink_nth_objformat_config(OSyncObjTypeSink *sink, int nth)
+OSyncObjFormatSink *osync_objtype_sink_find_objformat_sink(OSyncObjTypeSink *sink, OSyncObjFormat *objformat)
 {
 	osync_assert(sink);
-	char **format_vertice = osync_list_nth_data(sink->objformats, nth);
-	if (format_vertice) 
-		return format_vertice[1];
-	else
-		return NULL;
-}
+	osync_assert(objformat);
 
-/** @brief Returns the list of object formats in the sink
- * 
- * @param sink Pointer to the sink
- * @returns the list of formats in the sink
- * 
- */
-const OSyncList *osync_objtype_sink_get_objformats(OSyncObjTypeSink *sink)
-{
-	osync_assert(sink);
-	return sink->objformats;
-}
-
-/** @brief Adds an object format to the sink
- * 
- * @param sink Pointer to the sink
- * @param format name of the object format
- * 
- */
-void osync_objtype_sink_add_objformat(OSyncObjTypeSink *sink, const char *format)
-{
-	osync_assert(sink);
-	osync_assert(format);
-	
-	if (!_osync_objtype_sink_find_objformat(sink, format)) {
-	        char **format_vertice = g_malloc0(2*sizeof(char **));
-		format_vertice[0] = g_strdup(format);
-		format_vertice[1] = NULL;
-		sink->objformats = osync_list_append(sink->objformats, format_vertice);
+	OSyncList *f = sink->objformatsinks;
+	for (; f; f = f->next) {
+		OSyncObjFormatSink *formatsink = f->data;
+		const char *objformat_name = osync_objformat_get_name(objformat);
+		if (!strcmp(osync_objformat_sink_get_objformat(formatsink), objformat_name))
+			return formatsink;
 	}
+	return NULL;
 }
 
-/** @brief Adds an object format with its conversion config to the sink
+/*! @brief Get list of object format sinks 
  * 
  * @param sink Pointer to the sink
- * @param format name of the object format
- * @param format_config config of the object format for the conversion path
+ * @returns List of object format sinks 
  * 
  */
-void osync_objtype_sink_add_objformat_with_config(OSyncObjTypeSink *sink, const char *format, const char *format_config)
+OSyncList *osync_objtype_sink_get_objformat_sinks(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
-	osync_assert(format);
-	
-	if (!_osync_objtype_sink_find_objformat(sink, format)) {
-	        char **format_vertice = g_malloc0(2*sizeof(char **));
-		format_vertice[0] = g_strdup(format);
-		format_vertice[1] = g_strdup(format_config);
-		osync_trace(TRACE_INTERNAL, "CONFIG %s", format_vertice[1]);
-		sink->objformats = osync_list_append(sink->objformats, format_vertice);
+	return sink->objformatsinks;
+}
+
+
+/** @brief Adds an object format sink to the sink
+ * 
+ * @param sink Pointer to the sink
+ * @param objformat The object format sink to add 
+ * 
+ */
+void osync_objtype_sink_add_objformat_sink(OSyncObjTypeSink *sink, OSyncObjFormatSink *objformatsink)
+{
+	osync_assert(sink);
+	osync_assert(objformatsink);
+
+	if (!osync_list_find(sink->objformatsinks, objformatsink)) {
+		sink->objformatsinks = osync_list_append(sink->objformatsinks, objformatsink);
+		osync_objformat_sink_ref(objformatsink);
 	}
 }
 
@@ -262,18 +255,13 @@ void osync_objtype_sink_add_objformat_with_config(OSyncObjTypeSink *sink, const
  * @param format name of the object format to remove
  * 
  */
-void osync_objtype_sink_remove_objformat(OSyncObjTypeSink *sink, const char *format)
+void osync_objtype_sink_remove_objformat_sink(OSyncObjTypeSink *sink, OSyncObjFormatSink *objformatsink)
 {
-	OSyncList *f = NULL;
 	osync_assert(sink);
-	osync_assert(format);
-	for (f = sink->objformats; f; f = f->next) {
-		const char **format_vertice = f->data;
-		if (!strcmp(format_vertice[0], format)) {
-			sink->objformats = osync_list_remove(sink->objformats, f->data);
-			break;
-		}
-	}
+	osync_assert(objformatsink);
+
+	sink->objformatsinks = osync_list_remove(sink->objformatsinks, objformatsink);
+	osync_objformat_sink_unref(objformatsink);
 }
 
 /** @brief Sets the sink functions and user data
@@ -318,7 +306,7 @@ osync_bool osync_objtype_sink_get_function_read(OSyncObjTypeSink *sink)
 /*! @brief Sets the status of the read sink function
  *
  * @param sink Pointer to sink
- * @param write TRUE if the sink has a read function, FALSE otherwise
+ * @param read TRUE if the sink has a read function, FALSE otherwise
  */
 void osync_objtype_sink_set_function_read(OSyncObjTypeSink *sink, osync_bool read)
 {
@@ -340,7 +328,7 @@ osync_bool osync_objtype_sink_get_function_getchanges(OSyncObjTypeSink *sink)
 /*! @brief Sets the status of the get_changes sink function
  *
  * @param sink Pointer to sink
- * @param write TRUE if the sink has a get_changes function, FALSE otherwise
+ * @param getchanges TRUE if the sink has a get_changes function, FALSE otherwise
  */
 void osync_objtype_sink_set_function_getchanges(OSyncObjTypeSink *sink, osync_bool getchanges)
 {
@@ -717,7 +705,7 @@ void osync_objtype_sink_set_write(OSyncObjTypeSink *sink, osync_bool write)
  * See osync_objtype_sink_get_getchanges()
  *
  * @param sink Pointer to sink
- * @param write Set TRUE if the sink is allowed to get latest changes, FALSE otherwise
+ * @param getchanges Set TRUE if the sink is allowed to get latest changes, FALSE otherwise
  *
  */
 void osync_objtype_sink_set_getchanges(OSyncObjTypeSink *sink, osync_bool getchanges)
@@ -778,7 +766,7 @@ void osync_objtype_sink_set_read(OSyncObjTypeSink *sink, osync_bool read)
  *
  * If you are using hashtables, you should call this function in your sink's
  * get_changes() function and if slow-sync has been requested, call
- * osync_hashtable_reset() on your hashtable. If you don't do this, OpenSync
+ * osync_hashtable_slowsync() on your hashtable. If you don't do this, OpenSync
  * will assume that all entries should be deleted, which is usually not what 
  * the user wants.
  *
@@ -807,167 +795,329 @@ osync_bool osync_objtype_sink_get_slowsync(OSyncObjTypeSink *sink)
  */
 void osync_objtype_sink_set_slowsync(OSyncObjTypeSink *sink, osync_bool slowsync)
 {
-	osync_trace(TRACE_INTERNAL, "%s: Setting slow-sync of object type \"%s\" to %i", __func__, sink->objtype, slowsync);
 	osync_assert(sink);
+	osync_trace(TRACE_INTERNAL, "%s: Setting slow-sync of object type \"%s\" to %i", __func__, sink->objtype, slowsync);
 	sink->slowsync = slowsync;
 }
 
+/*! @brief Sets the connect timeout in seconds for the OSyncObjTypeSink 
+ * 
+ * @param sink Pointer to the sink
+ * @param timeout The timeout in seconds 
+ * 
+ */
 void osync_objtype_sink_set_connect_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
 {
 	osync_assert(sink);
 	sink->timeout.connect = timeout;
 }
 
+/*! @brief Get the current or default connect timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_connect_timeout_or_default(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.connect ? sink->timeout.connect : OSYNC_SINK_TIMEOUT_CONNECT;
 }
 
+/*! @brief Get the current connect timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_connect_timeout(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.connect;
 }
 
+/*! @brief Sets the disconnect timeout in seconds for the OSyncObjTypeSink 
+ * 
+ * @param sink Pointer to the sink
+ * @param timeout The timeout in seconds 
+ * 
+ */
 void osync_objtype_sink_set_disconnect_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
 {
 	osync_assert(sink);
 	sink->timeout.disconnect = timeout;
 }
 
+/*! @brief Get the current or default disconnect timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_disconnect_timeout_or_default(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.disconnect ? sink->timeout.disconnect : OSYNC_SINK_TIMEOUT_DISCONNECT;
 }
 
+/*! @brief Get the current disconnect timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_disconnect_timeout(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.disconnect;
 }
 
+/*! @brief Sets the get_changes timeout in seconds for the OSyncObjTypeSink 
+ * 
+ * @param sink Pointer to the sink
+ * @param timeout The timeout in seconds 
+ * 
+ */
 void osync_objtype_sink_set_getchanges_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
 {
 	osync_assert(sink);
 	sink->timeout.get_changes = timeout;
 }
 
+/*! @brief Get the current or default getchanges timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_getchanges_timeout_or_default(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.commit ? sink->timeout.get_changes : OSYNC_SINK_TIMEOUT_GETCHANGES;
 }
 
+/*! @brief Get the current getchanges timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_getchanges_timeout(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.get_changes;
 }
 
+/*! @brief Sets the commit timeout in seconds for the OSyncObjTypeSink 
+ * 
+ * @param sink Pointer to the sink
+ * @param timeout The timeout in seconds 
+ * 
+ */
 void osync_objtype_sink_set_commit_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
 {
 	osync_assert(sink);
 	sink->timeout.commit = timeout;
 }
 
+/*! @brief Get the current or default commit timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_commit_timeout_or_default(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.commit ? sink->timeout.commit : OSYNC_SINK_TIMEOUT_COMMIT;
 }
 
+/*! @brief Get the current commit timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_commit_timeout(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.commit;
 }
 
+/*! @brief Sets the batchcommit timeout in seconds for the OSyncObjTypeSink 
+ * 
+ * @param sink Pointer to the sink
+ * @param timeout The timeout in seconds 
+ * 
+ */
 void osync_objtype_sink_set_batchcommit_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
 {
 	osync_assert(sink);
 	sink->timeout.batch_commit = timeout;
 }
 
+/*! @brief Get the current or default batchcommit timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_batchcommit_timeout_or_default(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.batch_commit ? sink->timeout.batch_commit : OSYNC_SINK_TIMEOUT_BATCHCOMMIT;
 }
 
+/*! @brief Get the current batchcommit timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_batchcommit_timeout(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.batch_commit;
 }
 
+/*! @brief Sets the committedall timeout in seconds for the OSyncObjTypeSink 
+ * 
+ * @param sink Pointer to the sink
+ * @param timeout The timeout in seconds 
+ * 
+ */
 void osync_objtype_sink_set_committedall_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
 {
 	osync_assert(sink);
 	sink->timeout.committed_all = timeout;
 }
 
+/*! @brief Get the current or default committedall timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_committedall_timeout_or_default(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.committed_all ? sink->timeout.committed_all : OSYNC_SINK_TIMEOUT_COMMITTEDALL;
 }
 
+/*! @brief Get the current committedall timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_committedall_timeout(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.committed_all;
 }
 
+/*! @brief Sets the syncdone timeout in seconds for the OSyncObjTypeSink 
+ * 
+ * @param sink Pointer to the sink
+ * @param timeout The timeout in seconds 
+ * 
+ */
 void osync_objtype_sink_set_syncdone_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
 {
 	osync_assert(sink);
 	sink->timeout.sync_done = timeout;
 }
 
+/*! @brief Get the current or default syncdone timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_syncdone_timeout_or_default(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.sync_done ? sink->timeout.sync_done : OSYNC_SINK_TIMEOUT_SYNCDONE;
 }
 
+/*! @brief Get the current syncdone timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_syncdone_timeout(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.sync_done;
 }
 
+/*! @brief Sets the write timeout in seconds for the OSyncObjTypeSink 
+ * 
+ * @param sink Pointer to the sink
+ * @param timeout The timeout in seconds 
+ * 
+ */
 void osync_objtype_sink_set_write_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
 {
 	osync_assert(sink);
 	sink->timeout.write = timeout;
 }
 
+/*! @brief Get the current or default write timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_write_timeout_or_default(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.write ? sink->timeout.write : OSYNC_SINK_TIMEOUT_WRITE;
 }
 
+/*! @brief Get the current write timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_write_timeout(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.write;
 }
 
+/*! @brief Sets the read timeout in seconds for the OSyncObjTypeSink 
+ * 
+ * @param sink Pointer to the sink
+ * @param timeout The timeout in seconds 
+ * 
+ */
 void osync_objtype_sink_set_read_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
 {
 	osync_assert(sink);
 	sink->timeout.read = timeout;
 }
 
+/*! @brief Get the current or default read timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_read_timeout_or_default(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->timeout.read ? sink->timeout.read : OSYNC_SINK_TIMEOUT_READ;
 }
 
+/*! @brief Get the current read timeout in seconds 
+ * 
+ * @param sink Pointer to the sink
+ * @return The timeout in seconds 
+ * 
+ */
 unsigned int osync_objtype_sink_get_read_timeout(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
