@@ -66,8 +66,10 @@ typedef struct OSyncObjTypeSinkFunctionTimeouts {
 } OSyncObjTypeSinkFunctionTimeouts;
 
 struct OSyncObjTypeSink {
+	/** The preferred step or target format for the conversion path of this sink */
+	char *preferred_format;
 	/** The format which can be synchronized by this sink */
-	OSyncList *objformats;
+	OSyncList *objformatsinks;
 	/** The functions to be called */
 	OSyncObjTypeSinkFunctions functions;
 	void *userdata;
