@@ -87,6 +87,120 @@ error:
 	return -1;
 }
 
+/**
+ * @brief Loads description from a specified directory and returns an OSyncList
+ * with OSyncVersions objects.
+ *
+ * This internal function allows to adjust the description and schema directories.
+ *
+ * @param error Pointer to error-struct
+ * @param descriptiondir Path to description directory
+ * @param schemadir Path to XML schema directory
+ * @returns List of OSyncVersion objects, NULL if none found
+ */
+OSyncList *_osync_version_load_from_descriptions(OSyncError **error, const char *descriptiondir, const char *schemadir)
+{
+	GDir *dir = NULL;
+	GError *gerror = NULL;
+	const char *descpath = descriptiondir ? descriptiondir : OPENSYNC_DESCRIPTIONSDIR; 
+	const char *schemapath = schemadir ? schemadir : OPENSYNC_SCHEMASDIR; 
+	char *filename = NULL;
+	const gchar *de = NULL;
+	OSyncList *versions = NULL;
+	OSyncVersion *version = NULL;
+	xmlDocPtr doc;
+	xmlNodePtr root;
+	xmlNodePtr cur;
+	xmlNodePtr child;
+	
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, error);
+	
+	dir = g_dir_open(descpath, 0, &gerror);
+	if (!dir) {
+		/* If description directory doesn't exist (e.g. unittests), just ignore this. */
+		osync_trace(TRACE_EXIT, "Unable to open directory %s: %s", descpath, gerror->message);
+		g_error_free(gerror);
+		return NULL;
+	}
+	
+	while ((de = g_dir_read_name(dir))) {
+		filename = g_strdup_printf ("%s%c%s", descpath, G_DIR_SEPARATOR, de);
+		
+		if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR) || !g_pattern_match_simple("*.xml", filename)) {
+			g_free(filename);
+			continue;
+		}
+		
+		doc = xmlReadFile(filename, NULL, XML_PARSE_NOBLANKS);
+		if(!doc) {
+			g_free(filename);
+			continue;
+		}
+		
+		g_free(filename);
+		
+		root = xmlDocGetRootElement(doc);
+		if(!root || !xmlStrEqual(root->name, BAD_CAST "versions")) {
+			xmlFreeDoc(doc);
+			continue;
+		}
+
+		char *schemafilepath = g_strdup_printf("%s%c%s", schemapath, G_DIR_SEPARATOR, "descriptions.xsd");
+ 		osync_bool res = osync_xml_validate_document(doc, schemafilepath);
+ 		g_free(schemafilepath);
+
+		if(res == FALSE) {
+			xmlFreeDoc(doc);
+			continue;
+		}
+		
+		cur = root->children;
+		for(; cur != NULL; cur = cur->next) {
+		
+			version = osync_version_new(error);
+			if(!version) {
+				xmlFreeDoc(doc);
+				OSyncList *cur = osync_list_first(versions);
+				while(cur) {
+					osync_version_unref(cur->data);
+					cur = cur->next;	
+				}
+				goto error;
+			}
+				
+			child = cur->children;
+			osync_version_set_plugin(version, (const char *)osync_xml_node_get_content(child));
+			child = child->next;
+			osync_version_set_priority(version, (const char *)osync_xml_node_get_content(child));
+			child = child->next;
+			osync_version_set_vendor(version, (const char *)osync_xml_node_get_content(child));
+			child = child->next;
+			osync_version_set_modelversion(version, (const char *)osync_xml_node_get_content(child));
+			child = child->next;
+			osync_version_set_firmwareversion(version, (const char *)osync_xml_node_get_content(child));
+			child = child->next;
+			osync_version_set_softwareversion(version, (const char *)osync_xml_node_get_content(child));
+			child = child->next;
+			osync_version_set_hardwareversion(version, (const char *)osync_xml_node_get_content(child));
+			child = child->next;
+			osync_version_set_identifier(version, (const char *)osync_xml_node_get_content(child));
+			
+			versions = osync_list_append(versions, version);
+		}
+		
+		xmlFreeDoc(doc);
+	}
+	
+	g_dir_close(dir);
+	
+	osync_trace(TRACE_EXIT, "%s: %p", __func__, versions);
+	return versions;
+
+error:
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	return NULL;
+}
+
 /*@}*/
 
 /**
@@ -171,46 +285,92 @@ void osync_version_unref(OSyncVersion *version)
 	}
 }
 
+/**
+ * @brief Get Plugin Name
+ * @param version The pointer to a version object
+ * @returns Plugin Name or NULL
+ */
+/* FIXME: char* to const char* */
 char *osync_version_get_plugin(OSyncVersion *version)
 {
 	return version->plugin;
 }
 
+/**
+ * @brief Get Priority 
+ * @param version The pointer to a version object
+ * @returns Priority or NULL
+ */
 char *osync_version_get_priority(OSyncVersion *version)
 {
 	return version->priority;
 }
 
+/**
+ * @brief Get Vendor 
+ * @param version The pointer to a version object
+ * @returns Vendor or NULL
+ */
 char *osync_version_get_vendor(OSyncVersion *version)
 {
 	return version->vendor;
 }
 
+/**
+ * @brief Get Model Version
+ * @param version The pointer to a version object
+ * @returns Model Version or NULL
+ */
 char *osync_version_get_modelversion(OSyncVersion *version)
 {
 	return version->modelversion;
 }
 
+/**
+ * @brief Get Firmware Version
+ * @param version The pointer to a version object
+ * @returns Firmware Version or NULL
+ */
 char *osync_version_get_firmwareversion(OSyncVersion *version)
 {
 	return version->firmwareversion;
 }
 
+/**
+ * @brief Get Software Version
+ * @param version The pointer to a version object
+ * @returns Software Version or NULL
+ */
 char *osync_version_get_softwareversion(OSyncVersion *version)
 {
 	return version->softwareversion;
 }
 
+/**
+ * @brief Get Hardware Version
+ * @param version The pointer to a version object
+ * @returns Hardware Version or NULL
+ */
 char *osync_version_get_hardwareversion(OSyncVersion *version)
 {
 	return version->hardwareversion;
 }
 
+/**
+ * @brief Get Identifier 
+ * @param version The pointer to a version object
+ * @returns Identifier or NULL
+ */
 char *osync_version_get_identifier(OSyncVersion *version)
 {
 	return version->identifier;
 }
 
+/**
+ * @brief Set Plugin Name 
+ * @param version The pointer to a version object
+ * @param plugin Plugin Name
+ */
 void osync_version_set_plugin(OSyncVersion *version, const char *plugin)
 {
 	if(version->plugin)
@@ -221,6 +381,11 @@ void osync_version_set_plugin(OSyncVersion *version, const char *plugin)
 		version->plugin = g_strdup(plugin);
 }
 
+/**
+ * @brief Set Priority
+ * @param version The pointer to a version object
+ * @param priority Priority
+ */
 void osync_version_set_priority(OSyncVersion *version, const char *priority)
 {
 	if(version->priority)
@@ -231,6 +396,11 @@ void osync_version_set_priority(OSyncVersion *version, const char *priority)
 		version->priority =  g_strdup(priority);
 }
 
+/**
+ * @brief Set Vendor 
+ * @param version The pointer to a version object
+ * @param vendor Vendor 
+ */
 void osync_version_set_vendor(OSyncVersion *version, const char *vendor)
 {
 	if(version->vendor)
@@ -241,6 +411,11 @@ void osync_version_set_vendor(OSyncVersion *version, const char *vendor)
 		version->vendor =  g_strdup(vendor);
 }
 
+/**
+ * @brief Set Model Version 
+ * @param version The pointer to a version object
+ * @param modelversion Model Version 
+ */
 void osync_version_set_modelversion(OSyncVersion *version, const char *modelversion)
 {
 	if(version->modelversion)
@@ -251,6 +426,11 @@ void osync_version_set_modelversion(OSyncVersion *version, const char *modelvers
 		version->modelversion =  g_strdup(modelversion);
 }
 
+/**
+ * @brief Set Firmware Version 
+ * @param version The pointer to a version object
+ * @param firmwareversion Firmware Version 
+ */
 void osync_version_set_firmwareversion(OSyncVersion *version, const char *firmwareversion)
 {
 	if(version->firmwareversion)
@@ -261,6 +441,11 @@ void osync_version_set_firmwareversion(OSyncVersion *version, const char *firmwa
 		version->firmwareversion =  g_strdup(firmwareversion);
 }
 
+/**
+ * @brief Set Software Version 
+ * @param version The pointer to a version object
+ * @param softwareversion Software Version 
+ */
 void osync_version_set_softwareversion(OSyncVersion *version, const char *softwareversion)
 {
 	if(version->softwareversion)
@@ -272,6 +457,11 @@ void osync_version_set_softwareversion(OSyncVersion *version, const char *softwa
 	
 }
 
+/**
+ * @brief Set Hardware Version 
+ * @param version The pointer to a version object
+ * @param hardwareversion Hardware Version 
+ */
 void osync_version_set_hardwareversion(OSyncVersion *version, const char *hardwareversion)
 {
 	if(version->hardwareversion)
@@ -282,6 +472,11 @@ void osync_version_set_hardwareversion(OSyncVersion *version, const char *hardwa
 		version->hardwareversion =  g_strdup(hardwareversion);
 }
 
+/**
+ * @brief Set Identifier 
+ * @param version The pointer to a version object
+ * @param identifier Identifier 
+ */
 void osync_version_set_identifier(OSyncVersion *version, const char *identifier)
 {
 	if(version->identifier)
@@ -292,6 +487,15 @@ void osync_version_set_identifier(OSyncVersion *version, const char *identifier)
 		version->identifier =  g_strdup(identifier);
 }
 
+/**
+ * @brief Matchs the version object with a pattern object and returns the priority
+ * of the pattern if it matchs the original version object.
+ *
+ * @param pattern The pointer to a version object which acts as pattern
+ * @param version The version (original) object supplied to find a fitting version pattern
+ * @param error Pointer to a error-struct
+ * @returns Priority of matching pattern, -1 on error
+ */
 int osync_version_matches(OSyncVersion *pattern, OSyncVersion *version, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, pattern, version, error);
@@ -336,116 +540,25 @@ error:
 	return -1;
 }
 
-
-OSyncList *_osync_version_load_from_descriptions(OSyncError **error, const char *descriptiondir, const char *schemadir)
-{
-	GDir *dir = NULL;
-	GError *gerror = NULL;
-	const char *descpath = descriptiondir ? descriptiondir : OPENSYNC_DESCRIPTIONSDIR; 
-	const char *schemapath = schemadir ? schemadir : OPENSYNC_SCHEMASDIR; 
-	char *filename = NULL;
-	const gchar *de = NULL;
-	OSyncList *versions = NULL;
-	OSyncVersion *version = NULL;
-	xmlDocPtr doc;
-	xmlNodePtr root;
-	xmlNodePtr cur;
-	xmlNodePtr child;
-	
-	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, error);
-	
-	dir = g_dir_open(descpath, 0, &gerror);
-	if (!dir) {
-		/* If description directory doesn't exist (e.g. unittests), just ignore this. */
-		osync_trace(TRACE_EXIT, "Unable to open directory %s: %s", descpath, gerror->message);
-		g_error_free(gerror);
-		return NULL;
-	}
-	
-	while ((de = g_dir_read_name(dir))) {
-		filename = g_strdup_printf ("%s/%s", descpath, de);
-		
-		if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR) || !g_pattern_match_simple("*.xml", filename)) {
-			g_free(filename);
-			continue;
-		}
-		
-		doc = xmlReadFile(filename, NULL, XML_PARSE_NOBLANKS);
-		if(!doc) {
-			g_free(filename);
-			continue;
-		}
-		
-		g_free(filename);
-		
-		root = xmlDocGetRootElement(doc);
-		if(!root || !xmlStrEqual(root->name, BAD_CAST "versions")) {
-			xmlFreeDoc(doc);
-			continue;
-		}
-
-		char *schemafilepath = g_strdup_printf("%s%c%s", schemapath, G_DIR_SEPARATOR, "descriptions.xsd");
- 		osync_bool res = osync_xml_validate_document(doc, schemafilepath);
- 		g_free(schemafilepath);
-
-		if(res == FALSE) {
-			xmlFreeDoc(doc);
-			continue;
-		}
-		
-		cur = root->children;
-		for(; cur != NULL; cur = cur->next) {
-		
-			version = osync_version_new(error);
-			if(!version) {
-				xmlFreeDoc(doc);
-				OSyncList *cur = osync_list_first(versions);
-				while(cur) {
-					osync_version_unref(cur->data);
-					cur = cur->next;	
-				}
-				goto error;
-			}
-				
-			child = cur->children;
-			osync_version_set_plugin(version, (const char *)osync_xml_node_get_content(child));
-			child = child->next;
-			osync_version_set_priority(version, (const char *)osync_xml_node_get_content(child));
-			child = child->next;
-			osync_version_set_vendor(version, (const char *)osync_xml_node_get_content(child));
-			child = child->next;
-			osync_version_set_modelversion(version, (const char *)osync_xml_node_get_content(child));
-			child = child->next;
-			osync_version_set_firmwareversion(version, (const char *)osync_xml_node_get_content(child));
-			child = child->next;
-			osync_version_set_softwareversion(version, (const char *)osync_xml_node_get_content(child));
-			child = child->next;
-			osync_version_set_hardwareversion(version, (const char *)osync_xml_node_get_content(child));
-			child = child->next;
-			osync_version_set_identifier(version, (const char *)osync_xml_node_get_content(child));
-			
-			versions = osync_list_append(versions, version);
-		}
-		
-		xmlFreeDoc(doc);
-	}
-	
-	g_dir_close(dir);
-	
-	osync_trace(TRACE_EXIT, "%s: %p", __func__, versions);
-	return versions;
-
-error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return NULL;
-}
-
+/**
+ * @brief Loads description from the default description directory and returns an OSyncList
+ * with OSyncVersions objects.
+ *
+ * @param error Pointer to error-struct
+ * @returns List of OSyncVersion objects, NULL if none found
+ */
 OSyncList *osync_version_load_from_descriptions(OSyncError **error)
 {
 	return _osync_version_load_from_descriptions(error, NULL, NULL);
 }
 
-
+/**
+ * @brief Searching for capabilities for specified OSyncVersion object. 
+ *
+ * @param version Pointer to OSyncVersion object
+ * @param error Pointer to error-struct
+ * @returns Pointer to OSyncCapabilties object, NULL if none capabilities are found
+ */
 OSyncCapabilities *osync_version_find_capabilities(OSyncVersion *version, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, version, error);
@@ -491,8 +604,6 @@ OSyncCapabilities *osync_version_find_capabilities(OSyncVersion *version, OSyncE
 	if(priority > 0)
 	{
 		osync_trace(TRACE_INTERNAL, "Found capabilities file by version: %s ", (const char*)osync_version_get_identifier(winner));
-		if (capabilities)
-			osync_capabilities_unref(capabilities);
 
 		capabilities = osync_capabilities_load((const char*)osync_version_get_identifier(winner), error);
 		osync_version_unref(winner);
