@@ -32,6 +32,7 @@ struct OSyncVersion{
 	int ref_count;
 	char *plugin;
 	char *priority;
+	char *vendor;
 	char *modelversion;
 	char *firmwareversion;
 	char *softwareversion;
@@ -39,6 +40,6 @@ struct OSyncVersion{
 	char *identifier;
 };
 
-int _osync_version_match(char *pattern, char* string, OSyncError **error);
+OSyncList *_osync_version_load_from_descriptions(OSyncError **error, const char *descriptiondir, const char *schemadir);
 
 #endif /*OPENSYNC_VERSION_INTERNALS_H_*/
