@@ -9,6 +9,7 @@
 #include <opensync/opensync-context.h>
 
 #include "opensync/engine/opensync_engine_internals.h"
+#include "opensync/group/opensync_member_internals.h"
 
 #include "../mock-plugin/mock_sync.h"
 #include "../mock-plugin/mock_format.h"
@@ -36,15 +37,15 @@ START_TEST (engine_new)
 	fail_unless(group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	OSyncMember *member = osync_member_new(&error);
-	fail_unless(member != NULL, NULL);
+	OSyncMember *member1 = osync_member_new(&error);
+	fail_unless(member1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_group_add_member(group, member);
+	osync_group_add_member(group, member1);
 	
-	member = osync_member_new(&error);
-	fail_unless(member != NULL, NULL);
+	OSyncMember *member2 = osync_member_new(&error);
+	fail_unless(member2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_group_add_member(group, member);
+	osync_group_add_member(group, member2);
 	
 	OSyncEngine *engine = osync_engine_new(group, &error);
 	fail_unless(engine != NULL, NULL);
@@ -54,6 +55,8 @@ START_TEST (engine_new)
 	osync_engine_unref(engine);
 	osync_engine_unref(engine);
 
+	osync_member_unref(member1);
+	osync_member_unref(member2);
 	osync_group_unref(group);
 	
 	destroy_testbed(testbed);
@@ -64,37 +67,43 @@ START_TEST (engine_init)
 {
 	char *path = NULL;
 	char *testbed = setup_testbed("sync_setup");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
+	char *plugindir = g_strdup_printf("%s/plugins",  testbed);
 	
 	OSyncError *error = NULL;
 	OSyncGroup *group = osync_group_new(&error);
 	fail_unless(group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	OSyncMember *member = osync_member_new(&error);
-	fail_unless(member != NULL, NULL);
+	OSyncMember *member1 = osync_member_new(&error);
+	fail_unless(member1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_group_add_member(group, member);
-	_member_add_format(member, "mockobjtype1", "mockformat1");
-	osync_member_set_pluginname(member, "mock-sync");
+	osync_group_add_member(group, member1);
+	_member_add_format(member1, "mockobjtype1", "mockformat1");
+	osync_member_set_pluginname(member1, "mock-sync");
 	path = g_strdup_printf("%s/configs/group/1", testbed);
-	osync_member_set_configdir(member, path);
+	osync_member_set_configdir(member1, path);
+	osync_member_set_schemadir(member1, testbed);
 	g_free(path);
 	
-	member = osync_member_new(&error);
-	fail_unless(member != NULL, NULL);
+	OSyncMember *member2 = osync_member_new(&error);
+	fail_unless(member2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_group_add_member(group, member);
-	_member_add_format(member, "mockobjtype1", "mockformat1");
-	osync_member_set_pluginname(member, "mock-sync");
+	osync_group_add_member(group, member2);
+	_member_add_format(member2, "mockobjtype1", "mockformat1");
+	osync_member_set_pluginname(member2, "mock-sync");
 	path = g_strdup_printf("%s/configs/group/1", testbed);
-	osync_member_set_configdir(member, path);
+	osync_member_set_configdir(member2, path);
+	osync_member_set_schemadir(member2, testbed);
 	g_free(path);
 	
 	OSyncEngine *engine = osync_engine_new(group, &error);
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_engine_set_plugindir(engine, testbed);
-	osync_engine_set_formatdir(engine, testbed);
+
+	osync_engine_set_schemadir(engine, testbed);
+	osync_engine_set_plugindir(engine, plugindir);
+	osync_engine_set_formatdir(engine, formatdir);
 
 	
 	fail_unless(osync_engine_initialize(engine, &error), NULL);
@@ -105,8 +114,14 @@ START_TEST (engine_init)
 	
 	osync_engine_unref(engine);
 
+	osync_member_unref(member1);
+	osync_member_unref(member2);
+
 	osync_group_unref(group);
 	
+	g_free(formatdir);
+	g_free(plugindir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
@@ -180,7 +195,8 @@ static void *initialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError *
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "mockobjtype1");
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockobjtype1", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	OSyncObjTypeSinkFunctions functions;
 	memset(&functions, 0, sizeof(functions));
@@ -304,14 +320,17 @@ static void _engine_instrument_pluginenv(OSyncEngine *engine, OSyncDebugGroup *d
 START_TEST (engine_sync)
 {
 	char *testbed = setup_testbed("sync_setup");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	
 	OSyncError *error = NULL;
 	OSyncDebugGroup *debug = _create_group(testbed);
 	
 	OSyncEngine *engine = osync_engine_new(debug->group, &error);
+	osync_engine_set_schemadir(engine, testbed);
+
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_formatdir(engine, formatdir);
 
 	_engine_instrument_pluginenv(engine, debug);
 	
@@ -328,6 +347,8 @@ START_TEST (engine_sync)
 	
 	osync_engine_unref(engine);
 	
+	g_free(formatdir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
@@ -458,7 +479,8 @@ static void *initialize_multi(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "mockformat1");
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockformat1", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	OSyncObjTypeSinkFunctions functions;
 	memset(&functions, 0, sizeof(functions));
@@ -474,7 +496,8 @@ static void *initialize_multi(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "vcard");
+	format_sink = osync_objformat_sink_new("vcard", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	memset(&functions, 0, sizeof(functions));
 	functions.connect = connect2;
@@ -489,7 +512,8 @@ static void *initialize_multi(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "plain");
+	format_sink = osync_objformat_sink_new("plain", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	memset(&functions, 0, sizeof(functions));
 	functions.connect = connect2;
@@ -606,6 +630,7 @@ static OSyncDebugGroup *_create_group2(char *testbed)
 START_TEST (engine_sync_multi_obj)
 {
 	char *testbed = setup_testbed("sync_setup");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	
 	OSyncError *error = NULL;
 	OSyncDebugGroup *debug = _create_group2(testbed);
@@ -613,7 +638,8 @@ START_TEST (engine_sync_multi_obj)
 	OSyncEngine *engine = osync_engine_new(debug->group, &error);
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 
 	_engine_instrument_pluginenv(engine, debug);
 	
@@ -630,7 +656,10 @@ START_TEST (engine_sync_multi_obj)
 	
 	osync_engine_unref(engine);
 	
+	g_free(formatdir);
+	
 	destroy_testbed(testbed);
+
 }
 END_TEST
 
@@ -840,7 +869,8 @@ static void *initialize_order(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "mockobjtype1");
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockobjtype1", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	OSyncObjTypeSinkFunctions functions;
 	memset(&functions, 0, sizeof(functions));
@@ -857,7 +887,8 @@ static void *initialize_order(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "vcard");
+	format_sink = osync_objformat_sink_new("vcard", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	memset(&functions, 0, sizeof(functions));
 	functions.connect = connect3;
@@ -873,7 +904,8 @@ static void *initialize_order(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "plain");
+	format_sink = osync_objformat_sink_new("plain", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	memset(&functions, 0, sizeof(functions));
 	functions.connect = connect3;
@@ -995,6 +1027,7 @@ static OSyncDebugGroup *_create_group3(char *testbed)
 START_TEST (engine_sync_out_of_order)
 {
 	char *testbed = setup_testbed("sync_setup");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	
 	OSyncError *error = NULL;
 	OSyncDebugGroup *debug = _create_group3(testbed);
@@ -1002,7 +1035,8 @@ START_TEST (engine_sync_out_of_order)
 	OSyncEngine *engine = osync_engine_new(debug->group, &error);
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 	
 	_engine_instrument_pluginenv(engine, debug);
 
@@ -1019,6 +1053,8 @@ START_TEST (engine_sync_out_of_order)
 	
 	osync_engine_unref(engine);
 	
+	g_free(formatdir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
@@ -1070,7 +1106,8 @@ static void *initialize_reuse(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "mockobjtype1");
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockobjtype1", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	OSyncObjTypeSinkFunctions functions;
 	memset(&functions, 0, sizeof(functions));
@@ -1087,7 +1124,8 @@ static void *initialize_reuse(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "vcard");
+	format_sink = osync_objformat_sink_new("vcard", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	memset(&functions, 0, sizeof(functions));
 	functions.connect = connect3;
@@ -1103,7 +1141,8 @@ static void *initialize_reuse(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "plain");
+	format_sink = osync_objformat_sink_new("plain", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	memset(&functions, 0, sizeof(functions));
 	functions.connect = connect3;
@@ -1216,6 +1255,7 @@ static OSyncDebugGroup *_create_group4(char *testbed)
 START_TEST (engine_sync_reuse)
 {
 	char *testbed = setup_testbed("sync_setup");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	
 	OSyncError *error = NULL;
 	OSyncDebugGroup *debug = _create_group4(testbed);
@@ -1223,7 +1263,8 @@ START_TEST (engine_sync_reuse)
 	OSyncEngine *engine = osync_engine_new(debug->group, &error);
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 
 	_engine_instrument_pluginenv(engine, debug);
 	
@@ -1246,6 +1287,8 @@ START_TEST (engine_sync_reuse)
 	
 	osync_engine_unref(engine);
 	
+	g_free(formatdir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
@@ -1256,6 +1299,7 @@ START_TEST (engine_sync_stress)
 	int i = 0;
 	
 	char *testbed = setup_testbed("sync_setup");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	
 	OSyncError *error = NULL;
 	OSyncDebugGroup *debug = _create_group4(testbed);
@@ -1263,7 +1307,8 @@ START_TEST (engine_sync_stress)
 	OSyncEngine *engine = osync_engine_new(debug->group, &error);
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 	
 	_engine_instrument_pluginenv(engine, debug);
 
@@ -1282,6 +1327,8 @@ START_TEST (engine_sync_stress)
 	
 	osync_engine_unref(engine);
 	
+	g_free(formatdir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
@@ -1380,7 +1427,8 @@ static void *initialize5(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "mockformat1");
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockformat1", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	OSyncObjTypeSinkFunctions functions;
 	memset(&functions, 0, sizeof(functions));
@@ -1484,6 +1532,7 @@ static OSyncDebugGroup *_create_group5(char *testbed)
 START_TEST (engine_sync_read_write)
 {
 	char *testbed = setup_testbed("sync_setup");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	
 	OSyncError *error = NULL;
 	OSyncDebugGroup *debug = _create_group5(testbed);
@@ -1491,7 +1540,8 @@ START_TEST (engine_sync_read_write)
 	OSyncEngine *engine = osync_engine_new(debug->group, &error);
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 
 	_engine_instrument_pluginenv(engine, debug);
 
@@ -1508,6 +1558,8 @@ START_TEST (engine_sync_read_write)
 	
 	osync_engine_unref(engine);
 	
+	g_free(formatdir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
@@ -1575,7 +1627,8 @@ static void *initialize6(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "mockformat1");
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockformat1", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	OSyncObjTypeSinkFunctions functions;
 	memset(&functions, 0, sizeof(functions));
@@ -1679,6 +1732,7 @@ static OSyncDebugGroup *_create_group6(char *testbed)
 START_TEST (engine_sync_read_write_stress)
 {
 	char *testbed = setup_testbed("sync_setup");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	
 	OSyncError *error = NULL;
 	OSyncDebugGroup *debug = _create_group6(testbed);
@@ -1686,7 +1740,8 @@ START_TEST (engine_sync_read_write_stress)
 	OSyncEngine *engine = osync_engine_new(debug->group, &error);
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 	
 	_engine_instrument_pluginenv(engine, debug);
 
@@ -1703,6 +1758,8 @@ START_TEST (engine_sync_read_write_stress)
 	
 	osync_engine_unref(engine);
 	
+	g_free(formatdir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
@@ -1770,7 +1827,8 @@ static void *initialize7(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError
 	if (!sink)
 		goto error;
 	
-	osync_objtype_sink_add_objformat(sink, "mockformat1");
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockformat1", error);
+	osync_objtype_sink_add_objformat_sink(sink, format_sink);
 	
 	OSyncObjTypeSinkFunctions functions;
 	memset(&functions, 0, sizeof(functions));
@@ -1875,6 +1933,7 @@ static OSyncDebugGroup *_create_group7(char *testbed)
 START_TEST (engine_sync_read_write_stress2)
 {
 	char *testbed = setup_testbed("sync_setup");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	
 	OSyncError *error = NULL;
 	OSyncDebugGroup *debug = _create_group7(testbed);
@@ -1882,7 +1941,8 @@ START_TEST (engine_sync_read_write_stress2)
 	OSyncEngine *engine = osync_engine_new(debug->group, &error);
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 
 	_engine_instrument_pluginenv(engine, debug);
 	
@@ -1899,6 +1959,8 @@ START_TEST (engine_sync_read_write_stress2)
 	
 	osync_engine_unref(engine);
 	
+	g_free(formatdir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
@@ -1931,6 +1993,8 @@ int main(void)
 {
 	int nf;
 
+	check_env();
+
 	Suite *s = engine_suite();
 	
 	SRunner *sr;
