@@ -115,7 +115,7 @@ START_TEST (conv_env_objformat_find_false)
 }
 END_TEST
 
-osync_bool convert_func(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool convert_func(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void* userdata, OSyncError **error)
 {
 	*free_input = TRUE;
 	*output = g_strdup("test");
@@ -392,7 +392,7 @@ START_TEST (conv_find_path)
 	fail_unless(error == NULL, NULL);
  
 
-	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format2, &error);
+	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format2, NULL, &error);
 	fail_unless(path != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -402,7 +402,7 @@ START_TEST (conv_find_path)
 	osync_converter_path_unref(path);
 	
 	// intended to fail
-	path = osync_format_env_find_path_with_detectors(env, data2, format1, &error);
+	path = osync_format_env_find_path_with_detectors(env, data2, format1, NULL, &error);
 	fail_unless(path == NULL, NULL);
 	fail_unless(error != NULL, NULL);
 	
@@ -458,7 +458,7 @@ START_TEST (conv_find_path2)
 	fail_unless(error == NULL, NULL);
  
 
-	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format3, &error);
+	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format3, NULL, &error);
 	fail_unless(path != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -520,7 +520,7 @@ START_TEST (conv_find_path_false)
 	fail_unless(error == NULL, NULL);
  
 	// intended to fail
-	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format3, &error);
+	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format3, NULL, &error);
 	fail_unless(path == NULL, NULL);
 	fail_unless(error != NULL, NULL);
 	
@@ -592,7 +592,7 @@ START_TEST (conv_find_multi_path)
 	fail_unless(data1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
  
-	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format4, &error);
+	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format4, NULL, &error);
 	fail_unless(path != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -614,6 +614,85 @@ START_TEST (conv_find_multi_path)
 }
 END_TEST
 
+START_TEST (conv_find_multi_path_with_preferred)
+{
+	char *testbed = setup_testbed(NULL);
+	
+	OSyncError *error = NULL;
+	OSyncFormatEnv *env = osync_format_env_new(&error);
+	fail_unless(env != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	OSyncObjFormat *format1 = osync_objformat_new("format1", "objtype", &error);
+	fail_unless(format1 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format1);
+	
+	OSyncObjFormat *format2 = osync_objformat_new("format2", "objtype", &error);
+	fail_unless(format2 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format2);
+	
+	OSyncObjFormat *format3 = osync_objformat_new("format3", "objtype", &error);
+	fail_unless(format3 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format3);
+	
+	OSyncObjFormat *format4 = osync_objformat_new("format4", "objtype", &error);
+	fail_unless(format4 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format4);
+	
+	OSyncFormatConverter *converter1 = osync_converter_new(OSYNC_CONVERTER_CONV, format1, format2, convert_func, &error);
+	fail_unless(converter1 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter1);
+	osync_converter_unref(converter1);
+	
+	OSyncFormatConverter *converter2 = osync_converter_new(OSYNC_CONVERTER_CONV, format2, format4, convert_func, &error);
+	fail_unless(converter2 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter2);
+	osync_converter_unref(converter2);
+	
+	OSyncFormatConverter *converter3 = osync_converter_new(OSYNC_CONVERTER_CONV, format1, format3, convert_func, &error);
+	fail_unless(converter3 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter3);
+	osync_converter_unref(converter3);
+	
+	OSyncFormatConverter *converter4 = osync_converter_new(OSYNC_CONVERTER_CONV, format3, format4, convert_func, &error);
+	fail_unless(converter4 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter4);
+	osync_converter_unref(converter4);
+
+	OSyncData *data1 = osync_data_new("data", 5, format1, &error);
+	fail_unless(data1 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+ 
+	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format4, "format3", &error);
+	fail_unless(path != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_converter_path_num_edges(path) == 2, NULL);
+	fail_unless(osync_converter_path_nth_edge(path, 0) == converter3, NULL);
+	fail_unless(osync_converter_path_nth_edge(path, 1) == converter4, NULL);
+	
+	osync_converter_path_unref(path);
+	
+	osync_format_env_free(env);
+	
+	osync_data_unref(data1);
+	osync_objformat_unref(format1);
+	osync_objformat_unref(format2);
+	osync_objformat_unref(format3);
+	osync_objformat_unref(format4);
+	
+	destroy_testbed(testbed);
+}
+END_TEST
+
 START_TEST (conv_find_circular_false)
 {
 	char *testbed = setup_testbed(NULL);
@@ -666,7 +745,7 @@ START_TEST (conv_find_circular_false)
 	fail_unless(error == NULL, NULL);
 
 	// intended to fail
-	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format4, &error);
+	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format4, NULL, &error);
 	fail_unless(path == NULL, NULL);
 	fail_unless(error != NULL, NULL);
 	
@@ -789,7 +868,7 @@ START_TEST (conv_find_complex)
 	fail_unless(data1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 
-	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format6, &error);
+	OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(env, data1, format6, NULL, &error);
 	fail_unless(path != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -852,21 +931,20 @@ START_TEST (conv_find_multi_target)
 	osync_format_env_register_converter(env, converter2);
 	osync_converter_unref(converter2);
 
-	OSyncObjFormat *targets[3];
-	targets[0] = format2;
-	targets[1] = format3;
-	targets[2] = NULL;
+	OSyncList *targets = NULL; 
+	targets = osync_list_prepend(targets, format2);
+	targets = osync_list_prepend(targets, format3);
 
 	OSyncData *data1 = osync_data_new("data", 5, format1, &error);
 	fail_unless(data1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 
-	OSyncFormatConverterPath *path = osync_format_env_find_path_formats_with_detectors(env, data1, targets, &error);
+	OSyncFormatConverterPath *path = osync_format_env_find_path_formats_with_detectors(env, data1, targets, NULL, &error);
 	fail_unless(path != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
 	fail_unless(osync_converter_path_num_edges(path) == 1, NULL);
-	fail_unless(osync_converter_path_nth_edge(path, 0) == converter2, NULL);
+	fail_unless(osync_converter_path_nth_edge(path, 0) == converter1, NULL);
 	
 	osync_converter_path_unref(path);
 	
@@ -928,16 +1006,15 @@ START_TEST (conv_find_multi_target2)
 	osync_format_env_register_converter(env, converter3);
 	osync_converter_unref(converter3);
 
-	OSyncObjFormat *targets[3];
-	targets[0] = format4;
-	targets[1] = format3;
-	targets[2] = NULL;
+	OSyncList *targets = NULL; 
+	targets = osync_list_prepend(targets, format4);
+	targets = osync_list_prepend(targets, format3);
 
 	OSyncData *data1 = osync_data_new("data", 5, format1, &error);
 	fail_unless(data1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 
-	OSyncFormatConverterPath *path = osync_format_env_find_path_formats_with_detectors(env, data1, targets, &error);
+	OSyncFormatConverterPath *path = osync_format_env_find_path_formats_with_detectors(env, data1, targets, NULL, &error);
 	fail_unless(path != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -958,7 +1035,197 @@ START_TEST (conv_find_multi_target2)
 }
 END_TEST
 
-static osync_bool convert_addtest(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+START_TEST (conv_find_multi_path_multi_target)
+{
+	char *testbed = setup_testbed(NULL);
+	
+	OSyncError *error = NULL;
+	OSyncFormatEnv *env = osync_format_env_new(&error);
+	fail_unless(env != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	OSyncObjFormat *format1 = osync_objformat_new("format1", "objtype", &error);
+	fail_unless(format1 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format1);
+	
+	OSyncObjFormat *format2 = osync_objformat_new("format2", "objtype", &error);
+	fail_unless(format2 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format2);
+	
+	OSyncObjFormat *format3 = osync_objformat_new("format3", "objtype", &error);
+	fail_unless(format3 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format3);
+	
+	OSyncObjFormat *format4 = osync_objformat_new("format4", "objtype", &error);
+	fail_unless(format4 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format4);
+
+	OSyncObjFormat *format5 = osync_objformat_new("format5", "objtype", &error);
+	fail_unless(format5 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format5);
+	
+	OSyncFormatConverter *converter1 = osync_converter_new(OSYNC_CONVERTER_CONV, format1, format2, convert_func, &error);
+	fail_unless(converter1 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter1);
+	osync_converter_unref(converter1);
+	
+	OSyncFormatConverter *converter2 = osync_converter_new(OSYNC_CONVERTER_CONV, format2, format4, convert_func, &error);
+	fail_unless(converter2 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter2);
+	osync_converter_unref(converter2);
+	
+	OSyncFormatConverter *converter3 = osync_converter_new(OSYNC_CONVERTER_CONV, format1, format3, convert_func, &error);
+	fail_unless(converter3 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter3);
+	osync_converter_unref(converter3);
+	
+	OSyncFormatConverter *converter4 = osync_converter_new(OSYNC_CONVERTER_CONV, format3, format4, convert_func, &error);
+	fail_unless(converter4 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter4);
+	osync_converter_unref(converter4);
+
+	OSyncFormatConverter *converter5 = osync_converter_new(OSYNC_CONVERTER_CONV, format3, format5, convert_func, &error);
+	fail_unless(converter5 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter5);
+	osync_converter_unref(converter5);
+
+	OSyncList *targets = NULL; 
+	targets = osync_list_prepend(targets, format4);
+	targets = osync_list_prepend(targets, format5);
+
+	OSyncData *data1 = osync_data_new("data", 5, format1, &error);
+	fail_unless(data1 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+ 
+	OSyncFormatConverterPath *path = osync_format_env_find_path_formats_with_detectors(env, data1, targets, NULL, &error);
+	fail_unless(path != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_converter_path_num_edges(path) == 2, NULL);
+	fail_unless(osync_converter_path_nth_edge(path, 0) == converter1, NULL);
+	fail_unless(osync_converter_path_nth_edge(path, 1) == converter2, NULL);
+	
+	osync_converter_path_unref(path);
+	
+	osync_format_env_free(env);
+	
+	osync_data_unref(data1);
+	osync_objformat_unref(format1);
+	osync_objformat_unref(format2);
+	osync_objformat_unref(format3);
+	osync_objformat_unref(format4);
+	osync_objformat_unref(format5);
+	
+	destroy_testbed(testbed);
+}
+END_TEST
+
+START_TEST (conv_find_multi_path_multi_target_with_preferred)
+{
+	char *testbed = setup_testbed(NULL);
+	
+	OSyncError *error = NULL;
+	OSyncFormatEnv *env = osync_format_env_new(&error);
+	fail_unless(env != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	OSyncObjFormat *format1 = osync_objformat_new("format1", "objtype", &error);
+	fail_unless(format1 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format1);
+	
+	OSyncObjFormat *format2 = osync_objformat_new("format2", "objtype", &error);
+	fail_unless(format2 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format2);
+	
+	OSyncObjFormat *format3 = osync_objformat_new("format3", "objtype", &error);
+	fail_unless(format3 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format3);
+	
+	OSyncObjFormat *format4 = osync_objformat_new("format4", "objtype", &error);
+	fail_unless(format4 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format4);
+
+	OSyncObjFormat *format5 = osync_objformat_new("format5", "objtype", &error);
+	fail_unless(format5 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_objformat(env, format5);
+	
+	OSyncFormatConverter *converter1 = osync_converter_new(OSYNC_CONVERTER_CONV, format1, format2, convert_func, &error);
+	fail_unless(converter1 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter1);
+	osync_converter_unref(converter1);
+	
+	OSyncFormatConverter *converter2 = osync_converter_new(OSYNC_CONVERTER_CONV, format2, format4, convert_func, &error);
+	fail_unless(converter2 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter2);
+	osync_converter_unref(converter2);
+	
+	OSyncFormatConverter *converter3 = osync_converter_new(OSYNC_CONVERTER_CONV, format1, format3, convert_func, &error);
+	fail_unless(converter3 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter3);
+	osync_converter_unref(converter3);
+	
+	OSyncFormatConverter *converter4 = osync_converter_new(OSYNC_CONVERTER_CONV, format3, format4, convert_func, &error);
+	fail_unless(converter4 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter4);
+	osync_converter_unref(converter4);
+
+	OSyncFormatConverter *converter5 = osync_converter_new(OSYNC_CONVERTER_CONV, format3, format5, convert_func, &error);
+	fail_unless(converter5 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_format_env_register_converter(env, converter5);
+	osync_converter_unref(converter5);
+
+	OSyncList *targets = NULL; 
+	targets = osync_list_prepend(targets, format4);
+	targets = osync_list_prepend(targets, format5);
+
+	OSyncData *data1 = osync_data_new("data", 5, format1, &error);
+	fail_unless(data1 != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+ 
+	OSyncFormatConverterPath *path = osync_format_env_find_path_formats_with_detectors(env, data1, targets, "format5", &error);
+	fail_unless(path != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_converter_path_num_edges(path) == 2, NULL);
+	fail_unless(osync_converter_path_nth_edge(path, 0) == converter3, NULL);
+	fail_unless(osync_converter_path_nth_edge(path, 1) == converter5, NULL);
+	
+	osync_converter_path_unref(path);
+	
+	osync_format_env_free(env);
+	
+	osync_data_unref(data1);
+	osync_objformat_unref(format1);
+	osync_objformat_unref(format2);
+	osync_objformat_unref(format3);
+	osync_objformat_unref(format4);
+	osync_objformat_unref(format5);
+	
+	destroy_testbed(testbed);
+}
+END_TEST
+
+static osync_bool convert_addtest(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	*free_input = TRUE;
 	*output = g_strdup_printf("%stest", input);
@@ -966,7 +1233,7 @@ static osync_bool convert_addtest(char *input, unsigned int inpsize, char **outp
 	return TRUE;
 }
 
-static osync_bool convert_remtest(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+static osync_bool convert_remtest(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	*free_input = TRUE;
 	*output = strdup(input);
@@ -982,7 +1249,7 @@ static osync_bool convert_remtest(char *input, unsigned int inpsize, char **outp
 	}
 }
 
-static osync_bool convert_addtest2(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+static osync_bool convert_addtest2(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	*output = g_strdup_printf("%stest2", input);
 	*outpsize = inpsize + 5;
@@ -990,7 +1257,7 @@ static osync_bool convert_addtest2(char *input, unsigned int inpsize, char **out
 	return TRUE;
 }
 
-static osync_bool convert_remtest2(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+static osync_bool convert_remtest2(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	*free_input = TRUE;
 	*output = strdup(input);
@@ -1049,7 +1316,7 @@ START_TEST (conv_env_convert1)
 	mark_point();
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format3, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format3, NULL, &error);
 	osync_format_env_convert(env, path, data, &error); 
 	fail_unless(error == NULL, NULL);
 
@@ -1119,7 +1386,7 @@ START_TEST (conv_env_convert_back)
 	mark_point();
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format3, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format3, NULL, &error);
 	osync_format_env_convert(env, path, data, &error); 
 	fail_unless(error == NULL, NULL);
 
@@ -1136,7 +1403,7 @@ START_TEST (conv_env_convert_back)
 	mark_point();
 
 	path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format1, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format1, NULL, &error);
 	osync_format_env_convert(env, path, data, &error); 
 	fail_unless(error == NULL, NULL);
 	fail_unless(path != NULL, NULL);
@@ -1206,7 +1473,7 @@ START_TEST (conv_env_convert_desenc)
 	mark_point();
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format3, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format3, NULL, &error);
 	osync_format_env_convert(env, path, data, &error); 
 	fail_unless(error == NULL, NULL);
 
@@ -1223,7 +1490,7 @@ START_TEST (conv_env_convert_desenc)
 	mark_point();
 
 	path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format1, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format1, NULL, &error);
 	osync_format_env_convert(env, path, data, &error); 
 	fail_unless(error == NULL, NULL);
 	fail_unless(path != NULL, NULL);
@@ -1237,12 +1504,12 @@ START_TEST (conv_env_convert_desenc)
 }
 END_TEST
 
-static osync_bool detect_true(const char *data, int size)
+static osync_bool detect_true(const char *data, int size, void *userdata)
 {
 	return TRUE;
 }
 
-static osync_bool detect_false(const char *data, int size)
+static osync_bool detect_false(const char *data, int size, void *userdata)
 {
 	return FALSE;
 }
@@ -1346,7 +1613,7 @@ START_TEST (conv_env_convert_desenc_complex)
 
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format6, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format6, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	osync_format_env_convert(env, path, data, &error); 
@@ -1363,7 +1630,7 @@ START_TEST (conv_env_convert_desenc_complex)
 
 	mark_point();
 	path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format1, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format1, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	osync_format_env_convert(env, path, data, &error); 
@@ -1467,7 +1734,7 @@ START_TEST (conv_env_detect_and_convert)
 	mark_point();
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format4, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format4, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	osync_format_env_convert(env, path, data, &error);
@@ -1484,7 +1751,7 @@ START_TEST (conv_env_detect_and_convert)
 
 	mark_point();
 
-	path = osync_format_env_find_path_with_detectors(env, data, format1, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format1, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	osync_format_env_convert(env, path, data, &error);
@@ -1584,7 +1851,7 @@ START_TEST(conv_prefer_not_desencap)
 	mark_point();
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format5, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format5, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	osync_format_env_convert(env, path, data, &error);
@@ -1679,13 +1946,12 @@ START_TEST(conv_prefer_same_objtype)
 
 	mark_point();
 
-	OSyncObjFormat **fmtlist = osync_try_malloc0(sizeof(OSyncObjFormat*) * 3, &error);
-	fmtlist[0] = format_g1; 
-	fmtlist[1] = format6; 
-	fmtlist[2] = NULL;
+	OSyncList* fmtlist = NULL;
+	fmtlist = osync_list_prepend(fmtlist, format_g1);
+	fmtlist = osync_list_prepend(fmtlist, format6);
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_formats_with_detectors(env, data, fmtlist, &error);
+	path = osync_format_env_find_path_formats_with_detectors(env, data, fmtlist, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	osync_format_env_convert(env, path, data, &error);
@@ -1782,13 +2048,12 @@ START_TEST(conv_prefer_not_lossy_objtype_change)
 
 	mark_point();
 
-	OSyncObjFormat **fmtlist = osync_try_malloc0(sizeof(OSyncObjFormat*) * 3, &error);
-	fmtlist[0] = format_g1; 
-	fmtlist[1] = format6; 
-	fmtlist[2] = NULL;
+	OSyncList* fmtlist = NULL;
+	fmtlist = osync_list_prepend(fmtlist, format_g1);
+	fmtlist = osync_list_prepend(fmtlist, format6);
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_formats_with_detectors(env, data, fmtlist, &error);
+	path = osync_format_env_find_path_formats_with_detectors(env, data, fmtlist, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	osync_format_env_convert(env, path, data, &error);
@@ -1865,14 +2130,14 @@ START_TEST (conv_env_detect_false)
 	mark_point();
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format3, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format3, NULL, &error);
 	fail_unless(path == NULL, NULL);
 
 	//fail_unless(!osync_format_env_convert(env, path, data, &error), NULL); // path is supposed to be null and this function has an assert on path
 }
 END_TEST
 
-static osync_bool detect_plain_as_f2(const char *data, int size)
+static osync_bool detect_plain_as_f2(const char *data, int size, void *userdata)
 {
 	return TRUE;
 }
@@ -1931,7 +2196,7 @@ START_TEST (conv_env_decap_and_detect)
 	mark_point();
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format3, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format3, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	char *buf;
@@ -1951,7 +2216,7 @@ START_TEST (conv_env_decap_and_detect)
 
 	mark_point();
 
-	path = osync_format_env_find_path_with_detectors(env, data, format1, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format1, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	osync_format_env_convert(env, path, data, &error);
@@ -1968,14 +2233,14 @@ START_TEST (conv_env_decap_and_detect)
 }
 END_TEST
 
-static osync_bool detect_f2(const char *data, int size)
+static osync_bool detect_f2(const char *data, int size, void *userdata)
 {
 	if (!strcmp(data, "F2"))
 		return TRUE;
 	return FALSE;
 }
 
-static osync_bool convert_f1_to_f2(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+static osync_bool convert_f1_to_f2(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	fail_unless(!strcmp(input, "F1"), NULL);
 	
@@ -1985,7 +2250,7 @@ static osync_bool convert_f1_to_f2(char *input, unsigned int inpsize, char **out
 	return TRUE;
 }
 
-static osync_bool convert_f2_to_f1(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+static osync_bool convert_f2_to_f1(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	fail_unless(!strcmp(input, "F2"), NULL);
 	
@@ -2052,7 +2317,7 @@ START_TEST (conv_env_decap_and_detect2)
 	mark_point();
 
 	OSyncFormatConverterPath *path = NULL;
-	path = osync_format_env_find_path_with_detectors(env, data, format3, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format3, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	char *buf;
@@ -2072,7 +2337,7 @@ START_TEST (conv_env_decap_and_detect2)
 
 	mark_point();
 
-	path = osync_format_env_find_path_with_detectors(env, data, format1, &error);
+	path = osync_format_env_find_path_with_detectors(env, data, format1, NULL, &error);
 	fail_unless(path != NULL, NULL);
 
 	osync_format_env_convert(env, path, data, &error);
@@ -2114,13 +2379,14 @@ Suite *format_env_suite(void)
 	create_case(s, "conv_find_path2", conv_find_path2);
 	create_case(s, "conv_find_path_false", conv_find_path_false);
 	create_case(s, "conv_find_multi_path", conv_find_multi_path);
-	
+	create_case(s, "conv_find_multi_path_with_preferred", conv_find_multi_path_with_preferred);
 	create_case(s, "conv_find_circular_false", conv_find_circular_false);
 	create_case(s, "conv_find_complex", conv_find_complex);
 	
 	create_case(s, "conv_find_multi_target", conv_find_multi_target);
 	create_case(s, "conv_find_multi_target2", conv_find_multi_target2);
-	
+	create_case(s, "conv_find_multi_path_multi_target", conv_find_multi_path_multi_target);
+	create_case(s, "conv_find_multi_path_multi_target_with_preferred", conv_find_multi_path_multi_target_with_preferred);
 
 	// Gone?
 	//create_case(s, "conv_env_osp_circular_false", conv_env_osp_circular_false);
@@ -2138,7 +2404,7 @@ Suite *format_env_suite(void)
 	create_case(s, "conv_env_detect_false", conv_env_detect_false);
 	create_case(s, "conv_env_decap_and_detect", conv_env_decap_and_detect);
 	create_case(s, "conv_env_decap_and_detect2", conv_env_decap_and_detect2);
-	
+
 	/*
 	 * osync_bool osync_format_env_load_plugins(OSyncFormatEnv *env, const char *path, OSyncError **error);
 
