@@ -116,7 +116,7 @@ START_TEST (conv_env_objformat_find_false)
 }
 END_TEST
 
-osync_bool convert_func(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool convert_func(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return TRUE;
 }
@@ -369,7 +369,7 @@ START_TEST (conv_env_plugin)
 	fail_unless(error == NULL, NULL);
 	
 	char *curdir = g_get_current_dir();
-	char *path = g_strdup_printf("%s/mock-format.so", curdir);
+	char *path = g_strdup_printf("%s/formats/mock-format.%s", curdir, G_MODULE_SUFFIX);
 	fail_unless(osync_module_load(module, path, &error), NULL);
 	fail_unless(error == NULL, NULL);
 	g_free(path);
