@@ -3,13 +3,15 @@
 #include <opensync/opensync.h>
 #include <opensync/opensync-group.h>
 #include <opensync/opensync_internals.h>
-#include <opensync/engine/opensync_engine_internals.h>
+#include "opensync/engine/opensync_engine_internals.h"
+#include "opensync/group/opensync_group_internals.h"
 
 START_TEST (simple_lock)
 {
 	char *testbed = setup_testbed("multisync_easy_new");
 	
 	OSyncGroup *group = osync_group_new(NULL);
+	osync_group_set_schemadir(group, testbed);
 	osync_group_load(group, "configs/group", NULL);
 	
 	fail_unless(osync_group_lock(group) == OSYNC_LOCK_OK, NULL);
@@ -27,6 +29,7 @@ START_TEST (simple_seq_lock)
 	char *testbed = setup_testbed("multisync_easy_new");
 	
 	OSyncGroup *group = osync_group_new(NULL);
+	osync_group_set_schemadir(group, testbed);
 	osync_group_load(group, "configs/group", NULL);
 	
 	fail_unless(osync_group_lock(group) == OSYNC_LOCK_OK, NULL);
@@ -47,6 +50,7 @@ START_TEST (dual_lock)
 	char *testbed = setup_testbed("multisync_easy_new");
 	
 	OSyncGroup *group = osync_group_new(NULL);
+	osync_group_set_schemadir(group, testbed);
 	osync_group_load(group, "configs/group", NULL);
 	
 	fail_unless(osync_group_lock(group) == OSYNC_LOCK_OK, NULL);
@@ -64,8 +68,10 @@ START_TEST (dual_lock2)
 	char *testbed = setup_testbed("multisync_easy_new");
 	
 	OSyncGroup *group = osync_group_new(NULL);
+	osync_group_set_schemadir(group, testbed);
 	osync_group_load(group, "configs/group", NULL);
 	OSyncGroup *group2 = osync_group_new(NULL);
+	osync_group_set_schemadir(group, testbed);
 	osync_group_load(group2, "configs/group", NULL);
 	
 	fail_unless(osync_group_lock(group) == OSYNC_LOCK_OK, NULL);
@@ -82,10 +88,14 @@ END_TEST
 START_TEST (dual_sync_engine_lock)
 {
 	char *testbed = setup_testbed("multisync_easy_new");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
+	char *plugindir = g_strdup_printf("%s/plugins",  testbed);
 	
 	OSyncGroup *group = osync_group_new(NULL);
+	osync_group_set_schemadir(group, testbed);
 	osync_group_load(group, "configs/group", NULL);
 	OSyncGroup *group2 = osync_group_new(NULL);
+	osync_group_set_schemadir(group2, testbed);
 	osync_group_load(group2, "configs/group", NULL);
 	
 	OSyncError *error = NULL;
@@ -94,15 +104,17 @@ START_TEST (dual_sync_engine_lock)
 	fail_unless(error == NULL, osync_error_print(&error));
 	osync_engine_set_enginestatus_callback(engine, engine_status, NULL);
 
-	osync_engine_set_plugindir(engine, testbed);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_plugindir(engine, plugindir);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 
 	OSyncEngine *engine2 = osync_engine_new(group2, &error);
 	fail_unless(error == NULL, osync_error_print(&error));
 	osync_engine_set_enginestatus_callback(engine2, engine_status, NULL);
 
-	osync_engine_set_plugindir(engine2, testbed);
-	osync_engine_set_formatdir(engine2, testbed);
+	osync_engine_set_plugindir(engine2, plugindir);
+	osync_engine_set_formatdir(engine2, formatdir);
+	osync_engine_set_schemadir(engine2, testbed);
 
 	fail_unless(osync_engine_initialize(engine, &error), osync_error_print(&error));
 	fail_unless(!osync_engine_initialize(engine2, &error), osync_error_print(&error));
@@ -127,11 +139,15 @@ START_TEST (dual_sync_engine_lock)
 	osync_engine_unref(engine);
 	osync_engine_unref(engine2);
 	
-	fail_unless(!system("test \"x$(diff -x \".*\" data1 data2)\" == \"x\""), NULL);
-	fail_unless(!system("test \"x$(diff -x \".*\" data1 data3)\" == \"x\""), NULL);
+	fail_unless(!osync_testing_diff("data1", "data2"));
+	fail_unless(!osync_testing_diff("data1", "data3"));
 	
 	osync_group_unref(group);
 	osync_group_unref(group2);
+	
+	g_free(formatdir);
+	g_free(plugindir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
@@ -139,8 +155,11 @@ END_TEST
 START_TEST (dual_sync_engine_unclean)
 {
 	char *testbed = setup_testbed("multisync_easy_new");
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
+	char *plugindir = g_strdup_printf("%s/plugins",  testbed);
 	
 	OSyncGroup *group = osync_group_new(NULL);
+	osync_group_set_schemadir(group, testbed);
 	osync_group_load(group, "configs/group", NULL);
 	
 	OSyncError *error = NULL;
@@ -148,8 +167,9 @@ START_TEST (dual_sync_engine_unclean)
 	fail_unless(error == NULL, osync_error_print(&error));
 	osync_engine_set_enginestatus_callback(engine, engine_status, NULL);
 	
-	osync_engine_set_plugindir(engine, testbed);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_plugindir(engine, plugindir);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 
 	fail_unless(osync_engine_initialize(engine, &error), NULL);
 	fail_unless(error == NULL, osync_error_print(&error));
@@ -186,12 +206,14 @@ START_TEST (dual_sync_engine_unclean)
 
 
 	group = osync_group_new(NULL);
+	osync_group_set_schemadir(group, testbed);
 	osync_group_load(group, "configs/group", NULL);
 	engine = osync_engine_new(group, &error);
 	osync_engine_set_enginestatus_callback(engine, engine_status, NULL);
 
-	osync_engine_set_plugindir(engine, testbed);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_plugindir(engine, plugindir);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 	
 	num_engine_prev_unclean = 0;
 	fail_unless(osync_engine_initialize(engine, &error), NULL);
@@ -210,12 +232,14 @@ START_TEST (dual_sync_engine_unclean)
 	osync_group_unref(group);
 	
 	group = osync_group_new(NULL);
+	osync_group_set_schemadir(group, testbed);
 	osync_group_load(group, "configs/group", NULL);
 	engine = osync_engine_new(group, &error);
 	osync_engine_set_enginestatus_callback(engine, engine_status, NULL);
 	
-	osync_engine_set_plugindir(engine, testbed);
-	osync_engine_set_formatdir(engine, testbed);
+	osync_engine_set_plugindir(engine, plugindir);
+	osync_engine_set_formatdir(engine, formatdir);
+	osync_engine_set_schemadir(engine, testbed);
 	
 	for (o = engine->object_engines; o; o = o->next) {
 		OSyncObjEngine *objengine = o->data;
@@ -236,11 +260,14 @@ START_TEST (dual_sync_engine_unclean)
 	
 	fail_unless(num_engine_prev_unclean == 0, NULL);
 	
-	fail_unless(!system("test \"x$(diff -x \".*\" data1 data2)\" == \"x\""), NULL);
-	fail_unless(!system("test \"x$(diff -x \".*\" data1 data3)\" == \"x\""), NULL);
+	fail_unless(!osync_testing_diff("data1", "data2"));
+	fail_unless(!osync_testing_diff("data1", "data3"));
 
 	osync_group_unref(group);
 	
+	g_free(formatdir);
+	g_free(plugindir);
+	
 	destroy_testbed(testbed);
 }
 END_TEST
