@@ -1,6 +1,8 @@
 #include "support.h"
 
-#include <opensync/opensync-merger.h>
+#include <opensync/opensync-xmlformat.h>
+
+#include "opensync/xmlformat/opensync-xmlformat_internals.h"
 
 START_TEST (xmlformat_new)
 {
@@ -64,6 +66,34 @@ START_TEST (xmlformat_sort)
 }
 END_TEST
 
+START_TEST (xmlformat_is_sorted)
+{
+	char *testbed = setup_testbed("xmlformats");
+	
+	OSyncError *error = NULL;
+	char* buffer;
+	unsigned int size;
+	fail_unless(osync_file_read("xmlfield_unsorted.xml", &buffer, &size, &error), NULL);
+
+	OSyncXMLFormat *xmlformat = osync_xmlformat_parse(buffer, size, &error);
+	fail_unless(xmlformat != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_xmlformat_is_sorted(xmlformat) == FALSE, NULL);
+
+	osync_xmlformat_sort(xmlformat);
+
+	fail_unless(osync_xmlformat_is_sorted(xmlformat) == TRUE, NULL);
+	
+	osync_xmlformat_unref(xmlformat);
+
+	g_free(buffer);
+
+	destroy_testbed(testbed);
+}
+END_TEST
+
+
 START_TEST (xmlformat_search_field)
 {
 	char *testbed = setup_testbed("merger");
@@ -79,8 +109,9 @@ START_TEST (xmlformat_search_field)
 	g_free(buffer);
 	osync_xmlformat_sort(xmlformat);
 	
-	OSyncXMLFieldList *xmlfieldlist = osync_xmlformat_search_field(xmlformat, "Name", NULL);
+	OSyncXMLFieldList *xmlfieldlist = osync_xmlformat_search_field(xmlformat, "Name", &error, NULL);
 	fail_unless(xmlfieldlist != NULL, NULL);
+	fail_unless(error == NULL, NULL);
 	
 	fail_unless(size != osync_xmlfieldlist_get_length(xmlfieldlist), NULL);
 
@@ -272,6 +303,7 @@ START_TEST (xmlformat_event_schema)
 	char *buffer;
 	unsigned int size;
 	OSyncError *error = NULL;
+	OSyncXMLFormatSchema *schema = NULL;
 
 	fail_unless(osync_file_read("event.xml", &buffer, &size, &error), NULL);
 	fail_unless(error == NULL, NULL);
@@ -280,9 +312,12 @@ START_TEST (xmlformat_event_schema)
 	fail_unless(error == NULL, NULL);
 
 	g_free(buffer);
-
-	fail_unless(_osync_xmlformat_validate(xmlformat, testbed) != FALSE, NULL);
-
+	schema = osync_xmlformat_schema_new(xmlformat, testbed, &error);
+	fail_if( schema == NULL );
+	fail_unless( osync_xmlformat_schema_validate(schema, xmlformat, &error) );
+	
+	osync_xmlformat_schema_unref(schema);
+	
 	osync_xmlformat_unref(xmlformat);
 
 	destroy_testbed(testbed);
@@ -361,20 +396,57 @@ START_TEST (xmlfield_sort)
 }
 END_TEST
 
+START_TEST (xmlformat_schema_get_instance)
+{
+	char *testbed = setup_testbed("xmlformats");	
+
+	OSyncError *error = NULL;
+
+	//TODO disable libxml2 output to stderr
+
+	OSyncXMLFormat *xmlformat = osync_xmlformat_new("abc", &error);
+	OSyncXMLFormatSchema *failschema = osync_xmlformat_schema_get_instance_with_path(xmlformat, testbed, &error);
+	fail_unless(failschema == NULL);
+	osync_xmlformat_unref(xmlformat);
+
+	xmlformat = osync_xmlformat_new("contact", &error);
+	fail_if(xmlformat == NULL, NULL);
+	fail_if(error == NULL, NULL);
+	
+	OSyncXMLFormatSchema *schema1 = osync_xmlformat_schema_get_instance_with_path(xmlformat, testbed, &error);
+	OSyncXMLFormatSchema *schema2 = osync_xmlformat_schema_get_instance_with_path(xmlformat, testbed, &error);
+	fail_if(schema1 == NULL);
+	fail_if(schema2 == NULL);
+	fail_unless( schema1 == schema2 );
+	fail_unless( schema1->ref_count == 2 );
+
+	osync_xmlformat_schema_unref(schema1);
+	osync_xmlformat_schema_unref(schema2);
+
+	destroy_testbed(testbed);
+
+}
+END_TEST
+
 Suite *xmlformat_suite(void)
 {
 	Suite *s = suite_create("XMLFormat");
+//	Suite *s2 = suite_create("XMLFormat");
 
 	// xmlformat
 	create_case(s, "xmlformat_new", xmlformat_new);
 	create_case(s, "xmlformat_parse", xmlformat_parse);
 	create_case(s, "xmlformat_sort", xmlformat_sort);
+	create_case(s, "xmlformat_is_sorted", xmlformat_is_sorted);
 	create_case(s, "xmlformat_search_field", xmlformat_search_field);
 	create_case(s, "xmlformat_compare", xmlformat_compare);
 	create_case(s, "xmlformat_compare_field2null", xmlformat_compare_field2null);
 	create_case(s, "xmlformat_compare_ignore_fields", xmlformat_compare_ignore_fields);
 	create_case(s, "xmlformat_event_schema", xmlformat_event_schema);
 
+	// xmlformat schema
+	create_case(s, "xmlformat_schema_get_instance", xmlformat_schema_get_instance);
+
 	// xmlfield
 	create_case(s, "xmlfield_new", xmlfield_new);
 	create_case(s, "xmlfield_sort", xmlfield_sort);
