@@ -43,145 +43,16 @@ static osync_bool mock_get_error(long long int memberid, const char *domain)
         return FALSE;
 }
 
-static void free_dir(OSyncFileDir *dir)
-{
-	if (dir->path)
-		g_free(dir->path);
-		
-	if (dir->objtype)
-		g_free(dir->objtype);
-	
-	g_free(dir);
-}
-
-static void free_env(mock_env *env)
-{
-	while (env->directories) {
-		OSyncFileDir *dir = env->directories->data;
-		
-		if (dir->sink)
-			osync_objtype_sink_unref(dir->sink);
-		
-		free_dir(dir);
-		env->directories = g_list_remove(env->directories, dir);
-	}
-	
-	g_free(env);
-}
-
-static osync_bool mock_parse_directory(mock_env *env, OSyncPluginInfo *info, xmlNode *cur, OSyncError **error)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, env, info, cur, error);
-
-	OSyncFileDir *dir = osync_try_malloc0(sizeof(OSyncFileDir), error);
-	if (!dir)
-		goto error;
-	dir->env = env;
-
-	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
-	
-	while (cur != NULL) {
-		char *str = (char*)xmlNodeGetContent(cur);
-		if (str) {
-			if (!xmlStrcmp(cur->name, (const xmlChar *)"path")) {
-				dir->path = g_strdup(str);
-			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objtype")) {
-				dir->objtype = g_strdup(str);
-			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objformat")) {
-				dir->objformat = osync_format_env_find_objformat(formatenv, str);
-			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"recursive")) {
-				dir->recursive = (g_ascii_strcasecmp(str, "TRUE") == 0);
-			}
-			xmlFree(str);
-		}
-		cur = cur->next;
-	}
-
-	if (!dir->path) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Path not set");
-		goto error_free_dir;
-	}
-
-	osync_trace(TRACE_INTERNAL, "Got directory %s with objtype %s", dir->path, dir->objtype);
-
-	env->directories = g_list_append(env->directories, dir);
-
-	osync_trace(TRACE_EXIT, "%s", __func__);
-	return TRUE;
-
-error_free_dir:
-	free_dir(dir);
-error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return FALSE;
-}
-
-/*Load the state from a xml file and return it in the conn struct*/
-static osync_bool mock_parse_settings(mock_env *env, OSyncPluginInfo *info, OSyncError **error)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, env, info, error);
-	const char *data = osync_plugin_info_get_config(info);
-	xmlDoc *doc = NULL;
-	xmlNode *cur = NULL;
-
-	doc = xmlParseMemory(data, strlen(data) + 1);
-	if (!doc) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to parse settings");
-		goto error;
-	}
-
-	cur = xmlDocGetRootElement(doc);
-
-	if (!cur) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get root element of the settings");
-		goto error_free_doc;
-	}
-
-	if (xmlStrcmp(cur->name, (xmlChar*)"config")) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Config valid is not valid");
-		goto error_free_doc;
-	}
-
-	cur = cur->xmlChildrenNode;
-
-	while (cur != NULL) {
-		char *str = (char*)xmlNodeGetContent(cur);
-		if (str) {
-			if (!xmlStrcmp(cur->name, (const xmlChar *)"directory")) {
-				if (!mock_parse_directory(env, info, cur->xmlChildrenNode, error))
-					goto error_free_doc;
-			}
-			xmlFree(str);
-		}
-		cur = cur->next;
-	}
-
-	xmlFreeDoc(doc);
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-	return TRUE;
-
-error_free_doc:
-	xmlFreeDoc(doc);
-error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return FALSE;
-}
-
 static char *mock_generate_hash(struct stat *buf)
 {
-	char *hash = g_strdup_printf("%i-%i", (int)buf->st_mtime, (int)buf->st_ctime);
-	return hash;
+	return g_strdup_printf("%i-%i", (int)buf->st_mtime, (int)buf->st_ctime);
 }
 
 static void mock_connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
-	OSyncError *error = NULL;
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+	MockDir *dir = osync_objtype_sink_get_userdata(sink);
 
-        dir->committed_all = TRUE;
-	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
 	if (mock_get_error(info->memberid, "CONNECT_ERROR")) {
@@ -196,46 +67,59 @@ static void mock_connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 		return;
 	}
 
-	osync_trace(TRACE_INTERNAL, "The configdir: %s", osync_plugin_info_get_configdir(info));
-	char *tablepath = g_strdup_printf("%s/hashtable.db", osync_plugin_info_get_configdir(info));
-	dir->hashtable = osync_hashtable_new(tablepath, osync_objtype_sink_get_name(sink), &error);
-	g_free(tablepath);
-	
-	if (!dir->hashtable)
-		goto error;
+	if (mock_get_error(info->memberid, "CONNECT_SLOWSYNC"))
+		osync_objtype_sink_set_slowsync(sink, TRUE);
+
+	/* Skip Objtype related stuff like hashtable and anchor */
+	if (mock_get_error(info->memberid, "MAINSINK_CONNECT"))
+		goto end;
+
+
+	/* From this line MockDir *dir is required, for ObjTypeSink specific stuff: #821 */
+	osync_assert(dir);
+	dir->committed_all = TRUE;
 
 	char *anchorpath = g_strdup_printf("%s/anchor.db", osync_plugin_info_get_configdir(info));
 	char *path_field = g_strdup_printf("path_%s", osync_objtype_sink_get_name(sink));
 	if (!osync_anchor_compare(anchorpath, path_field, dir->path))
-		osync_objtype_sink_set_slowsync(dir->sink, TRUE);
+		osync_objtype_sink_set_slowsync(sink, TRUE);
+
 	g_free(anchorpath);
 	g_free(path_field);
 	
-	if (!g_file_test(dir->path, G_FILE_TEST_IS_DIR)) {
-		osync_error_set(&error, OSYNC_ERROR_GENERIC, "%s is not a directory", dir->path);
-		goto error;
-	}
-	
+	osync_assert(g_file_test(dir->path, G_FILE_TEST_IS_DIR));
+
+end:	
 	osync_context_report_success(ctx);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return;
-	
-error:
-	osync_context_report_osyncerror(ctx, error);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
-	osync_error_unref(&error);
 }
 
 static void mock_disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
-	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
 
-	if (!g_getenv("NO_COMMITTED_ALL_CHECK"))
-		fail_unless(dir->committed_all == TRUE, NULL);
-	dir->committed_all = FALSE;
+	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
+	MockDir *dir = osync_objtype_sink_get_userdata(sink);
+	mock_env *env = data;
+
+	if (!dir) {
+		GList *o = env->directories;
+		for (; o; o = o->next) {
+			MockDir *sink_dir = o->data;
+			if (!g_getenv("NO_COMMITTED_ALL_CHECK"))
+				osync_assert(sink_dir->committed_all == TRUE);
+
+			sink_dir->committed_all = FALSE;
+		}
+	} else {
+		if (!g_getenv("NO_COMMITTED_ALL_CHECK"))
+			osync_assert(dir->committed_all == TRUE);
+
+		dir->committed_all = FALSE;
+	}
+
 
 	if (mock_get_error(info->memberid, "DISCONNECT_ERROR")) {
 		osync_context_report_error(ctx, OSYNC_ERROR_EXPECTED, "Triggering DISCONNECT_ERROR error");
@@ -244,12 +128,6 @@ static void mock_disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx
 	if (mock_get_error(info->memberid, "DISCONNECT_TIMEOUT"))
 		return;
 
-	
-	if (dir->hashtable) {
-		osync_hashtable_free(dir->hashtable);
-		dir->hashtable = NULL;
-	}
-	
 	osync_context_report_success(ctx);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
@@ -264,14 +142,13 @@ static osync_bool mock_read(void *data, OSyncPluginInfo *info, OSyncContext *ctx
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, data, info, ctx, change);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+	MockDir *dir = osync_objtype_sink_get_userdata(sink);
 	OSyncError *error = NULL;
-	
+
 	char *filename = g_strdup_printf("%s/%s", dir->path, osync_change_get_uid(change));
 	
 	OSyncFileFormat *file = osync_try_malloc0(sizeof(OSyncFileFormat), &error);
-	if (!file)
-		goto error;
+	osync_assert(file);
 	file->path = g_strdup(osync_change_get_uid(change));
 	
 	struct stat filestats;
@@ -281,12 +158,10 @@ static osync_bool mock_read(void *data, OSyncPluginInfo *info, OSyncContext *ctx
 	file->mode = filestats.st_mode;
 	file->last_mod = filestats.st_mtime;
 			
-	if (!osync_file_read(filename, &(file->data), &(file->size), &error))
-		goto error_free_file;
-	
+	osync_assert(osync_file_read(filename, &(file->data), &(file->size), &error));
+
 	OSyncData *odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), dir->objformat, &error);
-	if (!odata)
-		goto error_free_data;
+	osync_assert(odata);
 
 	osync_trace(TRACE_INTERNAL, "odata: %p", odata);
 	
@@ -300,25 +175,13 @@ static osync_bool mock_read(void *data, OSyncPluginInfo *info, OSyncContext *ctx
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-
-error_free_data:
-	g_free(file->data);
-error_free_file:
-	g_free(file->path);
-	g_free(file);
-error:
-	g_free(filename);
-	osync_context_report_osyncerror(ctx, error);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
-	osync_error_unref(&error);
-	return FALSE;
 }
 
 static osync_bool mock_write(void *data, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, data, info, ctx, change);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+	MockDir *dir = osync_objtype_sink_get_userdata(sink);
 	OSyncError *error = NULL;
 	OSyncData *odata = NULL;
 	char *buffer = NULL;
@@ -328,16 +191,10 @@ static osync_bool mock_write(void *data, OSyncPluginInfo *info, OSyncContext *ct
 			
 	switch (osync_change_get_changetype(change)) {
 		case OSYNC_CHANGE_TYPE_DELETED:
-			if (!remove(filename) == 0) {
-				osync_error_set(&error, OSYNC_ERROR_FILE_NOT_FOUND, "Unable to write");
-				goto error;
-			}
+			osync_assert(remove(filename) == 0);
 			break;
 		case OSYNC_CHANGE_TYPE_ADDED:
-			if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
-				osync_error_set(&error, OSYNC_ERROR_EXISTS, "Entry already exists");
-				goto error;
-			}
+			osync_assert(!g_file_test(filename, G_FILE_TEST_EXISTS));
 			/* No break. Continue below */
 		case OSYNC_CHANGE_TYPE_MODIFIED:
 			//FIXME add ownership for file-sync
@@ -349,10 +206,13 @@ static osync_bool mock_write(void *data, OSyncPluginInfo *info, OSyncContext *ct
 			
 			OSyncFileFormat *file = (OSyncFileFormat *)buffer;
 			
-			if (!osync_file_write(filename, file->data, file->size, file->mode, &error))
-				goto error;
+			osync_assert(osync_file_write(filename, file->data, file->size, file->mode, &error));
 			break;
-		default:
+		case OSYNC_CHANGE_TYPE_UNMODIFIED:
+			fail("Unmodified in a change function?!");
+			break;
+		case OSYNC_CHANGE_TYPE_UNKNOWN:
+			fail("Unknown Change Type");
 			break;
 	}
 	
@@ -360,13 +220,6 @@ static osync_bool mock_write(void *data, OSyncPluginInfo *info, OSyncContext *ct
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-	
-error:
-	g_free(filename);
-	osync_context_report_osyncerror(ctx, error);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
-	osync_error_unref(&error);
-	return FALSE;
 }
 
 /** Report files on a directory
@@ -380,30 +233,27 @@ error:
  *            start with a slash. See note above.
  *
  */
-static void mock_report_dir(OSyncFileDir *directory, const char *subdir, OSyncContext *ctx, OSyncPluginInfo *info)
+static void mock_report_dir(MockDir *directory, const char *subdir, OSyncContext *ctx, OSyncPluginInfo *info)
 {
 	GError *gerror = NULL;
 	const char *de = NULL;
 	char *path = NULL;
 	GDir *dir = NULL;
 	OSyncError *error = NULL;
-	
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, directory, subdir, ctx);
 	
+	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
+	osync_assert(sink);
+
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
+	osync_assert(formatenv);
+
 	path = g_build_filename(directory->path, subdir, NULL);
 	osync_trace(TRACE_INTERNAL, "path %s", path);
 	
 	dir = g_dir_open(path, 0, &gerror);
-	if (!dir) {
-		/*FIXME: Permission errors may make files to be reported as deleted.
-		 * Make fs_report_dir() able to report errors
-		 */
-		osync_error_set(&error, OSYNC_ERROR_GENERIC, "Unable to open directory %s: %s", path, gerror ? gerror->message : "None");
-		osync_context_report_osyncwarning(ctx, error);
-		osync_error_unref(&error);
-		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
-		return;
-	}
+	osync_assert(dir);
 
 	while ((de = g_dir_read_name(dir))) {
 		char *filename = g_build_filename(path, de, NULL);
@@ -413,89 +263,61 @@ static void mock_report_dir(OSyncFileDir *directory, const char *subdir, OSyncCo
 		else
 			relative_filename = g_build_filename(subdir, de, NULL);
 			
-		osync_hashtable_report(directory->hashtable, relative_filename);
 		osync_trace(TRACE_INTERNAL, "path2 %s %s", filename, relative_filename);
 		
-		if (g_file_test(filename, G_FILE_TEST_IS_DIR)) {
-			/* Recurse into subdirectories */
-			if (directory->recursive)
-				mock_report_dir(directory, relative_filename, ctx, info);
-		} else if (g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
+		if (g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
 			
 			struct stat buf;
 			stat(filename, &buf);
 			char *hash = mock_generate_hash(&buf);
 			
-			OSyncChangeType type = osync_hashtable_get_changetype(directory->hashtable, relative_filename, hash);
-			if (type == OSYNC_CHANGE_TYPE_UNMODIFIED) {
-				g_free(hash);
-				g_free(filename);
-				g_free(relative_filename);
-				continue;
-			}
-			osync_hashtable_update_hash(directory->hashtable, type, relative_filename, hash);
-			
 			/* Report normal files */
 			OSyncChange *change = osync_change_new(&error);
-			if (!change) {
-				osync_context_report_osyncwarning(ctx, error);
-				osync_error_unref(&error);
-				g_free(hash);
-				g_free(filename);
-				g_free(relative_filename);
-				continue;
-			}
+			osync_assert(change);
 			
 			osync_change_set_uid(change, relative_filename);
-			osync_change_set_hash(change, hash);
-			osync_change_set_changetype(change, type);
 
+			osync_change_set_hash(change, hash);
 			g_free(hash);
+
+			OSyncChangeType type = osync_hashtable_get_changetype(directory->hashtable, change);
 			
-			OSyncFileFormat *file = osync_try_malloc0(sizeof(OSyncFileFormat), &error);
-			if (!file) {
-				osync_change_unref(change);
-				osync_context_report_osyncwarning(ctx, error);
-				osync_error_unref(&error);
+			osync_change_set_changetype(change, type);
+			osync_hashtable_update_change(directory->hashtable, change);
+
+			if (type == OSYNC_CHANGE_TYPE_UNMODIFIED) {
 				g_free(filename);
 				g_free(relative_filename);
+				osync_change_unref(change);
 				continue;
 			}
+
+			OSyncFileFormat *file = osync_try_malloc0(sizeof(OSyncFileFormat), &error);
+			osync_assert(file);
+
 			file->path = g_strdup(relative_filename);
 			
-			OSyncError *error = NULL;
 			OSyncData *odata = NULL;
 
-			if (mock_get_error(info->memberid, "ONLY_INFO")) {
-				odata = osync_data_new(NULL, 0, directory->objformat, &error);
-			} else {
-				if (!osync_file_read(filename, &(file->data), &(file->size), &error)) {
-					osync_change_unref(change);
-					osync_context_report_osyncwarning(ctx, error);
-					osync_error_unref(&error);
-					g_free(filename);
-					continue;
-				}
+			if (!mock_get_error(info->memberid, "ONLY_INFO")) {
+				osync_assert(osync_file_read(filename, &(file->data), &(file->size), &error));
 
 				if (mock_get_error(info->memberid, "SLOW_REPORT"))
 					sleep(1);
 				
 				odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), directory->objformat, &error);
-				if (!odata) {
-					osync_change_unref(change);
-					osync_context_report_osyncwarning(ctx, error);
-					osync_error_unref(&error);
-					continue;
-				}
+				osync_assert(odata);
+
+
+				osync_change_set_data(change, odata);
 
 			}
 			
-			osync_data_set_objtype(odata, osync_objtype_sink_get_name(directory->sink));
-			osync_change_set_data(change, odata);
+			osync_data_set_objtype(odata, osync_objtype_sink_get_name(sink));
 			osync_data_unref(odata);
 	
 			osync_context_report_change(ctx, change);
-			
+
 			osync_change_unref(change);
 		}
 
@@ -514,11 +336,10 @@ static void mock_get_changes(void *data, OSyncPluginInfo *info, OSyncContext *ct
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
-	int i = 0;
+	MockDir *dir = osync_objtype_sink_get_userdata(sink);
 	OSyncError *error = NULL;
 
-	fail_unless(dir->committed_all == TRUE, NULL);
+	osync_assert(dir->committed_all == TRUE);
 	dir->committed_all = FALSE;
 
 	if (mock_get_error(info->memberid, "GET_CHANGES_ERROR")) {
@@ -535,36 +356,27 @@ static void mock_get_changes(void *data, OSyncPluginInfo *info, OSyncContext *ct
 	if (mock_get_error(info->memberid, "GET_CHANGES_TIMEOUT2"))
 		sleep(8);
 		
-	if (osync_objtype_sink_get_slowsync(dir->sink)) {
+	if (osync_objtype_sink_get_slowsync(sink)) {
 		osync_trace(TRACE_INTERNAL, "Slow sync requested");
-		osync_hashtable_reset(dir->hashtable);
+		osync_assert(osync_hashtable_slowsync(dir->hashtable, &error));
 	}
 	
 	osync_trace(TRACE_INTERNAL, "get_changes for %s", osync_objtype_sink_get_name(sink));
 
 	mock_report_dir(dir, NULL, ctx, info);
 	
-	char **uids = osync_hashtable_get_deleted(dir->hashtable);
-	for (i = 0; uids[i]; i++) {
+	OSyncList *u, *uids = osync_hashtable_get_deleted(dir->hashtable);
+	for (u = uids; u; u = u->next) {
 		OSyncChange *change = osync_change_new(&error);
-		if (!change) {
-			g_free(uids[i]);
-			osync_context_report_osyncwarning(ctx, error);
-			osync_error_unref(&error);
-			continue;
-		}
+		osync_assert(change);
+
+		const char *uid = u->data;
 		
-		osync_change_set_uid(change, uids[i]);
+		osync_change_set_uid(change, uid);
 		osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_DELETED);
 		
 		OSyncData *odata = osync_data_new(NULL, 0, dir->objformat, &error);
-		if (!odata) {
-			g_free(uids[i]);
-			osync_change_unref(change);
-			osync_context_report_osyncwarning(ctx, error);
-			osync_error_unref(&error);
-			continue;
-		}
+		osync_assert(odata);
 		
 		osync_data_set_objtype(odata, osync_objtype_sink_get_name(sink));
 		osync_change_set_data(change, odata);
@@ -572,12 +384,10 @@ static void mock_get_changes(void *data, OSyncPluginInfo *info, OSyncContext *ct
 		
 		osync_context_report_change(ctx, change);
 		
-		osync_hashtable_update_hash(dir->hashtable, osync_change_get_changetype(change), osync_change_get_uid(change), NULL);
+		osync_hashtable_update_change(dir->hashtable, change);
 	
 		osync_change_unref(change);
-		g_free(uids[i]);
 	}
-	g_free(uids);
 	
 	osync_context_report_success(ctx);
 	osync_trace(TRACE_EXIT, "%s", __func__);
@@ -587,11 +397,11 @@ static void mock_commit_change(void *data, OSyncPluginInfo *info, OSyncContext *
 {
 	osync_trace(TRACE_ENTRY, "%s", __func__);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+	MockDir *dir = osync_objtype_sink_get_userdata(sink);
 	
 	char *filename = NULL;
 
-	fail_unless(dir->committed_all == FALSE, NULL);
+	osync_assert(dir->committed_all == FALSE);
 
 	if (mock_get_error(info->memberid, "COMMIT_ERROR")) {
 		osync_context_report_error(ctx, OSYNC_ERROR_EXPECTED, "Triggering COMMIT_ERROR error");
@@ -614,11 +424,13 @@ static void mock_commit_change(void *data, OSyncPluginInfo *info, OSyncContext *
 		struct stat buf;
 		stat(filename, &buf);
 		hash = mock_generate_hash(&buf);
+		osync_change_set_hash(change, hash);
+		g_free(hash);
 	}
 	g_free(filename);
 
-	osync_hashtable_update_hash(dir->hashtable, osync_change_get_changetype(change), osync_change_get_uid(change), hash);
-	g_free(hash);
+
+	osync_hashtable_update_change(dir->hashtable, change);
 	
 	osync_context_report_success(ctx);
 	
@@ -629,9 +441,9 @@ static void mock_batch_commit(void *data, OSyncPluginInfo *info, OSyncContext *c
 {
         osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, context, contexts, changes);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+	MockDir *dir = osync_objtype_sink_get_userdata(sink);
 
-        fail_unless(dir->committed_all == FALSE, NULL);
+        osync_assert(dir->committed_all == FALSE);
         dir->committed_all = TRUE;
 
         int i; 
@@ -644,17 +456,18 @@ static void mock_batch_commit(void *data, OSyncPluginInfo *info, OSyncContext *c
 				struct stat buf;
 				stat(filename, &buf);
 				hash = mock_generate_hash(&buf);
+				osync_change_set_hash(changes[i], hash);
 			}
 			g_free(filename);
 
-			osync_hashtable_update_hash(dir->hashtable, osync_change_get_changetype(changes[i]), osync_change_get_uid(changes[i]), hash);
+			osync_hashtable_update_change(dir->hashtable, changes[i]);
                         osync_context_report_success(contexts[i]);
                 }
         }
 
         if (g_getenv("NUM_BATCH_COMMITS")) {
                 int req = atoi(g_getenv("NUM_BATCH_COMMITS"));
-                fail_unless(req == i, NULL);
+                osync_assert(req == i);
         }
                 
         if (mock_get_error(info->memberid, "COMMITTED_ALL_ERROR")) {
@@ -671,9 +484,9 @@ static void mock_committed_all(void *data, OSyncPluginInfo *info, OSyncContext *
 {
         osync_trace(TRACE_ENTRY, "%s(%p)", __func__, context);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+	MockDir *dir = osync_objtype_sink_get_userdata(sink);
 
-        fail_unless(dir->committed_all == FALSE, NULL);
+        osync_assert(dir->committed_all == FALSE);
         dir->committed_all = TRUE;
 
         if (mock_get_error(info->memberid, "COMMITTED_ALL_ERROR")) {
@@ -691,7 +504,7 @@ static void mock_sync_done(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+	MockDir *dir = osync_objtype_sink_get_userdata(sink);
 
 	if (mock_get_error(info->memberid, "SYNC_DONE_ERROR")) {
 		osync_context_report_error(ctx, OSYNC_ERROR_EXPECTED, "Triggering SYNC_DONE_ERROR error");
@@ -705,6 +518,8 @@ static void mock_sync_done(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_anchor_update(anchorpath, path_field, dir->path);
 	g_free(anchorpath);
 	g_free(path_field);
+
+	osync_assert(osync_hashtable_save(dir->hashtable, NULL));
 	
 	osync_context_report_success(ctx);
 	
@@ -724,36 +539,82 @@ static void *mock_initialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncEr
 	}
 
 	mock_env *env = osync_try_malloc0(sizeof(mock_env), error);
-	if (!env)
-		goto error;
-	
-	osync_trace(TRACE_INTERNAL, "The config: %s", osync_plugin_info_get_config(info));
-	
+	osync_assert(env);
+
 	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
+	osync_assert(formatenv);
 	
-	if (!mock_parse_settings(env, info, error))
-		goto error_free_env;
+	OSyncPluginConfig *config = osync_plugin_info_get_config(info);
+	osync_assert(config);
+	
+	if (mock_get_error(info->memberid, "MAINSINK_CONNECT")) {
+		env->mainsink = osync_objtype_main_sink_new(error);
+
+		OSyncObjTypeSinkFunctions functions;
+		memset(&functions, 0, sizeof(functions));
+
+		functions.connect = mock_connect;
+		functions.disconnect = mock_disconnect;
+
+		osync_objtype_sink_set_functions(env->mainsink, functions, NULL);
+		osync_plugin_info_set_main_sink(info, env->mainsink);
+	}
 	
 	/* Now we register the objtypes that we can sync. This plugin is special. It can
 	 * synchronize any objtype we configure it to sync and where a conversion
 	 * path to the file format can be found */
-	GList *o = env->directories;
-	for (; o; o = o->next) {
-		OSyncFileDir *dir = o->data;
-		/* We register the given objtype here */
-		OSyncObjTypeSink *sink = osync_objtype_sink_new(dir->objtype, error);
-		if (!sink)
-			goto error_free_env;
+	int i, numobjs = osync_plugin_info_num_objtypes(info);
+	for (i = 0; i < numobjs; i++) {
+		MockDir *dir = osync_try_malloc0(sizeof(MockDir), error);
+		osync_assert(dir);
+
+		dir->committed_all = TRUE;
+
+		OSyncObjTypeSink *sink = osync_plugin_info_nth_objtype(info, i);
+		osync_assert(sink);
+
+		const char *objtype = osync_objtype_sink_get_name(sink);
+		dir->res = osync_plugin_config_find_active_resource(config, objtype);
+		dir->path = osync_plugin_resource_get_path(dir->res);
+		osync_assert(dir->path);
+
+		OSyncList *format_sinks = osync_plugin_resource_get_objformat_sinks(dir->res);
+		osync_assert(osync_list_length(format_sinks) == 1);
+		OSyncObjFormatSink *format_sink = osync_list_nth_data(format_sinks, 0);
+		const char *objformat_str = osync_objformat_sink_get_objformat(format_sink);
+		osync_assert(objformat_str);
+		dir->objformat = osync_format_env_find_objformat(formatenv, objformat_str);
+		osync_assert(dir->objformat);
+		osync_objformat_ref(dir->objformat);
+
+		osync_trace(TRACE_INTERNAL, "The configdir: %s", osync_plugin_info_get_configdir(info));
+		char *tablepath = g_strdup_printf("%s/hashtable.db", osync_plugin_info_get_configdir(info));
+		dir->hashtable = osync_hashtable_new(tablepath, objtype, error);
+		g_free(tablepath);
 		
-		dir->sink = sink;
-		
-		osync_objtype_sink_add_objformat(sink, osync_objformat_get_name(dir->objformat));
+		osync_assert(dir->hashtable);
+
+		osync_assert(osync_hashtable_load(dir->hashtable, error));
+
+
+		/*
+		const char *objformat = osync_objformat_get_name(dir->objformat); 
+		OSyncObjFormatSink *format_sink = osync_objformat_sink_new(objformat, error);
+		if (!format_sink)
+			return NULL;
+
+		osync_objtype_sink_add_objformat_sink(sink, format_sink);
+		*/
 		
 		/* All sinks have the same functions of course */
 		OSyncObjTypeSinkFunctions functions;
 		memset(&functions, 0, sizeof(functions));
-		functions.connect = mock_connect;
-		functions.disconnect = mock_disconnect;
+
+		if (!mock_get_error(info->memberid, "MAINSINK_CONNECT")) {
+			functions.connect = mock_connect;
+			functions.disconnect = mock_disconnect;
+		}
+
 		functions.get_changes = mock_get_changes;
 
 		//Rewrite the batch commit functions so we can enable them if necessary
@@ -768,10 +629,9 @@ static void *mock_initialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncEr
 		functions.write = mock_write;
 		functions.sync_done = mock_sync_done;
 		
-		/* We pass the OSyncFileDir object to the sink, so we dont have to look it up
+		/* We pass the MockDir object to the sink, so we dont have to look it up
 		 * again once the functions are called */
 		osync_objtype_sink_set_functions(sink, functions, dir);
-		osync_plugin_info_add_objtype(info, sink);
 
 		//Lets reduce the timeouts a bit so the checks work faster
 		osync_objtype_sink_set_connect_timeout(sink, 5);
@@ -816,24 +676,33 @@ static void *mock_initialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncEr
 			info->functions.is_available = mock_is_available;
 		*/
 
-#endif	
+#endif
+		env->directories = g_list_append(env->directories, dir);
 	}
 
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, env);
 	return (void *)env;
-
-error_free_env:
-	free_env(env);
-error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return NULL;
 }
 
 static void mock_finalize(void *data)
 {
 	mock_env *env = data;
 
-	free_env(env);
+	while (env->directories) {
+		MockDir *dir = env->directories->data;
+
+		osync_plugin_resource_unref(dir->res);
+		osync_objformat_unref(dir->objformat);
+		osync_hashtable_unref(dir->hashtable);
+
+		env->directories = g_list_remove(env->directories, dir);
+	}
+
+	if (env->mainsink)
+		osync_objtype_sink_unref(env->mainsink);
+
+
+	g_free(env);
 }
 
 /* Here we actually tell opensync which sinks are available. For this plugin, we
@@ -842,12 +711,25 @@ static osync_bool mock_discover(void *data, OSyncPluginInfo *info, OSyncError **
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, error);
 	
-	mock_env *env = (mock_env *)data;
-	GList *o = env->directories;
-	for (; o; o = o->next) {
-		OSyncFileDir *dir = o->data;
-		osync_objtype_sink_set_available(dir->sink, TRUE);
-	}
+	OSyncPluginConfig *config = osync_plugin_info_get_config(info);
+	osync_assert_msg(config, "No OSyncPluginConfig set for mock_discover!");
+
+	/*
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
+	osync_assert_msg(config, "No OSyncFormatEnv set for mock_discover!");
+	*/
+
+	OSyncList *r = osync_plugin_config_get_resources(config);
+	for (; r; r = r->next) {
+		OSyncPluginResource *res = r->data;
+		OSyncObjTypeSink *sink;
+
+		const char *objtype = osync_plugin_resource_get_objtype(res);
+		/* Check for ObjType sink */
+		if ((sink = osync_plugin_info_find_objtype(info, objtype)))
+			osync_objtype_sink_set_available(sink, TRUE);
+ 	}
+
 	
 	OSyncVersion *version = osync_version_new(error);
 	osync_version_set_plugin(version, "mock-sync");
@@ -869,11 +751,10 @@ static osync_bool mock_discover(void *data, OSyncPluginInfo *info, OSyncError **
 osync_bool get_sync_info(OSyncPluginEnv *env, OSyncError **error)
 {
 	OSyncPlugin *plugin = osync_plugin_new(error);
-	if (!plugin)
-		goto error;
+	osync_assert(plugin);
 	
 	osync_plugin_set_name(plugin, "mock-sync");
-	osync_plugin_set_longname(plugin, "File Synchronization Plugin");
+	osync_plugin_set_longname(plugin, "Mock Synchronization Plugin");
 	osync_plugin_set_description(plugin, "Plugin to synchronize files on the local filesystem");
 	
 	osync_plugin_set_initialize(plugin, mock_initialize);
@@ -884,8 +765,7 @@ osync_bool get_sync_info(OSyncPluginEnv *env, OSyncError **error)
 	osync_plugin_unref(plugin);
 
 	plugin = osync_plugin_new(error);
-	if (!plugin)
-		goto error;
+	osync_assert(plugin);
 	
 	osync_plugin_set_name(plugin, "mock-sync-external");
 	osync_plugin_set_longname(plugin, "Mock Synchronization Plugin with Start Type External");
@@ -901,11 +781,6 @@ osync_bool get_sync_info(OSyncPluginEnv *env, OSyncError **error)
 
 	
 	return TRUE;
-	
-error:
-	osync_trace(TRACE_ERROR, "Unable to register: %s", osync_error_print(error));
-	osync_error_unref(error);
-	return FALSE;
 }
 
 int get_version(void)
