@@ -6,6 +6,7 @@ static void reset_env(void)
 {
 	unsetenv("CONNECT_ERROR");
 	unsetenv("CONNECT_TIMEOUT");
+	unsetenv("CONNECT_SLOWSYNC");
 	unsetenv("INIT_NULL");
 	unsetenv("GET_CHANGES_ERROR");
 	unsetenv("GET_CHANGES_TIMEOUT");
@@ -19,9 +20,24 @@ static void reset_env(void)
 	unsetenv("BATCH_COMMIT");
 	unsetenv("COMMITTED_ALL_ERROR");
 	unsetenv("NO_COMMITTED_ALL_CHECK");
+	unsetenv("MAINSINK_CONNECT");
+
+	unsetenv("OSYNC_NOMEMORY");
 }
 
-char *setup_testbed(char *fkt_name)
+
+void check_env(void) {
+
+	if (g_getenv("OSYNC_TRACE"))
+		fprintf(stderr, "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"
+				"WARNING! Environment variable OSYNC_TRACE is set.\n"
+				"This unit contains stress & performance tests which\n"
+				"will fail/timeout because of bad performance impact of\n"
+				"the i/o which will be produced by OSYNC_TRACE!\n"
+				"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n");
+}
+
+char *setup_testbed(const char *fkt_name)
 {
 	
 	setuid(65534);
@@ -48,17 +64,27 @@ char *setup_testbed(char *fkt_name)
 		abort();
 	g_free(command);*/
 	
-	command = g_strdup_printf("cp ./mock-plugin/mock-sync.so %s", testbed);
+	command = g_strdup_printf("mkdir %s/formats",  testbed);
 	if (system(command))
 		abort();
 	g_free(command);
 	
-	command = g_strdup_printf("cp ./mock-plugin/mock-format.so %s", testbed);
+	command = g_strdup_printf("mkdir %s/plugins",  testbed);
 	if (system(command))
 		abort();
 	g_free(command);
 	
-	command = g_strdup_printf("cp -R ../formats/*.so %s", testbed);
+	command = g_strdup_printf("cp ./mock-plugin/mock-sync.%s %s/plugins", G_MODULE_SUFFIX, testbed);
+	if (system(command))
+		abort();
+	g_free(command);
+	
+	command = g_strdup_printf("cp ./mock-plugin/mock-format.%s %s/formats", G_MODULE_SUFFIX, testbed);
+	if (system(command))
+		abort();
+	g_free(command);
+	
+	command = g_strdup_printf("cp -R ../formats/*.%s %s/formats", G_MODULE_SUFFIX, testbed);
 	if (system(command))
 		abort();
 	g_free(command);
@@ -77,6 +103,8 @@ char *setup_testbed(char *fkt_name)
 	if (chdir(testbed))
 		abort();
 	
+	reset_counters();
+
 	osync_trace(TRACE_INTERNAL, "Seting up %s at %s", fkt_name, testbed);
 /*	printf(".");
 	fflush(NULL);*/
@@ -205,26 +233,17 @@ OSyncHashTable *hashtable_load(const char *path, const char *objtype, int entrie
 	OSyncHashTable *table = osync_hashtable_new(path, objtype, &error);
 	fail_unless(table != NULL, NULL);
 	fail_unless(error == NULL, NULL);
+
+	fail_unless(osync_hashtable_load(table, &error), NULL);
 	
-    fail_unless(osync_hashtable_num_entries(table) == entries, NULL);
+	fail_unless(osync_hashtable_num_entries(table) == entries, NULL);
     
     return table;
 }
 
 void check_hash(OSyncHashTable *table, const char *cmpuid)
 {
-	char *uid = NULL;
-	char *hash = NULL;
-	int i;
-	osync_bool found = FALSE;
-	for (i = 0; i < osync_hashtable_num_entries(table); i++) {
-		osync_hashtable_nth_entry(table, i, &uid, &hash);
-		if (!strcmp(cmpuid, uid))
-			found = TRUE;
-		g_free(hash);
-		g_free(uid);
-	}
-	fail_unless(found == TRUE, NULL);
+	fail_unless(!!osync_hashtable_get_hash(table, cmpuid), "Couldn't find hash!");
 }
 
 
@@ -453,7 +472,7 @@ void conflict_handler_choose_modified(OSyncEngine *engine, OSyncMappingEngine *m
 			return;
 		}
 	}
-	fail();
+	fail("");
 }
 
 void conflict_handler_ignore(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
@@ -482,6 +501,19 @@ void conflict_handler_duplicate(OSyncEngine *engine, OSyncMappingEngine *mapping
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
+void conflict_handler_abort(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
+	
+	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
+	fail_unless(num_engine_end_conflicts == 0, NULL);
+	
+	OSyncError *error = NULL;
+	fail_unless(osync_engine_abort(engine, &error), NULL);
+	
+	osync_trace(TRACE_EXIT, "%s", __func__);
+}
+
 OSyncEngine *gengine = NULL;
 
 void solve_conflict(OSyncMappingEngine *mapping)
@@ -560,3 +592,161 @@ void discover_all_once(OSyncEngine *engine, OSyncError **error)
 	}
 }
 
+/*! @brief Check if file or directory exists. No check for regular file! 
+ * 
+ * @param file filename or fullpath of file/directory 
+ * @returns TRUE if exists, FALSE otherwise
+ * 
+ */
+osync_bool osync_testing_file_exists(const char *file)
+{
+	return g_file_test(file, G_FILE_TEST_EXISTS);
+}
+
+/*! @brief Removes files and directories 
+ * 
+ * @param file filename or fullpath of file/directory 
+ * @returns TRUE on success, FALSE otherwise
+ * 
+ */
+osync_bool osync_testing_file_remove(const char *file)
+{
+	return g_remove(file);
+}
+
+/*! @brief Modifies permission of file - like chmod() 
+ * 
+ * @param file filename or fullpath of file 
+ * @param mode the permission mode like chmod()
+ * @returns TRUE on success, FALSE otherwise
+ * 
+ */
+osync_bool osync_testing_file_chmod(const char *file, int mode)
+{
+	/* GLib 2.16.3 Note: 
+	   "[...] Software that needs to manage file 
+	   permissions on Windows exactly should use the Win32 API."
+	TODO: Do we have to care about this on Windows?! */
+	return g_chmod(file, mode);
+}
+
+/*! @brief Copy files
+ *
+ * @param source source filename
+ * @param dest destination filename
+ * @returns TRUE on success, FALSE otherwise
+ *
+ */
+osync_bool osync_testing_file_copy(const char *source, const char *dest)
+{
+        gboolean ret;
+        const char *argv[] = { "cp", source, dest, NULL };
+        int exitstatus = -1;
+
+        ret = g_spawn_sync(NULL,	        /* working directory */
+                           (char **)argv,	/* arguments */
+                           NULL,	        /* environment */
+                           G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
+                           NULL,	        /* child setup function */
+                           NULL,	        /* user data for child setup func */
+                           NULL,	        /* stdin */
+                           NULL,	        /* stdout */
+                           &exitstatus,	        /* exit status */
+                           NULL		        /* error function */
+                        );
+
+        return ret && WEXITSTATUS(exitstatus) == 0;
+}
+
+/*! @brief Find differences between two files
+ *
+ * @param source source filename
+ * @param dest destination filename
+ * @returns TRUE on success, FALSE otherwise
+ *
+ */
+osync_bool osync_testing_diff(const char *file1, const char *file2)
+{
+        gchar *cmd;
+        int ret;
+
+	osync_assert(file1);
+	osync_assert(file2);
+
+        cmd = g_strdup_printf(DIFF " -x \".*\" %s %s", file1, file2);
+        ret = system(cmd);
+        g_free(cmd);
+
+        return ret;
+}
+
+/*! @brief Creates a simple OSyncPluginConfig with a single resource.
+ *         If config is not null the resource information gets added.
+ * 
+ * @param config OSyncPluginConfig pointer to add resource info
+ * @param path relative path of resource
+ * @param objformat the objformat of the resource
+ * @param format_config the format converter config paramter
+ * @returns OSyncPluginConfig pointer or asserts on error
+ * 
+ */
+OSyncPluginConfig *simple_plugin_config(OSyncPluginConfig *config, const char *path, const char *objtype, const char *objformat, const char *format_config) {
+
+	osync_assert(objtype);
+	osync_assert(objformat);
+
+	OSyncError *error = NULL;
+
+	if (!config)
+		config = osync_plugin_config_new(&error);
+
+	fail_unless(config != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+
+	OSyncObjFormatSink *format_sink = osync_objformat_sink_new(objformat, &error);
+	if (format_config)
+		osync_objformat_sink_set_config(format_sink, format_config);
+
+	OSyncPluginResource *res = osync_plugin_resource_new(&error);
+	osync_plugin_resource_set_objtype(res, objtype);
+	osync_plugin_resource_set_path(res, path); 
+	osync_plugin_resource_enable(res, TRUE);
+	osync_plugin_resource_add_objformat_sink(res, format_sink);
+
+
+	osync_plugin_config_add_resource(config, res);
+
+	return config;
+}
+
+/*! @brief Creates a simple OSyncEngine with nth Members with mock-sync
+ *         plugins.
+ * 
+ * @param member_size The number of member the Group for this Engine should have
+ * @returns OSyncEngine pointer or asserts on error
+ * 
+ */
+OSyncEngine *osync_testing_create_engine_dummy(unsigned int member_size)
+{
+	unsigned int u;
+	OSyncError *error = NULL;
+
+	OSyncGroup *group = osync_group_new(&error);
+	fail_unless(group != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+
+	for (u=0; u < member_size; u++) {
+		OSyncMember *member = osync_member_new(&error);
+		fail_unless(member != NULL, NULL);
+		fail_unless(error == NULL, NULL);
+
+		osync_group_add_member(group, member);
+	}
+
+	OSyncEngine *engine = osync_engine_new(group, &error);
+	fail_unless(engine != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+
+	return engine;
+}
+
